
struct xmlDoc;
struct xmlNode;
//struct xmlNs;
//struct xmlDtd;
//struct xmlAttr;
struct xmlDict;

// The different element types carried by an XML tree.
// NOTE: This is synchronized with DOM Level1 values
// See http://www.w3.org/TR/REC-DOM-Level-1/
// Actually this had diverged a bit, and now XML_DOCUMENT_TYPE_NODE should be deprecated to use an XML_DTD_NODE.
/*
enum xmlElementType
{
	XML_ELEMENT_NODE=		1,
	XML_ATTRIBUTE_NODE=		2,
	XML_TEXT_NODE=			3,
	XML_CDATA_SECTION_NODE=	4,
	XML_ENTITY_REF_NODE=	5,
	XML_ENTITY_NODE=		6,
	XML_PI_NODE=			7,
	XML_COMMENT_NODE=		8,
	XML_DOCUMENT_NODE=		9,
	XML_DOCUMENT_TYPE_NODE=	10,
	XML_DOCUMENT_FRAG_NODE=	11,
	XML_NOTATION_NODE=		12,
	XML_HTML_DOCUMENT_NODE=	13,
	XML_DTD_NODE=			14,
	XML_ELEMENT_DECL=		15,
	XML_ATTRIBUTE_DECL=		16,
	XML_ENTITY_DECL=		17,
	XML_NAMESPACE_DECL=		18,
	XML_XINCLUDE_START=		19,
	XML_XINCLUDE_END=		20,
	XML_DOCB_DOCUMENT_NODE=	21
};
*/
#define xmlElementType int

// A DTD Attribute type definition.
/*
enum xmlAttributeType
{
	XML_ATTRIBUTE_CDATA = 1,
	XML_ATTRIBUTE_ID,
	XML_ATTRIBUTE_IDREF,
	XML_ATTRIBUTE_IDREFS,
	XML_ATTRIBUTE_ENTITY,
	XML_ATTRIBUTE_ENTITIES,
	XML_ATTRIBUTE_NMTOKEN,
	XML_ATTRIBUTE_NMTOKENS,
	XML_ATTRIBUTE_ENUMERATION,
	XML_ATTRIBUTE_NOTATION
};
*/
#define xmlAttributeType int

// This is a basic byte in an UTF-8 encoded string.
// It's unsigned allowing to pinpoint case where char * are assigned
// to xmlChar* (possibly making serialization back impossible).
typedef unsigned char xmlChar;

// An XML namespace.
// Note that prefix == NULL is valid, it defines the default namespace within the subtree (until overridden).
// xmlNsType is unified with xmlElementType.
struct xmlNs
{
    xmlNs* next;			// next Ns link for this node
    xmlElementType type;	// global or local
    xmlChar* href;			// URL for the namespace
    xmlChar* prefix;		// prefix for the namespace
    void* _private;			// application data
    xmlDoc* context;		// normally an xmlDoc
};

// An XML DTD, as defined by <!DOCTYPE ...
// There is actually one for the internal subset and for the external subset.
struct xmlDtd
{
	void* _private;			// application data
	xmlElementType type;	// XML_DTD_NODE, must be second !
	xmlChar* name;			// Name of the DTD
	xmlNode* children;		// the value of the property link
	xmlNode* last;			// last child link
	xmlDoc* parent;			// child->parent link
	xmlNode* next;			// next sibling link
	xmlNode* prev;			// previous sibling link
	xmlDoc* doc;			// the containing document
	// End of common part
	void* notations;		// Hash table for notations if any
	void* elements;			// Hash table for elements if any
	void* attributes;		// Hash table for attributes if any
	void* entities;			// Hash table for entities if any
	xmlChar* ExternalID;	// External identifier for PUBLIC DTD
	xmlChar* SystemID;		// URI for a SYSTEM or PUBLIC DTD
	void* pentities;		// Hash table for param entities if any
};

// An attribute on an XML node
struct xmlAttr
{
    void* _private;			// application data
    xmlElementType type;	// XML_ATTRIBUTE_NODE, must be second !
    xmlChar* name;			// the name of the property
    xmlNode* children;		// the value of the property
    xmlNode* last;			// NULL
    xmlNode* parent;		// child->parent link
    xmlAttr* next;			// next sibling link 
    xmlAttr* prev;			// previous sibling link 
    xmlDoc* doc;			// the containing document
    xmlNs* ns;				// pointer to the associated namespace
    xmlAttributeType atype;	// the attribute type if validating
    void* psvi;				// for type/PSVI informations
};

// A node in an XML tree
struct xmlNode
{
	void* _private;			// application data
	xmlElementType type;	// type number, must be second !
	xmlChar* name;			// the name of the node, or the entity
	xmlNode* children;		// parent->childs link
	xmlNode* last;			// last child link
	xmlNode* parent;		// child->parent link
	xmlNode* next;			// next sibling link 
	xmlNode* prev;			// previous sibling link 
	xmlDoc* doc;			// the containing document
	// End of common part
	xmlNs* ns;				// pointer to the associated namespace
	xmlChar* content;		// the content
	xmlAttr* properties;	// properties list
	xmlNs* nsDef;			// namespace definitions on this node
	void* psvi;				// for type/PSVI informations
	unsigned short line;	// line number
	unsigned short extra;	// extra data for XPath/XSLT
};

// An XML document
struct xmlDoc
{
	void* _private;			// application data
	xmlElementType type;	// XML_DOCUMENT_NODE, must be second !
	char* name;				// name/filename/URI of the document
	xmlNode* children;		// the document tree
	xmlNode* last;			// last child link
	xmlNode* parent;		// child->parent link
	xmlNode* next;			// next sibling link 
	xmlNode* prev;			// previous sibling link 
	xmlDoc* doc;			// autoreference to itself
	// End of common part
	int compression;		// level of zlib compression
	int standalone;			// standalone document (no external refs): 1 if standalone="yes", 0 if standalone="no", -1 if there is no XML declaration, -2 if there is an XML declaration, but no standalone attribute was specified
	xmlDtd* intSubset;		// the document internal subset
	xmlDtd* extSubset;		// the document external subset
	xmlNs* oldNs;			// Global namespace, the old way
	xmlChar* version;		// the XML version string
	xmlChar* encoding;		// external initial encoding, if any
	void* ids;				// Hash table for ID attributes if any
	void* refs;				// Hash table for IDREFs attributes if any
	xmlChar* URL;			// The URI for that document
	int charset;			// Internal flag for charset handling, actually an xmlCharEncoding
	xmlDict* dict;			// dict used to allocate names or NULL
	void* psvi;				// for type/PSVI informations
	int parseFlags;			// set of xmlParserOption used to parse the document
	int properties;			// set of xmlDocProperties for this document set at the end of parsing
};



// 0082FEDA int __cdecl xmlStrcmp(const xmlChar* str1, const xmlChar* str2)
// 0082FEE0 xmlNode* __cdecl xmlDocGetRootElement(const xmlDoc* doc)
// 0082FEE6 xmlDoc* __cdecl xmlParseFile(const char* filename)
// 0082FEEC int __cdecl xmlSaveFormatFileEnc(const char* filename, xmlDoc* cur, const char* encoding, int format)
// 0082FEF2 int* __cdecl xmlIndentTreeOutput(void)
// 0082FEF8 xmlNode* __cdecl xmlDocSetRootElement(xmlDoc* doc, xmlNode* root)
// 0082FEFE xmlNode* __cdecl xmlNewNode(xmlNs* ns, const xmlChar* name)
// 0082FF04 xmlDoc* __cdecl xmlNewDoc(const xmlChar* version) 
// 0082FF0A xmlChar* __cdecl xmlGetProp(const xmlNode* node, const xmlChar* name)
// 0082FF10 void __cdecl xmlFreeDoc(xmlDoc* cur)
// 0082FF16 xmlAttr* __cdecl xmlSetProp(xmlNode* node, const xmlChar* name, const xmlChar* value) 
// 0082FF1C xmlNode* __cdecl xmlNewChild(xmlNode* parent, xmlNs* ns, const xmlChar* name, const xmlChar* content)
