
#include "mod_libraries.h"
#include "../Service/error.h"
#include "../defs.h"
////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------
static char* GameLibStr[] =
{
	"pureHD.exe",
	"gameserver.exe",
	"granny.dll",

};
char* GetGameLibStr(int lib)
{
	return GameLibStr[lib];
}

//----------------------------------------------------------------------
DWORD g_dwSacred = 0;	DWORD g_dwSacred_Base = 0;
DWORD g_dwGameServ = 0;	DWORD g_dwGameServ_Base = 0;
DWORD g_dwGranny = 0;	DWORD g_dwGranny_Base = 0;

//----------------------------------------------------------------------
DWORD _Lib_LoadLibrary(LPCSTR pszLibraryName, DWORD* pdwLibraryBase)
{
	DWORD dwOffset = (DWORD)::LoadLibrary(pszLibraryName);
	if(!dwOffset)
	{
		exit(-1);
	}

	DWORD dwOffsOptionalHeader = ((IMAGE_DOS_HEADER*)dwOffset)->e_lfanew + sizeof(DWORD) + IMAGE_SIZEOF_FILE_HEADER;
	IMAGE_OPTIONAL_HEADER* pImageOptionalHeader = (IMAGE_OPTIONAL_HEADER*)(dwOffset + dwOffsOptionalHeader);
	*pdwLibraryBase = pImageOptionalHeader->ImageBase;
	return dwOffset;
}

//----------------------------------------------------------------------
void _Lib_FreeLibrary(LPCSTR pszLibraryName, DWORD dwOffset)
{
	if(dwOffset)
	{
		::FreeLibrary((HINSTANCE)dwOffset);
	}
}

//----------------------------------------------------------------------
// Change the protection scheme of a loaded
// DLL called libraryName in memory space at address addr+size to allow us to customize it.
//----------------------------------------------------------------------
void _Lib_HookLibrary(LPCSTR pszLibraryName, DWORD dwAddr, BOOL isLog)
{
	DWORD dw = 0;
	DWORD dwOffsOptionalHeader = ((IMAGE_DOS_HEADER*)dwAddr)->e_lfanew + sizeof(DWORD) + IMAGE_SIZEOF_FILE_HEADER;
	IMAGE_OPTIONAL_HEADER* pImageOptionalHeader = (IMAGE_OPTIONAL_HEADER*)(dwAddr + dwOffsOptionalHeader);
	DWORD dwSizeOfCode = pImageOptionalHeader->SizeOfCode;
	DWORD dwSectionAlignment = pImageOptionalHeader->SectionAlignment;
	BYTE* pBaseOfCode = (BYTE*)(dwAddr + dwSectionAlignment);

	if(!::VirtualProtect(pBaseOfCode, dwSizeOfCode, PAGE_EXECUTE_READWRITE, &dw))
	{
		exit(-1);
	}
}

//----------------------------------------------------------------------
void _Lib_UnhookLibrary(LPCSTR pszLibraryName, DWORD dwAddr, BOOL isLog)
{
	DWORD dw = 0;
	DWORD dwOffsOptionalHeader = ((IMAGE_DOS_HEADER*)dwAddr)->e_lfanew + sizeof(DWORD) + IMAGE_SIZEOF_FILE_HEADER;
	IMAGE_OPTIONAL_HEADER* pImageOptionalHeader = (IMAGE_OPTIONAL_HEADER*)(dwAddr + dwOffsOptionalHeader);
	DWORD dwSizeOfCode = pImageOptionalHeader->SizeOfCode;
	DWORD dwSectionAlignment = pImageOptionalHeader->SectionAlignment;
	BYTE* pBaseOfCode = (BYTE*)(dwAddr + dwSectionAlignment);
	::VirtualProtect(pBaseOfCode, dwSizeOfCode, PAGE_EXECUTE_READ, &dw);
}

//----------------------------------------------------------------------
void _Lib_LoadGameLibraries()
{
	g_dwSacred = _Lib_LoadLibrary(_LibStr(EXE), &g_dwSacred_Base);
	g_dwGranny = _Lib_LoadLibrary(_LibStr(GRN), &g_dwGranny_Base);
}

//----------------------------------------------------------------------
void _Lib_FreeGameLibraries()
{
	_Lib_FreeLibrary(_LibStr(EXE), g_dwSacred);
	_Lib_FreeLibrary(_LibStr(GRN), g_dwGranny);
}

//----------------------------------------------------------------------
void _Lib_HookGameLibraries(BOOL isLog)
{
	_Lib_HookLibrary(_LibStr(EXE), g_dwSacred, isLog);
	_Lib_HookLibrary(_LibStr(GRN), g_dwGranny, isLog);
}

//----------------------------------------------------------------------
void _Lib_UnhookGameLibraries(BOOL isLog)
{
	_Lib_UnhookLibrary(_LibStr(EXE), g_dwSacred, isLog);
	_Lib_UnhookLibrary(_LibStr(GRN), g_dwGranny, isLog);
}