
#pragma pack(1)
////////////////////////////////////////////////////////////////////////////


#define MAX_SPEAKERS             9        // Up to 9 hardware output channels supported on Win32
#define MAX_RECEIVER_SPECS       32       // Up to 32 receiver point specifications
#define EXTRA_BUILD_BUFFERS 1


typedef int HTIMER;                             // Handle to timer
typedef LPVOID AILLPDIRECTSOUND;
typedef LPVOID AILLPDIRECTSOUNDBUFFER;
typedef struct SAMPLE *HSAMPLE;           // Handle to sample
typedef struct DIG_DRIVER *HDIGDRIVER;    // Handle to digital driver
typedef struct HWAVEOUT__ *HWAVEOUT;
typedef UINT HPROVIDER;
typedef int HASISTREAM;
typedef int ASIRESULT;
typedef UINT HPROPERTY;
typedef int HSAMPLESTATE;
typedef int HDRIVERSTATE;
typedef struct STREAM *HSTREAM;           // Handle to stream
typedef UINT HPROENUM;





struct MWAVEHDR
{
	char *       lpData;                 
	UINT       dwBufferLength;         
	UINT       dwBytesRecorded;        
	UINT   dwUser;                 
	UINT       dwFlags;                
	UINT       dwLoops;                
	MWAVEHDR  *lpNext;     
	UINT   reserved;               
};
struct MWAVEFORMAT
{
	WORD    wFormatTag;        
	WORD    nChannels;         
	UINT   nSamplesPerSec;    
	UINT   nAvgBytesPerSec;   
	WORD    nBlockAlign;       
};
struct MPCMWAVEFORMAT
{
	MWAVEFORMAT  wf;
	WORD        wBitsPerSample;
};


typedef enum 
{
	MSS_MC_INVALID             = 0,       // Used for configuration-function errors
	MSS_MC_MONO                = 1,       // For compatibility with int channel param
	MSS_MC_STEREO              = 2,
	MSS_MC_USE_SYSTEM_CONFIG   = 0x10,    // Leave space between entries for new variations
	MSS_MC_HEADPHONES          = 0x20,    // with similar quality levels/speaker counts
	MSS_MC_DOLBY_SURROUND      = 0x30,    
	MSS_MC_SRS_CIRCLE_SURROUND = 0x40,
	MSS_MC_40_DTS              = 0x48,
	MSS_MC_40_DISCRETE         = 0x50,
	MSS_MC_51_DTS              = 0x58,
	MSS_MC_51_DISCRETE         = 0x60,
	MSS_MC_61_DISCRETE         = 0x70,
	MSS_MC_71_DISCRETE         = 0x80,
	MSS_MC_81_DISCRETE         = 0x90,
	MSS_MC_DIRECTSOUND3D       = 0xA0,
	MSS_MC_EAX2                = 0xC0,
	MSS_MC_EAX3                = 0xD0,
	MSS_MC_EAX4                = 0xE0
}
MSS_MC_SPEC;

typedef enum
{
	DP_FLUSH = 0,      // Must be "MSS mixer" provider
	DP_DEFAULT_FILTER, // Must be "MSS pipeline filter" provider (sets the default)
	DP_DEFAULT_MERGE,  // Must be "MSS mixer" provider (sets the default)
	DP_COPY,           // Must be "MSS mixer" provider
	DP_MC_COPY,        // Must be "MSS mixer" provider
	DP_ADPCM_DECODE,   // Must be "MSS mixer" provider
	N_DIGDRV_STAGES,   // Placeholder for end of list (= # of valid stages)
	DIGDRV_ALL_STAGES  // Used to signify all pipeline stages, for shutdown
}
DIGDRVSTAGE;

typedef enum
{
	SP_ASI_DECODER = 0,          // Must be "ASI codec stream" provider
	SP_FILTER,                   // Must be "MSS pipeline filter" provider
	SP_FILTER_0 = SP_FILTER,     // Must be "MSS pipeline filter" provider
	SP_FILTER_1,                 // Must be "MSS pipeline filter" provider
	SP_FILTER_2,                 // Must be "MSS pipeline filter" provider
	SP_FILTER_3,                 // Must be "MSS pipeline filter" provider
	SP_FILTER_4,                 // Must be "MSS pipeline filter" provider
	SP_FILTER_5,                 // Must be "MSS pipeline filter" provider
	SP_FILTER_6,                 // Must be "MSS pipeline filter" provider
	SP_FILTER_7,                 // Must be "MSS pipeline filter" provider
	SP_MERGE,                    // Must be "MSS mixer" provider
	N_SAMPLE_STAGES,             // Placeholder for end of list (= # of valid sample pipeline stages)
	SP_OUTPUT = N_SAMPLE_STAGES, // Used to set/get prefs/attribs on a driver's output or matrix filter (if present) 
	SAMPLE_ALL_STAGES            // Used to signify all pipeline stages, for shutdown
}
SAMPLESTAGE;

struct MSSVECTOR3D
{
	float x;
	float y;
	float z;
};
struct MSS_RECEIVER_LIST
{
	MSSVECTOR3D direction;                      // Normalized direction vector from listener
	int         speaker_index[MAX_SPEAKERS];    // List of speakers affected by sounds in this direction
	float         speaker_level[MAX_SPEAKERS];    // Each speaker's degree of effect from this source
	int         n_speakers_affected; 
};

struct D3DSTATE
{
	int         mute_at_max;
	MSSVECTOR3D listen_position;
	MSSVECTOR3D listen_face;
	MSSVECTOR3D listen_up;
	MSSVECTOR3D listen_cross;
	MSSVECTOR3D listen_velocity;
	float         rolloff_factor;
	float         doppler_factor;
	float         distance_factor;
	float         falloff_power;
	// Precalculated listener info
	int         ambient_channels    [MAX_SPEAKERS];      // E.g., LFE
	int         n_ambient_channels;
	int         directional_channels[MAX_SPEAKERS+1];    // Channel index, or -1 if virtual
	MSSVECTOR3D listener_to_speaker [MAX_SPEAKERS+1];  
	int         n_directional_channels;
	MSS_RECEIVER_LIST receiver_specifications[MAX_RECEIVER_SPECS];  // Constellation of receiver vectors
	int               n_receiver_specs;         
	MSSVECTOR3D speaker_positions           [MAX_SPEAKERS]; // Listener-relative speaker locations
	float         speaker_wet_reverb_response [MAX_SPEAKERS]; // Reverb sensitivity of each speaker
	float         speaker_dry_reverb_response [MAX_SPEAKERS];
};

struct MSS_BB              // Used in both MC and conventional mono/stereo configurations
{
	int  *buffer;                 // Build buffer
	int      bytes;                  // Size in bytes
	int      chans;                  // Always mono (1) or stereo (2)
	int      speaker_offset;         // Destination offset in interleaved PCM block for left channel
};


//############################################################################
//##                                                                        ##
//## Interface "ASI stream"                                                 ##
//##                                                                        ##
//############################################################################
typedef int (__stdcall  * AILASIFETCHCB) (UINT     user,            // User value passed to ASI_open_stream()
                                               void  *dest,            // Location to which stream data should be copied by app
                                               int       bytes_requested, // # of bytes requested by ASI codec
                                               int       offset);         // If not -1, application should seek to this point in stream


//
// Open a stream, returning handle to stream
//

typedef HASISTREAM (__stdcall  *ASI_STREAM_OPEN) (UINT         user,              // User value passed to fetch callback
                                                   AILASIFETCHCB fetch_CB,          // Source data fetch handler
                                                   UINT           total_size);       // Total size for %-done calculations (0=unknown)

//
// Translate data in stream, returning # of bytes actually decoded or encoded
//
// Any number of bytes may be requested.  Requesting more data than is
// available in the codec's internal buffer will cause the AILASIFETCHCB
// handler to be called to fetch more data from the stream.
//

typedef int  (__stdcall  *ASI_STREAM_PROCESS) (HASISTREAM  stream,              // Handle of stream
                                                void    *buffer,              // Destination for processed data
                                                int         buffer_size);        // # of bytes to return in buffer

//
// Restart stream decoding process at new offset
//
// Relevant for decoders only
//
// Seek destination is given as offset in bytes from beginning of stream
//
// At next ASI_stream_process() call, decoder will seek to the closest possible
// point in the stream which occurs at or after the specified position
//
// This function has no effect for decoders which do not support random
// seeks on a given stream type
//
// Warning: some decoders may need to implement seeking by reparsing
// the entire stream up to the specified offset, through multiple calls
// to the data-fetch callback.  This operation may be extremely
// time-consuming on large files or slow network connections.
//
// A stream_offset value of -1 may be used to inform the decoder that the
// application has changed the input stream offset on its own, e.g. for a
// double-buffering application where the ASI decoder is not accessing the
// stream directly.  ASI decoders should respond to this by flushing all
// internal buffers and resynchronizing themselves to the data stream.
//

typedef ASIRESULT (__stdcall  *ASI_STREAM_SEEK)    (HASISTREAM stream,
                                                     int        stream_offset);

//
// Retrieve or set a property value by index (returns 1 on success)
//

typedef int (__stdcall  *ASI_STREAM_PROPERTY) (HASISTREAM stream,
                                                HPROPERTY  property,
                                                void  * before_value,
                                                void const  * new_value,
                                                void  * after_value
                                                );

//
// Close stream, freeing handle and all internally-allocated resources
//

typedef ASIRESULT (__stdcall  *ASI_STREAM_CLOSE) (HASISTREAM stream);



struct ASISTAGE
{
	ASI_STREAM_OPEN           ASI_stream_open;
	ASI_STREAM_PROCESS        ASI_stream_process;
	ASI_STREAM_SEEK           ASI_stream_seek;
	ASI_STREAM_CLOSE          ASI_stream_close;
	ASI_STREAM_PROPERTY       ASI_stream_property;

	HPROPERTY INPUT_BIT_RATE;
	HPROPERTY INPUT_SAMPLE_RATE;
	HPROPERTY INPUT_BITS;
	HPROPERTY INPUT_CHANNELS;
	HPROPERTY OUTPUT_BIT_RATE;
	HPROPERTY OUTPUT_SAMPLE_RATE;
	HPROPERTY OUTPUT_BITS;
	HPROPERTY OUTPUT_CHANNELS;
	HPROPERTY OUTPUT_CHANNEL_MASK;
	HPROPERTY OUTPUT_RESERVOIR;
	HPROPERTY POSITION;
	HPROPERTY PERCENT_DONE;
	HPROPERTY MIN_INPUT_BLOCK_SIZE;
	HPROPERTY RAW_RATE;
	HPROPERTY RAW_BITS;
	HPROPERTY RAW_CHANNELS;
	HPROPERTY REQUESTED_RATE;
	HPROPERTY REQUESTED_BITS;
	HPROPERTY REQUESTED_CHANS;
	HPROPERTY STREAM_SEEK_POS;
	HPROPERTY DATA_START_OFFSET;
	HPROPERTY DATA_LEN;

	HASISTREAM stream;
};

typedef void (__stdcall  *MIXER_MERGE)(void const **src, UINT *src_fract, void const  *src_end,
                                           int **dest, void *dest_end, int *left_val, int *right_val,
                                           int playback_ratio, int scale_left, int scale_right,
                                           UINT operation, UINT MMX_available);

struct FLTSTAGE
{
	struct _FLTPROVIDER  *provider;
	HSAMPLESTATE             sample_state;
};

struct SPINFO
{
	int       active;    // Pass-through if 0, active if 1
	HPROVIDER provider;

	union
	{
		ASISTAGE ASI;
		MIXER_MERGE MSS_mixer_merge;
		FLTSTAGE FLT;
	}
	TYPE;
};


struct ADPCMDATA
{
	UINT blocksize;
	UINT extrasamples;
	UINT blockleft;
	UINT step;
	UINT savesrc;
	UINT sample;
	UINT destend;
	UINT srcend;
	UINT samplesL;
	UINT samplesR;
	WORD moresamples[16];
};

typedef void (__stdcall  *MIXER_FLUSH)(int* dest, int len, UINT MMX_available);
typedef void (__stdcall  *MIXER_COPY)(void const *src, int src_len,  void *dest, UINT operation, UINT option);
typedef void (__stdcall  * MIXER_MC_COPY)( MSS_BB  * build, int n_build_buffers, void  * lpWaveAddr, int hw_format, int use_MMX, int samples_per_buffer, int physical_channels_per_sample );
typedef void (__stdcall  * MIXER_ADPCM_DECODE ) ( void  * dest, void const  * in, int out_len, int in_len, int input_format, ADPCMDATA  *adpcm_data);

struct DPINFO
{
	int       active;    // Pass-through if 0, active if 1
	HPROVIDER provider;

	union
	{
		MIXER_FLUSH         MSS_mixer_flush;
		MIXER_COPY          MSS_mixer_copy;
		MIXER_MC_COPY       MSS_mixer_mc_copy;
		MIXER_ADPCM_DECODE  MSS_mixer_adpcm_decode;
	}
	TYPE;
};

typedef int (__stdcall  *SS_STREAM_CB)   (HSAMPLE  S, short  *dest_mono_sample_buffer, int dest_buffer_size);

struct REVERB_CONSTANT_INFO
{
	float * start0, * start1, * start2, * start3, * start4, * start5;
	float * end0, * end1, * end2, * end3, * end4, * end5;
	float C0, C1, C2, C3, C4, C5;
	float A;
	float B0, B1;
};

struct REVERB_UPDATED_INFO
{
	float  * address0,  * address1,  * address2,  * address3,  * address4,  * address5;
	float X0, X1, Y0, Y1;
};


struct REVERB_INFO
{
	REVERB_UPDATED_INFO u;
	REVERB_CONSTANT_INFO c;
};


struct DIG_DRIVER //0xFD0 =4048         // Handle to digital audio driver
{
	char        tag[4];              // HDIG
	HTIMER      backgroundtimer;     // Background timer handle
	float         master_volume;       // Master sample volume 0-1.0
	int         DMA_rate;            // Hardware sample rate
	int         hw_format;           // DIG_F code in use
	int         n_active_samples;    // # of samples being processed
	MSS_MC_SPEC channel_spec;        // Original "channels" value passed to AIL_open_digital_driver()
	D3DSTATE    D3D;                 // 3D listener parms for all platforms
	int         quiet;               // # of consecutive quiet sample periods
	UINT         hw_mode_flags;       // DIG_PCM_ flags for mode in use
	int         playing;             // Playback active if non-zero
	int         bytes_per_channel;            // # of bytes per channel (always 1 or 2 for 8- or 16-bit hardware output)
	int         samples_per_buffer;           // # of samples per build buffer / half-buffer
	int         physical_channels_per_sample; // # of channels per *physical* sample (1 or 2, or more in discrete MC mode)
	int         logical_channels_per_sample;  // # of logical channels per sample (may differ from physical channel count in matrix formats)
	HSAMPLE     samples;             // Pointer to list of SAMPLEs
	UINT        *sample_status;       // SMP_ flags: _FREE, _DONE, _PLAYING, moved out of SAMPLEs for faster iteration
	int         n_samples;           // # of SAMPLEs

	int       system_data[8];      // Miscellaneous system data
	// Build buffers
	MSS_BB      build[MAX_SPEAKERS+EXTRA_BUILD_BUFFERS];
	int         n_build_buffers;      // # of build buffers actually used for output processing
	int         build_size;           // # of bytes in build buffer 0, used for position comparisons
	int         hardware_buffer_size; // Size of each output buffer
	
	// waveOut-specific interface data
	HWAVEOUT    hWaveOut;            // Wave output driver
	UINT         reset_works;         // TRUE if OK to do waveOutReset
	UINT         request_reset;       // If nonzero, do waveOutReset ASAP
	LPWAVEHDR   first;               // Pointer to first WAVEHDR in chain
	int         n_buffers;           // # of output WAVEHDRs in chain
	LPWAVEHDR volatile  *return_list; // Circular list of returned WAVEHDRs
	int       volatile      return_head; // Head of WAVEHDR list (insertion point)
	int       volatile      return_tail; // Tail of WAVEHDR list (retrieval point)
	// DirectSound-specific interface data
	UINT                  guid;        // The guid id of the ds driver
	AILLPDIRECTSOUND       pDS;         // DirectSound output driver (don't
	UINT                    ds_priority; // priority opened with
	int                    emulated_ds; // is ds emulated or not?
	AILLPDIRECTSOUNDBUFFER lppdsb;      // primary buffer or null
	UINT                  dsHwnd;      // HWND used with DirectSound
	AILLPDIRECTSOUNDBUFFER  * lpbufflist;   // List of pointers to secondary buffers
	HSAMPLE          *samp_list;      // HSAMPLE associated with each buffer
	int              *sec_format;     // DIG_F_ format for secondary buffer
	int                  max_buffs;      // Max. allowable # of secondary buffers
	// Driver output configuration
	MPCMWAVEFORMAT wformat;          // format from waveout open
	char             wfextra[32];      // Extension to PCMWAVEFORMAT (e.g., WAVE_FORMAT_EXTENSIBLE)
	// Misc. data
	int         released;            // has the sound manager been released?
	UINT         foreground_timer;    // Foreground timer handle
	HDIGDRIVER  next;                // Pointer to next HDIGDRIVER in use
	int      callingCT;              // Calling EXE's task number (16 bit only)
	int      callingDS;              // Calling EXE's DS (used in 16 bit only)
	// Vars for waveOut emulation
	int DS_initialized;
	AILLPDIRECTSOUNDBUFFER DS_sec_buff;    // Secondary buffer (or NULL if none)
	AILLPDIRECTSOUNDBUFFER DS_out_buff;    // Output buffer (may be sec or prim)
	int DS_buffer_size;                    // Size of entire output buffer
	int DS_frag_cnt;                 // Total fragment count and size, and
	int DS_frag_size;                // last fragment occupied by play cursor
	int DS_last_frag;
	int DS_last_write;
	int DS_last_timer;
	int DS_skip_time;
	int DS_use_default_format;       // 1 to force use of default DS primary buffer format
	UINT position_error;              // last status from position report (can be used
	UINT last_ds_play;
	UINT last_ds_write;
	UINT last_ds_move;
	int         use_MMX;             // Use MMX with this driver if TRUE
	UINT us_count;
	UINT ms_count;
	UINT last_ms_polled;
	UINT last_percent;
	// Digital driver pipeline filter stages
	DPINFO pipeline[N_DIGDRV_STAGES];
	struct _FLTPROVIDER  *voice_filter;
	SS_STREAM_CB             stream_callback;
	struct _FLTPROVIDER  *matrix_filter;
	// Reverb
	int      room_type;           // Changes to this drive master_wet and duration/damping/predelay!
	float      master_wet;          // Master reverb level 0-1.0
	float      master_dry;          // Master non-reverb level 0-1.0
	REVERB_INFO ri;
	int  *reverb_build_buffer;
	int      reverb_total_size;
	int      reverb_fragment_size;
	int      reverb_buffer_size;
	int      reverb_on;
	UINT      reverb_off_time_ms;
	UINT      reverb_duration_ms;
	float      reverb_decay_time_s;
	float      reverb_predelay_s;
	float      reverb_damping;
	int      reverb_head;
	int      reverb_tail;
	int                  no_wom_done;    // don't process WOM_DONEs on this driver
	UINT                  wom_done_buffers;
};

struct SMPBUF           
{
	void const  *start;          // Sample buffer address (W)
	UINT             len;            // Sample buffer size in bytes (W)
	UINT             pos;            // Index to next byte (R/W)
	UINT             done;           // Nonzero if buffer with len=0 sent by app
	int             reset_ASI;      // Reset the ASI decoder at the end of the buffer
	int             reset_seek_pos; // New destination offset in stream source data, for ASI codecs that care
};

struct LOWPASS_CONSTANT_INFO
{
	int A;
	int B0, B1;
};

struct LOWPASS_UPDATED_INFO
{
	int XL0, XL1;
	int YL0, YL1;
	int XR0, XR1;
	int YR0, YR1;
};

struct LOWPASS_INFO
{
	LOWPASS_UPDATED_INFO u;
	LOWPASS_CONSTANT_INFO c;
	float cutoff;
	int on;
};

typedef void (__stdcall * AILSAMPLECB)   (HSAMPLE sample);
typedef float  (__stdcall * AILFALLOFFCB)  (HSAMPLE sample, float distance, float rolloff_factor, float min_dist, float max_dist);

AILSAMPLECB __stdcall AIL_register_EOS_callback(HSAMPLE s, AILSAMPLECB eos);



struct S3DSTATE           // Portion of HSAMPLE that deals with 3D positioning
{
	MSSVECTOR3D   position;         // 3D position
	MSSVECTOR3D   face;             // 3D orientation
	MSSVECTOR3D   up;               // 3D up-vector
	MSSVECTOR3D   velocity;         // 3D velocity
	int           doppler_valid;    // TRUE if OK to apply Doppler shift
	float           doppler_shift;    // Scalar for S->playback rate
	float           inner_angle;      // Cone attenuation parameters
	float           outer_angle;      // (Angles divided by two and convered to rads for dot-product comparisons)
	float           outer_volume;
	int           cone_enabled;
	float           max_dist;         // Sample distances
	float           min_dist;
	int           dist_changed;     // TRUE if min/max distances have changed and need to be sent to the hardware
	int           auto_3D_atten;    // TRUE if distance/cone attenuation should be applied to wet signal
	float           atten_3D;         // Attenuation due to distance/cone effects, calculated by software 3D positioner
	HSAMPLE       owner;            // May be NULL if used for temporary/internal calculations
	AILFALLOFFCB  falloff_function; // User function for min/max distance calculations, if desired
};


struct SAMPLE           // Sample instance
{
	char       tag[4];            // HSAM
	HDIGDRIVER driver;            // Driver for playback
	int        index;             // Numeric index of this sample 
	SMPBUF  *buf;                 // Source data buffers
	UINT      src_fract;           // Fractional part of source address
	int     *left_val;            // Mixer source value from end of last buffer
	int     *right_val;
	int      n_buffers;           // # of buffers (default = 2)
	int      head;
	int      tail;
	int      starved;             // Buffer stream has run out of data
	int      exhaust_ASI;         // Are we prolonging the buffer lifetime until ASI output is exhausted?
	int      loop_count;          // # of cycles-1 (1=one-shot, 0=indefinite)
	int      loop_start;          // Starting offset of loop block (0=SOF)
	int      loop_end;            // End offset of loop block (-1=EOF)
	int      orig_loop_count;     // Original loop properties specified by app, before any
	int      orig_loop_start;     // alignment constraints
	int      orig_loop_end;
	int      format;              // DIG_F format (8/16 bits, mono/stereo/multichannel)
	int      n_channels;          // # of channels (which can be >2 for multichannel formats)
	UINT      flags;               // DIG_PCM_SIGN / DIG_PCM_ORDER (stereo only)
	UINT      channel_mask;        // Same as WAVEFORMATEXTENSIBLE.dwChannelMask
	int      playback_rate;       // Playback rate in hertz
	float      save_volume;         // Sample volume 0-1.0
	float      save_pan;            // Mono panpot/stereo balance (0=L ... 1.0=R)
	float      left_volume;         // Left/mono volume 0 to 1.0
	float      right_volume;        // Right volume 0 to 1.0
	float      wet_level;           // reverb level 0 to 1.0
	float      dry_level;           // non-reverb level 0 to 1.0
	float      obstruction;
	float      occlusion;
	float      exclusion;
	float      auto_3D_channel_levels[MAX_SPEAKERS]; // Channel levels set by 3D positioner (always 1.0 if not 3D-positioned)
	float     *user_channel_levels[MAX_SPEAKERS]; // Channel levels set by AIL_set_sample_channel_levels() [source_channels][driver->logical_channels]
	int     *cur_scale          [MAX_SPEAKERS]; // Calculated 11-bit volume scale factors for current/previous mixing interval
	int     *prev_scale         [MAX_SPEAKERS]; // (These are all indexed by build buffer*2, not speaker indexes!)
	int     *ramps_left         [MAX_SPEAKERS];
	char           speaker_enum_to_source_chan[20];   // array[MSS_SPEAKER_xx] = -1 if not present, else channel #
	LOWPASS_INFO *lp;             // low pass info
	float      cutoff_param;         
	float      calculated_cut;
	int      service_type;        // 1 if single-buffered; 2 if streamed

	AILSAMPLECB SOB; // Start-of-block callback function
	AILSAMPLECB EOB; // End-of-buffer callback function
	AILSAMPLECB EOS; // End-of-sample callback function
	int user_data[8]; // Miscellaneous user data
	int system_data[8]; // Miscellaneous system data
	
	ADPCMDATA adpcm;
	int     *last_decomp;         // last sample in an asi or adpcm buffer
	int      doeob;               // Flags to trigger callbacks
	int      dosob;
	int      doeos;
	// Sample pipeline stages
	SPINFO   pipeline[N_SAMPLE_STAGES];
	int      n_active_filters;    // # of SP_FILTER_n stages active
	// 3D-related state for all platforms (including Xbox)
	int      is_3D;               // TRUE if channel levels are derived automatically from 3D positional state, FALSE if they're controlled manually
	S3DSTATE S3D;                 // Software version applies 3D positioning only if is_3D == TRUE, but output filters always use it
	void  *voice;              // Optional object used by output filter to store per-sample information such as DS3D buffers
	// Platform-specific members
	float leftb_volume;         // Left/mono volume 0 to 1.0 (back)
	float rightb_volume;        // Right volume 0 to 1.0 (back)
	float center_volume;        // Center volume 0 to 1.0
	float low_volume;           // Low volume 0 to 1.0
	float save_fb_pan;          // Sample volume 0-1.0
	float save_center;          // saved center level
	float save_low;             // saved sub level
	// DirectSound-specific data
	int      service_interval;    // Service sample every n ms
	int      service_tick;        // Current service countdown value
	int      buffer_segment_size; // Buffer segment size to fill
	int      prev_segment;        // Previous segment # (0...n)
	int      prev_cursor;         // Previous play cursor location
	int      bytes_remaining;     // # of bytes left to play (if not -1)
	int      direct_control;      // 1 if app controls buffer, 0 if MSS
};

typedef void (__stdcall * AILSTREAMCB)   (HSTREAM stream);

struct STREAM //0x11C =284
{
	int block_oriented; // 1 if this is an ADPCM or ASI-compressed stream
	int using_ASI;      // 1 if using ASI decoder to uncompress stream data
	ASISTAGE  *ASI;  // handy pointer to our ASI coded
	HSAMPLE samp;       // the sample handle
	UINT fileh;        // the open file handle
	BYTE * bufs[3];    // the data buffers
	UINT bufsizes[3];    // the size of each buffer
	int reset_ASI[3];   // should we reset the ASI at the end of the buffer?
	int reset_seek_pos[3]; // new stream position after reset
	int bufstart[3];    // offset of where this buffer started
	void * asyncs[3];// async read structures
	int loadedbufstart[2]; // offset of where the loaded buffer started
	int loadedorder[2]; // order of the buffers as they were loaded
	int loadorder;      // incremented as each buffer is loaded
	int bufsize;        // size of each buffer
	int readsize;       // size of each read block
	UINT buf1;           // 0,1,2 (current buffer that we are reading into)
	int size1;          // holds the current amount of data read
	UINT buf2;           // 0,1,2 (the next buffer that we are reading into)
	int size2;          // next buffer loaded up to
	UINT buf3;           // 0,1,2 (the next buffer that we are reading into)
	int size3;          // next buffer loaded up to
	UINT datarate;       // datarate in bytes per second
	int filerate;       // original datarate of the file
	int filetype;       // file format type
	UINT fileflags;      // file format flags (signed or unsigned)
	int totallen;       // total length of the sound data
	int substart;       // subblock loop start
	int sublen;         // subblock loop len
	int subpadding;     // amount to pad the final block
	UINT blocksize;      // ADPCM block size
	int padding;        // padding to be done
	int padded;         // padding done
	int loadedsome;     // have we done any loads?
	UINT startpos;       // point that the sound data begins
	UINT totalread;      // total bytes read from the disk
	UINT loopsleft;      // how many loops are left
	UINT error;          // read error has occurred
	int preload;        // preload the file into the first buffer
	UINT preloadpos;     // position to use in preload
	int noback;         // no background processing
	int alldone;        // alldone
	int primeamount;    // amount to load after a seek
	int readatleast;    // forced amount to read on next service
	int playcontrol;    // control: 0=stopped, 1=started, |8=paused, |16=sample paused
	AILSTREAMCB callback;  // end of stream callback
	int user_data[8];   // Miscellaneous user data
	void * next;       // pointer to next stream
	int autostreaming;  // are we autostreaming this stream
	int docallback;     // set when it time to poll for a callback
};

struct AILSOUNDINFO
{
	int format;
	void const * data_ptr;
	UINT data_len;
	UINT rate;
	int bits;
	int channels;
	UINT channel_mask;
	UINT samples;
	UINT block_size;
	void const * initial_ptr;
};


//typedef HWAVEIN  *LPHWAVEIN;
typedef HWAVEOUT  *LPHWAVEOUT;
//typedef struct MWAVEFORMAT *LPWAVEFORMAT;


enum MILES_DIGITAL_STATUS
{
	SMP_FREE          = 0x0001,    // Sample is available for allocation
	SMP_DONE          = 0x0002,    // Sample has finished playing, or has never been started
	SMP_PLAYING       = 0x0004,    // Sample is playing
	SMP_STOPPED       = 0x0008,    // Sample has been stopped
	SMP_PLAYINGBUTRELEASED = 0x0010 // Sample is playing, but digital handle
};


int __stdcall AIL_digital_CPU_percent(HDIGDRIVER dig);


DWORD __stdcall AIL_startup(void);
int __stdcall AIL_get_preference(DWORD number);
HSAMPLE __stdcall AIL_allocate_sample_handle(HDIGDRIVER dig);
int __stdcall AIL_set_preference(DWORD number, int value);
void __stdcall AIL_digital_configuration(HDIGDRIVER dig, int* rate, int* format, char* string);
HSTREAM __stdcall AIL_open_stream(HDIGDRIVER dig, char const* filename, int stream_mem);
int __stdcall AIL_stream_status(HSTREAM stream);
void __stdcall AIL_close_stream(HSTREAM stream);
void __stdcall AIL_pause_stream(HSTREAM stream, int onoff);
void __stdcall AIL_stream_volume_pan(HSTREAM stream, float* volume, float* pan);
void __stdcall AIL_set_stream_volume_pan(HSTREAM stream, float volume, float pan);
void __stdcall AIL_stream_ms_position(HSTREAM stream, int* total_milliseconds, int* current_milliseconds);
void __stdcall AIL_set_stream_ms_position(HSTREAM stream, int milliseconds);



int __stdcall AIL_init_sample(HSAMPLE s, int format, UINT flags);
void __stdcall AIL_set_sample_volume_pan(HSAMPLE s, float volume, float pan);
int __stdcall AIL_set_sample_file(HSAMPLE s, void const* file_image, int block);
void __stdcall AIL_release_sample_handle(HSAMPLE s);
UINT __stdcall AIL_sample_status(HSAMPLE s);
void __stdcall AIL_sample_volume_pan(HSAMPLE s, float* volume, float* pan);




//UINT flags
#define AIL_OPEN_DIGITAL_FORCE_PREFERENCE 1
#define AIL_OPEN_DIGITAL_NEED_HW_3D       2
#define AIL_OPEN_DIGITAL_NEED_FULL_3D     4
#define AIL_OPEN_DIGITAL_NEED_LIGHT_3D    8
#define AIL_OPEN_DIGITAL_NEED_HW_REVERB   16
#define AIL_OPEN_DIGITAL_NEED_REVERB      32
#define AIL_OPEN_DIGITAL_USE_IOP_CORE0    64
HDIGDRIVER __stdcall AIL_open_digital_driver(UINT frequency, int bits, int channel, UINT flags);


int __stdcall AIL_waveOutOpen(HDIGDRIVER* drvr, LPHWAVEOUT *lphWaveOut, int wDeviceID, LPWAVEFORMAT lpFormat);
int __stdcall AIL_WAV_info(void const* data, AILSOUNDINFO* info);

enum MILES_FILE_TYPE
{
	AILFILETYPE_UNKNOWN         =0,
	AILFILETYPE_PCM_WAV         =1,
	AILFILETYPE_ADPCM_WAV       =2,
	AILFILETYPE_OTHER_WAV       =3,
	AILFILETYPE_VOC             =4,
	AILFILETYPE_MIDI            =5,
	AILFILETYPE_XMIDI           =6,
	AILFILETYPE_XMIDI_DLS       =7,
	AILFILETYPE_XMIDI_MLS       =8,
	AILFILETYPE_DLS             =9,
	AILFILETYPE_MLS            =10,
	AILFILETYPE_MPEG_L1_AUDIO  =11,
	AILFILETYPE_MPEG_L2_AUDIO  =12,
	AILFILETYPE_MPEG_L3_AUDIO  =13,
	AILFILETYPE_OTHER_ASI_WAV  =14,
	AILFILETYPE_XBOX_ADPCM_WAV =15,
	AILFILETYPE_OGG_VORBIS     =16,
	AILFILETYPE_V12_VOICE      =17,
	AILFILETYPE_V24_VOICE      =18,
	AILFILETYPE_V29_VOICE      =19,
	AILFILETYPE_OGG_SPEEX      =20,
	AILFILETYPE_S8_VOICE       =21,
	AILFILETYPE_S16_VOICE      =22,
	AILFILETYPE_S32_VOICE      =23
};

int __stdcall AIL_file_type(void const* data, UINT size);
int __stdcall AIL_decompress_ADPCM(AILSOUNDINFO const* info, void** outdata, UINT* outsize);
typedef int (__stdcall *AILLENGTHYCB)(UINT state, UINT user);
int __stdcall AIL_decompress_ASI(void const* indata, UINT insize, char const* filename_ext, void** wav, UINT* wavsize, AILLENGTHYCB callback);

/*

#ifndef MSS_VERSION

#define MSS_VERSION      "7.2a"     // Version string needs to be 4 characters long for benefit of MIDIECHW and SETSOUND 
#define MSS_MAJOR_VERSION 7
#define MSS_MINOR_VERSION 2
#define MSS_SUB_VERSION   0
#define MSS_VERSION_DATE "20-Dec-07"

#define MSS_COPYRIGHT "Copyright (C) 1991-2008, RAD Game Tools, Inc."

#endif

#ifndef MSS_H
#define MSS_H

// IS_WINDOWS for Windows or Win32
// IS_WIN32 for Win32
// IS_WIN32API for Windows, Xbox and Xenon
// IS_32 for 32-bit DOS or Win32
// IS_LE for little endian (PCs)
// IS_X86 for Intel

#ifdef IS_WIN32
#undef IS_WIN32
#endif

#ifdef IS_32
#undef IS_32
#endif


#ifdef IS_LE
#undef IS_LE
#endif


#ifdef IS_X86
#undef IS_X86
#endif

#undef MSSRESTRICT
#define MSSRESTRICT



//
// Pipeline filters supported on following platforms
//

#if defined(IS_WIN32API)
   #define MSS_FLT_SUPPORTED 1
   
   #define FLT_A (MAX_SPEAKERS)

   #if defined(IS_WIN32API) 
      #define MSS_VFLT_SUPPORTED 1
   #endif
#else
   #define EXTRA_BUILD_BUFFERS 0
#endif

#if defined(IS_WIN32)
   #define MSS_REVERB_SUPPORTED 1
#endif


#undef MSS_STRUCT
#define MSS_STRUCT struct


#define __stdcall //$ __attribute__((cdecl))
#define AILEXPORT //$ __attribute__((cdecl))
#define DXDEC extern
#define DXDEF
#define __stdcall //$__attribute__((cdecl))

#define FOURCC UINT



#define AILLIBCALLBACK //__attribute__((cdecl))

#define MSS_MAIN_DEF

#define MSS_REDIST_DIR_NAME "redist"

#define MSS_DIR_SEP "\\"
#define MSS_DIR_UP ".." MSS_DIR_SEP
#define MSS_DIR_UP_TWO MSS_DIR_UP MSS_DIR_UP


#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN

#ifndef WIN32_EXTRA_LEAN
#define WIN32_EXTRA_LEAN
#endif

#ifndef STRICT
#define STRICT
#endif

//typedef int                 BOOL;
//typedef long LONG;
//typedef char *LPSTR, *PSTR;


//typedef ULONG_PTR DWORD_PTR, *PDWORD_PTR;
typedef unsigned long       DWORD;
typedef unsigned short      WORD;
typedef unsigned int        UINT;
typedef struct HWAVE__ *HWAVE;
typedef struct HWAVEIN__ *HWAVEIN;

#ifndef WAVE_MAPPER
#define WAVE_MAPPER     ((UINT)-1)
#endif


typedef struct HMIDIOUT__ *HMIDIOUT;
typedef HMIDIOUT  *LPHMIDIOUT;
typedef struct HWND__ *HWND;
typedef struct HINSTANCE__ *HINSTANCE;
typedef HINSTANCE HMODULE;      
typedef struct wavehdr_tag *LPWAVEHDR;
//typedef DWORD   FOURCC;         


#define MSS_MAIN_DEF __cdecl

//
// If compiling MSS DLL, use __declspec(dllexport) for both
// declarations and definitions
//
// If compiling MSS16 library or application, use "extern" in declarations
// and nothing in definitions
//

#ifdef IS_WIN32

  #undef 
  #define 

  #ifndef FORNONWIN
  #define AILLIBCALLBACK __stdcall
  #define __stdcall        __stdcall
  #define __stdcall    __stdcall
  #define AILEXPORT      __stdcall
  #else
  #define AILLIBCALLBACK __cdecl
  #define __stdcall        __cdecl
  #define __stdcall    __cdecl
  #define AILEXPORT      __cdecl
  #endif
  
  #ifdef __RADINDLL__
    #define DXDEC __declspec(dllexport)
    #define DXDEF __declspec(dllexport)
  #else

    #ifdef __BORLANDC__
      #define DXDEC extern
    #else
      #define DXDEC __declspec(dllimport)
    #endif

  #endif

    #define MSSDLLNAME "MSS32.DLL"
    #define MSS_REDIST_DIR_NAME "redist"

  #define MSS_DIR_SEP "\\"
  #define MSS_DIR_UP ".." MSS_DIR_SEP
  #define MSS_DIR_UP_TWO MSS_DIR_UP MSS_DIR_UP

#else

  #define __stdcall WINAPI
  #define AILEXPORT __export WINAPI
  #define __stdcall AILEXPORT
  #define AILLIBCALLBACK WINAPI

  #define  extern
  #define DXDEF

  #define MSSDLLNAME "MSS16.DLL"
  #define MSS_REDIST_DIR_NAME "redist"

  #define MSS_DIR_SEP "\\"
  #define MSS_DIR_UP ".." MSS_DIR_SEP
  #define MSS_DIR_UP_TWO MSS_DIR_UP MSS_DIR_UP

#endif

typedef void * LPVOID;

#else



  #ifdef IS_X86
    #define AILLIBCALLBACK __attribute__((cdecl))
    #define __stdcall        __attribute__((cdecl))
    #define __stdcall    __attribute__((cdecl))
    #define AILEXPORT      __attribute__((cdecl)) __attribute__((visibility("default")))

    #ifdef __RADINDLL__
      #define DXDEC __attribute__((visibility("default")))
      #define DXDEF
    #else
      #define DXDEC extern
      #define DXDEF
    #endif

  #else
    #define AILLIBCALLBACK 
    #define __stdcall        
    #define AILEXPORT      
    #define __stdcall    

    #ifdef __RADINDLL__
      #define DXDEC __declspec(export)
      #define DXDEF
    #else
      #define DXDEC extern
      #define DXDEF
    #endif

  #endif


#define MSS_MAIN_DEF


//
// Misc. constant definitions
//

#define MAX_DRVRS                16       // Max. # of simultaneous drivers
#define MAX_TIMERS               16       // Max. # of simultaneous timers
#define MAX_NOTES                32       // Max # of notes "on"
#define FOR_NEST                 4        // # of nested XMIDI FOR loops
#define NUM_CHANS                16       // # of possible MIDI channels
#define MAX_W_VOICES             16       // Max virtual wave synth voice cnt
#define MAX_W_ENTRIES            512      // 512 wave library entries max.

#define MAX_SAVED_FILTER_PROPERTIES 48    // Up to 48 filter attributes can be stored in an HSATTRIBS structure

#define MIN_CHAN                 ( 1-1)   // Min channel recognized (0-based)
#define MAX_CHAN                 (16-1)   // Max channel recognized
#define MIN_LOCK_CHAN            ( 1-1)   // Min channel available for locking
#define MAX_LOCK_CHAN            (16-1)   // Max channel available for locking
#define PERCUSS_CHAN             (10-1)   // Percussion channel (no locking)

#define AIL_MAX_FILE_HEADER_SIZE 8192     // AIL_set_named_sample_file() requires at least 8K
                                          // of data or the entire file image, whichever is less,
                                          // to determine sample format
#define DIG_F_16BITS_MASK          1
#define DIG_F_STEREO_MASK          2
#define DIG_F_ADPCM_MASK           4
#define DIG_F_XBOX_ADPCM_MASK      8
#define DIG_F_MULTICHANNEL_MASK    16
#define DIG_F_OUTPUT_FILTER_IN_USE 32

#define DIG_F_MONO_8                     0        // PCM data formats
#define DIG_F_MONO_16                    (DIG_F_16BITS_MASK)
#define DIG_F_STEREO_8                   (DIG_F_STEREO_MASK)
#define DIG_F_MULTICHANNEL_8             (DIG_F_MULTICHANNEL_MASK)   // (not actually supported)
#define DIG_F_STEREO_16                  (DIG_F_STEREO_MASK|DIG_F_16BITS_MASK)
#define DIG_F_MULTICHANNEL_16            (DIG_F_MULTICHANNEL_MASK|DIG_F_16BITS_MASK)
#define DIG_F_ADPCM_MONO_16              (DIG_F_ADPCM_MASK |DIG_F_16BITS_MASK)
#define DIG_F_ADPCM_STEREO_16            (DIG_F_ADPCM_MASK |DIG_F_16BITS_MASK|DIG_F_STEREO_MASK)
#define DIG_F_ADPCM_MULTICHANNEL_16      (DIG_F_ADPCM_MASK |DIG_F_16BITS_MASK|DIG_F_STEREO_MASK)
#define DIG_F_XBOX_ADPCM_MONO_16         (DIG_F_XBOX_ADPCM_MASK |DIG_F_16BITS_MASK)
#define DIG_F_XBOX_ADPCM_STEREO_16       (DIG_F_XBOX_ADPCM_MASK |DIG_F_16BITS_MASK|DIG_F_STEREO_MASK)
#define DIG_F_XBOX_ADPCM_MULTICHANNEL_16 (DIG_F_XBOX_ADPCM_MASK |DIG_F_16BITS_MASK|DIG_F_MULTICHANNEL_MASK)

#define DIG_F_NOT_8_BITS (DIG_F_16BITS_MASK | DIG_F_ADPCM_MASK | DIG_F_XBOX_ADPCM_MASK | DIG_F_MULTICHANNEL_MASK)

#define DIG_F_USING_ASI          16

#define DIG_PCM_SIGN             0x0001   // (obsolete)
#define DIG_PCM_ORDER            0x0002

#define DIG_PCM_POLARITY         0x0004   // PCM flags used by driver hardware
#define DIG_PCM_SPLIT            0x0008
#define DIG_BUFFER_SERVICE       0x0010
#define DIG_DUAL_DMA             0x0020
#define DIG_RECORDING_SUPPORTED  0x8000

#define WAVE_FORMAT_PCM          1
//#define WAVE_FORMAT_IMA_ADPCM    0x0011
#define WAVE_FORMAT_XBOX_ADPCM   0x0069
#define WAVE_FORMAT_EXTENSIBLE   0xFFFE

typedef enum
{
   MSS_SPEAKER_FRONT_LEFT            = 0,     // Speaker order indexes correspond to 
   MSS_SPEAKER_FRONT_RIGHT           = 1,     // bitmasks in PSDK's ksmedia.h
   MSS_SPEAKER_FRONT_CENTER          = 2,     // Also see microsoft.com/whdc/device/audio/multichaud.mspx
   MSS_SPEAKER_LOW_FREQUENCY         = 3,
   MSS_SPEAKER_BACK_LEFT             = 4,
   MSS_SPEAKER_BACK_RIGHT            = 5,
   MSS_SPEAKER_FRONT_LEFT_OF_CENTER  = 6,
   MSS_SPEAKER_FRONT_RIGHT_OF_CENTER = 7,
   MSS_SPEAKER_BACK_CENTER           = 8,
   MSS_SPEAKER_SIDE_LEFT             = 9,
   MSS_SPEAKER_SIDE_RIGHT            = 10,
   MSS_SPEAKER_TOP_CENTER            = 11,
   MSS_SPEAKER_TOP_FRONT_LEFT        = 12,
   MSS_SPEAKER_TOP_FRONT_CENTER      = 13,
   MSS_SPEAKER_TOP_FRONT_RIGHT       = 14,
   MSS_SPEAKER_TOP_BACK_LEFT         = 15,
   MSS_SPEAKER_TOP_BACK_CENTER       = 16,
   MSS_SPEAKER_TOP_BACK_RIGHT        = 17,
   MSS_SPEAKER_MAX_INDEX             = 17
}
MSS_SPEAKER;


//
// Pass to AIL_midiOutOpen for NULL MIDI driver
//

#define MIDI_NULL_DRIVER ((UINT)(int)-2)

#endif


//
// Non-specific XMIDI/MIDI controllers and event types
//

#define SYSEX_BYTE        105
#define PB_RANGE          106
#define CHAN_MUTE         107
#define CALLBACK_PFX      108
#define SEQ_BRANCH        109
#define CHAN_LOCK         110
#define CHAN_PROTECT      111
#define VOICE_PROTECT     112
#define TIMBRE_PROTECT    113
#define PATCH_BANK_SEL    114
#define INDIRECT_C_PFX    115
#define FOR_LOOP          116
#define NEXT_LOOP         117
#define CLEAR_BEAT_BAR    118
#define CALLBACK_TRIG     119
#define SEQ_INDEX         120

#define GM_BANK_MSB       0
#define MODULATION        1
#define DATA_MSB          6
#define PART_VOLUME       7
#define PANPOT            10
#define EXPRESSION        11
#define GM_BANK_LSB       32
#define DATA_LSB          38
#define SUSTAIN           64
#define REVERB            91
#define CHORUS            93
#define RPN_LSB           100
#define RPN_MSB           101
#define RESET_ALL_CTRLS   121
#define ALL_NOTES_OFF     123

#define EV_NOTE_OFF       0x80
#define EV_NOTE_ON        0x90
#define EV_POLY_PRESS     0xa0
#define EV_CONTROL        0xb0
#define EV_PROGRAM        0xc0
#define EV_CHAN_PRESS     0xd0
#define EV_PITCH          0xe0
#define EV_SYSEX          0xf0
#define EV_ESC            0xf7
#define EV_META           0xff

#define META_EOT          0x2f
#define META_TEMPO        0x51
#define META_TIME_SIG     0x58

//
// SAMPLE.system_data[] usage
//

#define VOC_BLK_PTR       1  // Pointer to current block
#define VOC_REP_BLK       2  // Pointer to beginning of repeat loop block
#define VOC_N_REPS        3  // # of iterations left in repeat loop
#define VOC_MARKER        4  // Marker to search for, or -1 if all
#define VOC_MARKER_FOUND  5  // Desired marker found if 1, else 0
#define STR_HSTREAM       6  // Stream, if any, that owns the HSAMPLE
#define SSD_TEMP          7  // Temporary storage location for general use

//
// Timer status values
//

#define AILT_FREE         0         // Timer handle is free for allocation
#define AILT_STOPPED      1         // Timer is stopped
#define AILT_RUNNING      2         // Timer is running

//
// SAMPLE.status flag values
//

                                      // has been temporarily released



//
// SEQUENCE.status flag values
//

#define SEQ_FREE          0x0001    // Sequence is available for allocation

#define SEQ_DONE          0x0002    // Sequence has finished playing, or has
                                    // never been started

#define SEQ_PLAYING       0x0004    // Sequence is playing

#define SEQ_STOPPED       0x0008    // Sequence has been stopped

#define SEQ_PLAYINGBUTRELEASED 0x0010 // Sequence is playing, but MIDI handle
                                      // has been temporarily released

 #ifndef REALPTR
 #define REALPTR(x) ((void *) (UINT) ((((UINT) (x))>>16<<4) + ((x) & 0xffff)))
 #endif

#endif

#else

#ifdef IS_WINDOWS

//
// AIL_set_direct_buffer_control() command values
//

#define AILDS_RELINQUISH  0         // App returns control of secondary buffer
#define AILDS_SEIZE       1         // App takes control of secondary buffer
#define AILDS_SEIZE_LOOP  2         // App wishes to loop the secondary buffer

#endif

#endif

//
// General type definitions for portability
//






typedef void VOIDFUNC(void);



#ifndef float
#define float float
#endif

#ifndef F64
#define F64 double
#endif


#ifndef REALFAR
#define REALFAR unsigned int
#endif

#ifndef FILE_ERRS
#define FILE_ERRS

#define AIL_NO_ERROR        0
#define AIL_IO_ERROR        1
#define AIL_OUT_OF_MEMORY   2
#define AIL_FILE_NOT_FOUND  3
#define AIL_CANT_WRITE_FILE 4
#define AIL_CANT_READ_FILE  5
#define AIL_DISK_FULL       6

#endif

#define MIN_VAL 0
#define NOM_VAL 1
#define MAX_VAL 2

//
// Preference names and default values
//

#define DIG_RESAMPLING_TOLERANCE   0
#define DEFAULT_DRT                131    // Resampling triggered at +/- 0.2%

#define DIG_MIXER_CHANNELS         1
#define DEFAULT_DMC                64     // 64 allocatable SAMPLE structures

#define DIG_MAX_PREDELAY_MS        2
#define DEFAULT_MPDMS              500    // Max predelay reverb time in ms

#define MDI_SERVICE_RATE           3
#define DEFAULT_MSR                120    // XMIDI sequencer timing = 120 Hz

#define MDI_SEQUENCES              4
#define DEFAULT_MS                 8      // 8 sequence handles/driver

#define MDI_DEFAULT_VOLUME         5
#define DEFAULT_MDV                127    // Default sequence volume = 127 (0-127)

#define MDI_QUANT_ADVANCE          6
#define DEFAULT_MQA                1      // Beat/bar count +1 interval

#define MDI_ALLOW_LOOP_BRANCHING   7
#define DEFAULT_ALB                NO     // Branches cancel XMIDI FOR loops

#define MDI_DEFAULT_BEND_RANGE     8
#define DEFAULT_MDBR               2      // Default pitch-bend range = 2

#define DIG_3D_MUTE_AT_MAX        46
#define DEFAULT_D3MAM             NO      // off by default

#if defined(IS_WINDOWS)

#define MDI_DOUBLE_NOTE_OFF        9
#define DEFAULT_MDNO               NO     // For stuck notes on SB daughterboards

#endif

#define DIG_ENABLE_RESAMPLE_FILTER 31     // Enable resampling filter by
#define DEFAULT_DERF               YES    // default

#if defined(IS_WINDOWS)

#define MDI_SYSEX_BUFFER_SIZE      10
#define DEFAULT_MSBS               1536   // Default sysex buffer = 1536 bytes

#define DIG_OUTPUT_BUFFER_SIZE     11
#define DEFAULT_DOBS               49152  // 48K output buffer size

#define AIL_MM_PERIOD              12

#if defined(IS_WINDOWS)
#define DEFAULT_AMP                5      // Default MM timer period = 5 msec.
#else
#define DEFAULT_AMP                15     // Default MM timer period = 15 msec (mac and linux)
#endif

#endif




#if defined(IS_WINDOWS)

#define DIG_DS_FRAGMENT_SIZE       34
#define DEFAULT_DDFS               8      // Use 8 millisecond buffer fragments with DirectSound if MSS mixer in use

#define DIG_DS_FRAGMENT_CNT        35
#define DEFAULT_DDFC               96     // Use 96 buffer fragments with DirectSound if MSS mixer in use

#define DIG_DS_MIX_FRAGMENT_CNT    42
#define DEFAULT_DDMFC              8      // Mix ahead 8 buffer fragments

#define DIG_DS_USE_PRIMARY         36
#define DEFAULT_DDUP               NO     // Mix into secondary DirectSound buffer by default

#define DIG_DS_DSBCAPS_CTRL3D      37
#define DEFAULT_DDDC               NO     // Do not use DSBCAPS_CTRL3D by default

#define DIG_DS_CREATION_HANDLER    38
#define DEFAULT_DDCH               NULL   // Use DirectSoundCreate() by default

#define AIL_TIMERS                 13
#define DEFAULT_AT                 16     // 16 allocatable HTIMER handles

#define DIG_MAX_CHAIN_ELEMENT_SIZE 14
#define DEFAULT_MCES               8192   // max of 8192 bytes/waveOut buffer

#define DIG_MIN_CHAIN_ELEMENT_TIME 45
#define DEFAULT_MCET               100    // 100 milliseconds buffers

#define DIG_USE_WAVEOUT            15
#define DEFAULT_DUW                NO     // Use DirectSound by default

#define DIG_DS_SECONDARY_SIZE      16
#define DEFAULT_DDSS               (32*1024L)  // Must be 2^n -- use 32K by default

#define DIG_DS_SAMPLE_CEILING      17
#define DEFAULT_DDSC               44100  // Allow up to 44 kHz samples

#define AIL_LOCK_PROTECTION        18
#define DEFAULT_ALP                NO     // Don't suspend foreground thread by default

#define AIL_MUTEX_PROTECTION       44
#define DEFAULT_AMPR               YES    // Lock each call into Miles with a mutex

#define DIG_PREFERRED_WO_DEVICE    40    
#define DEFAULT_DPWOD              WAVE_MAPPER  // Preferred WaveOut device == WAVE_MAPPER

#define DIG_PREFERRED_DS_DEVICE    47
#define DEFAULT_DPDSD              0            // Preferred DirectSound device == default NULL GUID

#else


#endif

// ----------------------------------
// DLS Preference names and default values
// Unless otherwise specified, values must be established
// BEFORE calling DLSMSSOpen()!
// ----------------------------------

#define DLS_TIMEBASE               20
#define DEFAULT_DTB                120  // 120 intervals/second by default

#define DLS_VOICE_LIMIT            21
#define DEFAULT_DVL                64   // 64 voices supported

#define DLS_BANK_SELECT_ALIAS      22
#define DEFAULT_DBSA               NO   // Do not treat controller 114 as bank

#define DLS_STREAM_BOOTSTRAP       23   // Don't submit first stream buffer
#define DEFAULT_DSB                YES  // until at least 2 available

#define DLS_VOLUME_BOOST           24
#define DEFAULT_DVB                0    // Boost final volume by 0 dB

#define DLS_ENABLE_FILTERING       25   // Filtering = on by default
#define DEFAULT_DEF                YES  // (may be changed at any time)

#ifdef IS_X86

#define AIL_ENABLE_MMX_SUPPORT     27   // Enable MMX support if present
#define DEFAULT_AEMS               YES  // (may be changed at any time)

#endif

#define DLS_GM_PASSTHROUGH         28   // Pass unrecognized traffic on to
#define DEFAULT_DGP                YES  // default GM driver layer
                                        // (may be changed at any time)

#define DLS_ADPCM_TO_ASI_THRESHOLD 39   // Size in samples to switch to ASI
#define DEFAULT_DATAT              32768

//
// Add'l platform-independent prefs
//

#define DIG_INPUT_LATENCY          41   // Use >= 250-millisecond input buffers if
#define DEFAULT_DIL                250  // explicit size request cannot be satisfied

#ifdef IS_WINDOWS

#define DIG_USE_WAVEIN             43
#define DEFAULT_DUWI               YES  // Use waveIn for input by default

#endif

#define DIG_LEVEL_RAMP_SAMPLES     48
#define DEFAULT_DLRS               32  // Ramp level changes over first 32 samples in each buffer to reduce zipper noise

#define DIG_ENCODE_QUALITY_PERCENT 49
#define DEFAULT_DEQP               50  // Default encoding quality (e.g., for DTS) = 50%

#define N_PREFS 51                      // # of preference types


typedef MSS_STRUCT Mwaveformatex_tag {
    WORD    wFormatTag;        
    WORD    nChannels;         
    UINT   nSamplesPerSec;    
    UINT   nAvgBytesPerSec;   
    WORD    nBlockAlign;       
    WORD    wBitsPerSample;
    WORD    cbSize;
} MWAVEFORMATEX;
typedef MSS_STRUCT Mwaveformatextensible_tag {
  MWAVEFORMATEX Format;
  union {
      WORD wValidBitsPerSample;
      WORD wSamplesPerBlock;
      WORD wReserved;
  } Samples;
  UINT dwChannelMask;
  BYTE SubFormat[16];
} MWAVEFORMATEXTENSIBLE;




#ifndef RIB_H        // RIB.H contents included if RIB.H not already included

#define RIB_H
#define ARY_CNT(x) (sizeof((x)) / sizeof((x)[0]))

// ----------------------------------
// RIB data types
// ----------------------------------

typedef int RIBRESULT;

#define RIB_NOERR                    0   // Success -- no error
#define RIB_NOT_ALL_AVAILABLE        1   // Some requested functions/attribs not available
#define RIB_NOT_FOUND                2   // Resource not found
#define RIB_OUT_OF_MEM               3   // Out of system RAM

//
// Handle to interface provider
//


//
// Handle representing token used to obtain property data
//
// This needs to be large enough to store a function pointer
//


//
// Handle representing an enumerated interface entry
//
// RIB_enumerate_interface() returns 1 if valid next entry found, else
// 0 if end of list reached
//

typedef UINT HINTENUM;
#define HINTENUM_FIRST 0

//
// Handle representing an enumerated provider entry
//
// RIB_enumerate_providers() returns 1 if valid next entry found, else
// 0 if end of list reached
//

#define HPROENUM_FIRST 0

//
// Data types for RIB properties
//

typedef enum
{
   RIB_NONE = 0, // No type
   RIB_CUSTOM,   // Used for pointers to application-specific structures
   RIB_DEC,      // Used for 32-bit integer values to be reported in decimal
   RIB_HEX,      // Used for 32-bit integer values to be reported in hex
   RIB_FLOAT,    // Used for 32-bit single-precision FP values
   RIB_PERCENT,  // Used for 32-bit single-precision FP values to be reported as percentages
   RIB_BOOL,     // Used for Boolean-constrained integer values to be reported as TRUE or FALSE
   RIB_STRING,   // Used for pointers to null-terminated ASCII strings
   RIB_READONLY = 0x80000000  // Property is read-only
}
RIB_DATA_SUBTYPE;

//
// RIB_ENTRY_TYPE structure, used to register an interface or request one
//

typedef enum
{
   RIB_FUNCTION = 0,
   RIB_PROPERTY       // Property: read-only or read-write data type
}
RIB_ENTRY_TYPE;

//
// RIB_INTERFACE_ENTRY, used to represent a function or data entry in an
// interface
//

typedef MSS_STRUCT
{
   RIB_ENTRY_TYPE   type;        // See list above
   char           *entry_name;  // Name of desired function or property
   UINT            token;       // Function pointer or property token
   RIB_DATA_SUBTYPE subtype;     // Property subtype
}
RIB_INTERFACE_ENTRY;

//
// Standard RAD Interface Broker provider identification properties
//

#define PROVIDER_NAME    ((UINT) (int) (-100))    // RIB_STRING name of decoder
#define PROVIDER_VERSION ((UINT) (int) (-101))    // RIB_HEX BCD version number

//
// Standard function to obtain provider properties (see PROVIDER_ defines
// above)
//
// Each provider of a searchable interface must export this function
//

typedef int (__stdcall  *PROVIDER_PROPERTY) (HPROPERTY        index,
                                              void  *       before_value,
                                              void const  * new_value,
                                              void  *       after_value
                                              );

//
// Macros to simplify interface registrations/requests for functions,
// and properties
//

#define FN(entry_name)        { RIB_FUNCTION, #entry_name, (UINT) &(entry_name), RIB_NONE }
#define REG_FN(entry_name)    { RIB_FUNCTION, #entry_name, (UINT) &(entry_name), RIB_NONE }

#define PR(entry_name,ID)             { RIB_PROPERTY, (entry_name), (UINT) &(ID), RIB_NONE }
#define REG_PR(entry_name,ID,subtype) { RIB_PROPERTY, (entry_name), (UINT)  (ID), subtype  }

#define RIB_register(x,y,z)   RIB_register_interface  ((HPROVIDER)(x), y, ARY_CNT(z), z)
#define RIB_unregister(x,y,z) RIB_unregister_interface((HPROVIDER)(ssx), y, ARY_CNT(z), z)
#define RIB_unregister_all(x) RIB_unregister_interface((HPROVIDER)(x), NULL, 0, NULL)
#define RIB_free_libraries()  RIB_free_provider_library((HPROVIDER)(NULL));
#define RIB_request(x,y,z)    RIB_request_interface   (x, y, ARY_CNT(z), z)

// ----------------------------------
// Standard RIB API prototypes
// ----------------------------------

 HPROVIDER  __stdcall RIB_alloc_provider_handle   (long           module);
 void       __stdcall RIB_free_provider_handle    (HPROVIDER      provider);

 HPROVIDER  __stdcall RIB_load_provider_library   (char const  *filename);
 void       __stdcall RIB_free_provider_library   (HPROVIDER      provider);

 RIBRESULT  __stdcall RIB_register_interface      (HPROVIDER                      provider,
                                                       char const                    *interface_name,
                                                       int                            entry_count,
                                                       RIB_INTERFACE_ENTRY const  *rlist);

 RIBRESULT  __stdcall RIB_unregister_interface    (HPROVIDER                      provider,
                                                       char const                   *interface_name,
                                                       int                            entry_count,
                                                       RIB_INTERFACE_ENTRY const  *rlist);

 RIBRESULT  __stdcall RIB_request_interface       (HPROVIDER                provider,
                                                       char const             *interface_name,
                                                       int                      entry_count,
                                                       RIB_INTERFACE_ENTRY  *rlist);

 RIBRESULT  __stdcall RIB_request_interface_entry (HPROVIDER                provider,
                                                       char const             *interface_name,
                                                       RIB_ENTRY_TYPE           entry_type,
                                                       char const             *entry_name,
                                                       UINT                *token);

 int        __stdcall RIB_enumerate_interface     (HPROVIDER                provider,
                                                       char                   *interface_name,
                                                       RIB_ENTRY_TYPE           type,
                                                       HINTENUM             *next,
                                                       RIB_INTERFACE_ENTRY  *dest);

 int        __stdcall RIB_enumerate_providers     (char                   *interface_name,
                                                       HPROENUM             *next,
                                                       HPROVIDER            *dest);

 char  *   __stdcall RIB_type_string             (void const  *         data,
                                                       RIB_DATA_SUBTYPE         subtype);

 HPROVIDER  __stdcall RIB_find_file_provider      (char const             *interface_name,
                                                       char const             *property_name,
                                                       char const             *file_suffix);

 HPROVIDER  __stdcall RIB_find_provider           (char const             *interface_name,
                                                       char const             *property_name,
                                                       void const           *property_value);

//
// Static library definitions
//

  #define RIB_MAIN_NAME( name ) RIB_Main
  DXDEC int __stdcall RIB_Main( HPROVIDER provider_handle, UINT up_down );

typedef int ( __stdcall  * RIB_MAIN_FUNC) ( HPROVIDER provider_handle, UINT up_down );

DXDEC HPROVIDER __stdcall RIB_load_static_provider_library (RIB_MAIN_FUNC main, char * description);


 HPROVIDER  __stdcall RIB_find_files_provider     (char const  *interface_name,
                                                       char const  *property_name_1,
                                                       char const  *file_suffix_1,
                                                       char const  *property_name_2,
                                                       char const  *file_suffix_2);

 HPROVIDER  __stdcall RIB_find_file_dec_provider  (char const  *interface_name,
                                                       char const  *property_name_1,
                                                       UINT           decimal_property_value_1,
                                                       char const  *property_name_2,
                                                       char const  *file_suffix_2);

 int        __stdcall RIB_load_application_providers
                                                      (char const   *filespec);

 void       __stdcall RIB_set_provider_user_data  (HPROVIDER provider,
                                                       UINT       index,
                                                       int     value);

 int      __stdcall RIB_provider_user_data      (HPROVIDER provider,
                                                       UINT       index);

 void       __stdcall RIB_set_provider_system_data
                                                      (HPROVIDER provider,
                                                       UINT       index,
                                                       int     value);

 int      __stdcall RIB_provider_system_data    (HPROVIDER provider,
                                                       UINT       index);

 char  *  __stdcall  RIB_error                   (void);

#endif

#ifndef MSS_ASI_VERSION // MSSASI.H contents included if MSSASI.H not already included

#define AIL_ASI_VERSION  1
#define AIL_ASI_REVISION 0

//
// Handle to stream being managed by ASI codec
//


//
// ASI result codes
//


#define ASI_NOERR                   0   // Success -- no error
#define ASI_NOT_ENABLED             1   // ASI not enabled
#define ASI_ALREADY_STARTED         2   // ASI already started
#define ASI_INVALID_PARAM           3   // Invalid parameters used
#define ASI_INTERNAL_ERR            4   // Internal error in ASI driver
#define ASI_OUT_OF_MEM              5   // Out of system RAM
#define ASI_ERR_NOT_IMPLEMENTED     6   // Feature not implemented
#define ASI_NOT_FOUND               7   // ASI supported device not found
#define ASI_NOT_INIT                8   // ASI not initialized
#define ASI_CLOSE_ERR               9   // ASI not closed correctly

// ----------------------------------
// Application-provided ASI callbacks
// ----------------------------------

//
// AILASIFETCHCB: Called by ASI to obtain data from stream source
//
// offset normally will be either 0 at the first call made by the codec
// or -1 to specify a continuous stream, except when ASI_stream_seek()
// is called to restart the stream codec at a new stream offset.  In this
// case, the application must execute the seek operation on the ASI codec's
// behalf.
//
// In response to this callback, the application should read the requested
// data and copy it to the specified destination buffer, returning the number
// of bytes copied (which can be less than bytes_requested if the end of
// the stream is reached).
//


//############################################################################
//##                                                                        ##
//## Interface "ASI codec"                                                  ##
//##                                                                        ##
//############################################################################

//
// Initialize ASI stream codec
//
// No other ASI functions may be called outside an ASI_startup() /
// ASI_shutdown() pair, except for the standard RIB function
// PROVIDER_property() where appropriate. 
//

typedef ASIRESULT (__stdcall  *ASI_STARTUP)(void);

//
// Shut down ASI codec
//

typedef ASIRESULT (__stdcall  * ASI_SHUTDOWN)(void);

//
// Return codec error message, or NULL if no errors have occurred since
// last call
//
// The ASI error text state is global to all streams
//

typedef char  *  (__stdcall  * ASI_ERROR)(void);


//############################################################################
//##                                                                        ##
//## Interface "MSS mixer services"                                         ##
//##                                                                        ##
//############################################################################

//
// Operation flags used by mixer and filter modules
//

#define M_DEST_STEREO  1       // Set to enable stereo mixer output
#define M_SRC_16       2       // Set to enable mixing of 16-bit samples
#define M_FILTER       4       // Set to enable filtering when resampling
#define M_SRC_STEREO   8       // Set to enable mixing of stereo input samples
#define M_ORDER        16      // Set to reverse L/R stereo order for sample
#define M_RESAMPLE     32      // Set to enable playback ratios other than 65536
#define M_VOL_SCALING  64      // Set to enable volume scalars other than 2048
#define M_COPY16_NOVOL 128

#ifdef IS_32

//
// Initialize mixer
//
// No other mixer functions may be called outside a MIXER_startup() /
// MIXER_shutdown() pair, except for the standard RIB function
// PROVIDER_property() as appropriate.
//

typedef void (__stdcall  *MIXER_STARTUP)(void);

//
// Shut down mixer
//

typedef void (__stdcall  *MIXER_SHUTDOWN)(void);

//
// Flush mixer buffer
//


//
// Perform audio mixing operation
//



//
// Translate mixer buffer contents to final output format
//
// "option" parameter is big_endian_output on Mac, MMX on x86, overwrite flag on PS2 
//

typedef void (__stdcall  *MIXER_COPY) (void const   *src,
                                        int       src_len,
                                        void  *dest,
                                        UINT       operation
#if defined(IS_X86) || defined(IS_PS2) 
                                        ,UINT       option
#endif
                                        );  

#else

//
// Initialize mixer
//
// No other mixer functions may be called outside a MIXER_startup() /
// MIXER_shutdown() pair, except for the standard RIB function
// PROVIDER_property() as appropriate.
//

typedef void (__stdcall  *MIXER_STARTUP)(void);

//
// Shut down mixer
//

typedef void (__stdcall  *MIXER_SHUTDOWN)(void);

//
// Flush mixer buffer
//

typedef void (__stdcall  *MIXER_FLUSH)   (int  *dest,
                                           int      len,
                                           UINT      MMX_available);

//
// Perform audio mixing operation
//

typedef void (__stdcall  *MIXER_MERGE)   (UINT             src_sel,
                                           UINT             dest_sel,
                                           UINT         *src_fract,
                                           UINT         *src_offset,
                                           UINT         *dest_offset,
                                           UINT             src_end_offset,
                                           UINT             dest_end_offset,
                                           int         *left_val,
                                           int         *right_val,
                                           int             playback_ratio,
                                           int             scale_both,
                                           UINT             operation);

//
// Translate mixer buffer contents to final output format
//

#endif










//
// Type definitions
//

MSS_STRUCT _DIG_DRIVER;

MSS_STRUCT _MDI_DRIVER;


typedef MSS_STRUCT _MDI_DRIVER  * HMDIDRIVER;    // Handle to XMIDI driver


typedef MSS_STRUCT _SMPATTRIBS  * HSATTRIBS;     // Handle to sample performance attributes 

typedef MSS_STRUCT _SEQUENCE  * HSEQUENCE;       // Handle to sequence




//
// Function pointer types
//

typedef void (__stdcall * AILINCB)       (void const  *data, int len, UINT user_data);

typedef void (__stdcall * AILTRACECB)    (char *text, int nest_depth);

typedef void (__stdcall * AILTIMERCB)    (UINT user);



typedef int  (__stdcall * AILEVENTCB)    (HMDIDRIVER hmi,HSEQUENCE seq,int status,int data_1,int data_2);

typedef int  (__stdcall * AILTIMBRECB)   (HMDIDRIVER hmi,int bank,int patch);

typedef int  (__stdcall * AILPREFIXCB)   (HSEQUENCE seq,int log,int data);

typedef void (__stdcall * AILTRIGGERCB)  (HSEQUENCE seq,int log,int data);

typedef void (__stdcall * AILBEATCB)     (HMDIDRIVER hmi,HSEQUENCE seq,int beat,int measure);

typedef void (__stdcall * AILSEQUENCECB) (HSEQUENCE seq);


//
// Handle to sample and driver being managed by pipeline filter
//


//
// Digital pipeline stages
//
// These are the points at which external modules may be installed into
// a given HSAMPLE or HDIGDRIVER's processing pipeline
//











//
// Other data types
//

typedef enum 
{
   WIN32_TIMER_THREAD,   // Handle to thread used to run mixer and all other background tasks/timers
   WIN32_STREAM_THREAD,  // Handle to thread to do disk IO in the streaming API
   WIN32_HWAVEOUT,       // waveOut handle for HDIGDRIVER, if any
   WIN32_HWAVEIN,        // waveIn handle for HDIGINPUT, if any
   WIN32_LPDS,           // lpDirectSound pointer for HSAMPLE
   WIN32_LPDSB,          // lpDirectSoundBuffer pointer for HSAMPLE
   WIN32_HWND,           // HWND that will be used to open DirectSound driver
   WIN32_POSITION_ERR,   // Nonzero if DirectSound play cursor stops moving (e.g., headphones removed)

   PS3_TIMER_THREAD,     // Handle to thread to run background task/timer
   PS3_STREAM_THREAD,    // Handle to thread to do disk IO in the streaming API
   PS3_AUDIO_PORT,       // cellaudio port that Miles is using
   PS3_AUDIO_ADDRESS,    // address of cellaudio sound buffer
   PS3_AUDIO_LENGTH,     // length of cellaudio sound buffer
   PS3_AUDIO_POSITION,   // current playback position of cellaudio sound buffer

   XB_TIMER_THREAD,      // Handle to thread used to run background tasks/timers 
   XB_STREAM_THREAD,     // Handle to thread to do disk IO in the streaming API
   XB_LPDS,              // lpDirectSound pointer for HSAMPLE
   XB_LPDSB,             // lpDirectSoundBuffer pointer for HSAMPLE

   XB360_TIMER_THREAD,   // Handle to thread used to run mixer and all other background tasks/timers
   XB360_STREAM_THREAD,  // Handle to thread to do disk IO in the streaming API
   XB360_LPXAB           // IXAudioSourceVoice pointer for HDIGDRIVER
}
MSS_PLATFORM_PROPERTY;


typedef MSS_STRUCT _SMPATTRIBS
{
   float cone_inner_angle;
   float cone_outer_angle;
   float cone_outer_volume;
   float max_distance;
   float min_distance;
   int auto_3D_wet_atten;
   int is_3D;
   float X_pos;
   float Y_pos;
   float Z_pos;
   float X_face; 
   float Y_face; 
   float Z_face; 
   float X_up;   
   float Y_up;   
   float Z_up;
   float X_meters_per_ms;
   float Y_meters_per_ms;
   float Z_meters_per_ms;
   float channel_levels[MAX_SPEAKERS][MAX_SPEAKERS]; // [dest chans][src chans]
   int n_channel_levels;
   float exclusion;
   float obstruction;
   float occlusion;
   int loop_count;
   int loop_start_offset;
   int loop_end_offset;
   float LPF_cutoff;
   int playback_rate;
   UINT position;
   float dry_level;
   float wet_level;
   UINT playback_status;

   float FL_51_level;
   float FR_51_level;
   float BL_51_level;
   float BR_51_level;
   float C_51_level; 
   float LFE_51_level;

   float left_volume;     // Used in non-5.1 API modes
   float right_volume;

   SAMPLESTAGE filter_stages         [MAX_SAVED_FILTER_PROPERTIES];
   HPROVIDER   filter_providers      [MAX_SAVED_FILTER_PROPERTIES];
   UINT       filter_property_tokens[MAX_SAVED_FILTER_PROPERTIES];
   UINT         filter_property_values[MAX_SAVED_FILTER_PROPERTIES];
   int         n_filter_properties;
}
SMPATTRIBS;

typedef MSS_STRUCT _AIL_INPUT_INFO        // Input descriptor type
{
   UINT        device_ID;       // DS LPGUID or wave device ID
   UINT        hardware_format; // e.g., DIG_F_STEREO_16
   UINT        hardware_rate;   // e.g., 22050
   AILINCB    callback;        // Callback function to receive incoming data
   int        buffer_size;     // Maximum # of bytes to be passed to callback (-1 to use DIG_INPUT_LATENCY)
   UINT      user_data;       // this is a user defined value
}
AIL_INPUT_INFO;

typedef MSS_STRUCT _AILTIMER                 // Timer instance
{
   UINT      status;
   AILTIMERCB callback;
   UINT    user;

   int      elapsed;
   int      value;
   int      callingCT;           // Calling EXE's task number (16 bit only)
   int      callingDS;           // Calling EXE's DS (used in 16 bit only)
} AILTIMERSTR;






typedef union STAGE_BUFFER
{
  union STAGE_BUFFER * next;
  BYTE data[ 1 ];
} STAGE_BUFFER;





typedef MSS_STRUCT _SMPBUF           
{
   void const  *start;          // Sample buffer address (W)
   UINT             len;            // Sample buffer size in bytes (W)
   UINT             pos;            // Index to next byte (R/W)
   UINT             done;           // Nonzero if buffer with len=0 sent by app
   int             reset_ASI;      // Reset the ASI decoder at the end of the buffer
   int             reset_seek_pos; // New destination offset in stream source data, for ASI codecs that care
}
SMPBUF;



//
// used for AIL_process
//

typedef MSS_STRUCT _AILMIXINFO {
  AILSOUNDINFO Info;
  ADPCMDATA mss_adpcm;
  UINT src_fract;
  int left_val;
  int right_val;
} AILMIXINFO;


#if defined(IS_WINDOWS)

 UINT     __stdcall  AIL_get_timer_highest_delay   (void);

 void    __stdcall AIL_serve(void);



#define N_WAVEIN_BUFFERS 8     // Use a ring of 8 buffers by default

typedef MSS_STRUCT _DIG_INPUT_DRIVER  *HDIGINPUT; // Handle to digital input driver



#define AIL_DIGITAL_INPUT_DEFAULT ((UINT)WAVE_MAPPER)

typedef MSS_STRUCT _DIG_INPUT_DRIVER    // Handle to digital input driver
{
   char     tag[4];                   // HDIN

   HTIMER background_timer;         // Background timer handle

   AIL_INPUT_INFO info;             // Input device descriptor
   
   int       input_enabled;         // 1 if enabled, 0 if not

   UINT     callback_user;         // Callback user value

   //
   // Provider-independent data
   //

   UINT       DMA_size;              // Size of each DMA sub-buffer in bytes
   void  *DMA[N_WAVEIN_BUFFERS]; // Simulated DMA buffers

   BYTE        silence;               // Silence value for current format (0 or 128)

   int       device_active;         // 1 if buffers submittable, 0 if not

   //
   // waveOut-specific data
   //

   HWAVEIN          hWaveIn;                        // Handle to wave input device
   volatile MWAVEHDR wavehdr[N_WAVEIN_BUFFERS];     // Handles to wave headers

}
DIG_INPUT_DRIVER;

#endif












typedef MSS_STRUCT                      // MIDI status log structure
   {
   int      program   [NUM_CHANS];  // Program Change
   int      pitch_l   [NUM_CHANS];  // Pitch Bend LSB
   int      pitch_h   [NUM_CHANS];  // Pitch Bend MSB

   int      c_lock    [NUM_CHANS];  // Channel Lock
   int      c_prot    [NUM_CHANS];  // Channel Lock Protection
   int      c_mute    [NUM_CHANS];  // Channel Mute
   int      c_v_prot  [NUM_CHANS];  // Voice Protection
   int      bank      [NUM_CHANS];  // Patch Bank Select
   int      gm_bank_l [NUM_CHANS];  // GM Bank Select
   int      gm_bank_m [NUM_CHANS];  // GM Bank Select
   int      indirect  [NUM_CHANS];  // ICA indirect controller value
   int      callback  [NUM_CHANS];  // Callback Trigger

   int      mod       [NUM_CHANS];  // Modulation
   int      vol       [NUM_CHANS];  // Volume
   int      pan       [NUM_CHANS];  // Panpot
   int      exp       [NUM_CHANS];  // Expression
   int      sus       [NUM_CHANS];  // Sustain
   int      reverb    [NUM_CHANS];  // Reverb
   int      chorus    [NUM_CHANS];  // Chorus

   int      bend_range[NUM_CHANS];  // Bender Range (data MSB, RPN 0 assumed)

   int      RPN_L     [NUM_CHANS];  // RPN # LSB
   int      RPN_M     [NUM_CHANS];  // RPN # MSB
   }
CTRL_LOG;

typedef MSS_STRUCT _SEQUENCE                  // XMIDI sequence state table
{
   char     tag[4];                       // HSEQ

   HMDIDRIVER driver;                     // Driver for playback

   UINT      status;                       // SEQ_ flags

   void const    *TIMB;                // XMIDI IFF chunk pointers
   void const    *RBRN;
   void const    *EVNT;

   BYTE const      *EVNT_ptr;            // Current event pointer

   BYTE       *ICA;                      // Indirect Controller Array

   AILPREFIXCB   prefix_callback;         // XMIDI Callback Prefix handler
   AILTRIGGERCB  trigger_callback;        // XMIDI Callback Trigger handler
   AILBEATCB     beat_callback;           // XMIDI beat/bar change handler
   AILSEQUENCECB EOS;                     // End-of-sequence callback function

   int      loop_count;                   // 0=one-shot, -1=indefinite, ...

   int      interval_count;               // # of intervals until next event
   int      interval_num;                 // # of intervals since start

   int      volume;                       // Sequence volume 0-127
   int      volume_target;                // Target sequence volume 0-127
   int      volume_accum;                 // Accumulated volume period
   int      volume_period;                // Period for volume stepping

   int      tempo_percent;                // Relative tempo percentage 0-100
   int      tempo_target;                 // Target tempo 0-100
   int      tempo_accum;                  // Accumulated tempo period
   int      tempo_period;                 // Period for tempo stepping
   int      tempo_error;                  // Error counter for tempo DDA

   int      beat_count;                   // Sequence playback position
   int      measure_count;

   int      time_numerator;               // Sequence timing data
   int      time_fraction;
   int      beat_fraction;
   int      time_per_beat;

   void const  *FOR_ptrs[FOR_NEST];    // Loop stack
   int      FOR_loop_count [FOR_NEST];

   int      chan_map       [NUM_CHANS];   // Physical channel map for sequence

   CTRL_LOG shadow;                       // Controller values for sequence

   int      note_count;                   // # of notes "on"

   int      note_chan      [MAX_NOTES];   // Channel for queued note (-1=free)
   int      note_num       [MAX_NOTES];   // Note # for queued note
   int      note_time      [MAX_NOTES];   // Remaining duration in intervals

   int    user_data  [8];               // Miscellaneous user data
   int    system_data[8];               // Miscellaneous system data

} SEQUENCE;


typedef MSS_STRUCT _MDI_DRIVER          // Handle to XMIDI driver
{
   char     tag[4];                 // HMDI

   HTIMER      timer;               // XMIDI quantization timer
   int         interval_time;       // XMIDI quantization timer interval in uS

   int         disable;             // > 0 to disable XMIDI service

   HSEQUENCE   sequences;           // Pointer to list of SEQUENCEs
   int         n_sequences;         // # of SEQUENCEs

   int         lock  [NUM_CHANS];   // 1 if locked, 2 if protected, else 0
   HSEQUENCE   locker[NUM_CHANS];   // HSEQUENCE which locked channel
   HSEQUENCE   owner [NUM_CHANS];   // HSEQUENCE which owned locked channel
   HSEQUENCE   user  [NUM_CHANS];   // Last sequence to use channel
   int         state [NUM_CHANS];   // Lock state prior to being locked

   int         notes [NUM_CHANS];   // # of active notes in channel

   AILEVENTCB  event_trap;          // MIDI event trap callback function
   AILTIMBRECB timbre_trap;         // Timbre request callback function

   int         master_volume;       // Master XMIDI note volume 0-127

   int       system_data[8];      // Miscellaneous system data

#if defined(IS_WINDOWS)

   int         released;            // has the hmidiout handle been released
   UINT         deviceid;            // ID of the MIDI device
   BYTE       *sysdata;            // SysEx buffer

#endif


#ifdef IS_WINDOWS

   struct midihdr_tag  *mhdr;               // SysEx header

   HMDIDRIVER  next;                // Pointer to next HMDIDRIVER in use
   int      callingCT;              // Calling EXE's task number (16 bit only)
   int      callingDS;              // Calling EXE's DS (used in 16 bit only)

   HMIDIOUT    hMidiOut;            // MIDI output driver

#else


#endif

}
MDI_DRIVER;

typedef MSS_STRUCT                      // XMIDI TIMB IFF chunk
   {
   char    name[4];

   BYTE    msb;
   BYTE    lsb;
   BYTE    lsb2;
   BYTE    lsb3;

   WORD   n_entries;

   WORD   timbre[1];
   }
TIMB_chunk;

typedef MSS_STRUCT                      // XMIDI RBRN IFF entry
   {
   short   bnum;
   UINT   offset;
   }
RBRN_entry;

typedef MSS_STRUCT                      // Wave library entry
{
   int   bank;                      // XMIDI bank, MIDI patch for sample
   int   patch;

   int   root_key;                  // Root MIDI note # for sample (or -1)

   UINT   file_offset;               // Offset of wave data from start-of-file
   UINT   size;                      // Size of wave sample in bytes

   int   format;                    // DIG_F format (8/16 bits, mono/stereo)
   UINT   flags;                     // DIG_PCM_SIGN / DIG_PCM_ORDER (stereo)
   int   playback_rate;             // Playback rate in hertz
}
WAVE_ENTRY;

typedef MSS_STRUCT                      // Virtual "wave synthesizer" descriptor
{
   HMDIDRIVER mdi;                  // MIDI driver for use with synthesizer
   HDIGDRIVER dig;                  // Digital driver for use with synthesizer

   WAVE_ENTRY  *library;         // Pointer to wave library

   AILEVENTCB  prev_event_fn;       // Previous MIDI event trap function
   AILTIMBRECB prev_timb_fn;        // Previous timbre request trap function

   CTRL_LOG    controls;            // MIDI controller states

   WAVE_ENTRY  *wave [NUM_CHANS];// Pointer to WAVE_ENTRY for each channel

   HSAMPLE     S    [MAX_W_VOICES]; // List of HSAMPLE voices
   int         n_voices;            // Actual # of voices allocated to synth

   int         chan [MAX_W_VOICES]; // MIDI channel for each voice, or -1
   int         note [MAX_W_VOICES]; // MIDI note number for voice
   int         root [MAX_W_VOICES]; // MIDI root note for voice
   int         rate [MAX_W_VOICES]; // Playback rate for voice
   int         vel  [MAX_W_VOICES]; // MIDI note velocity for voice
   UINT         time [MAX_W_VOICES]; // Timestamp for voice

   UINT         event;               // Event counter for LRU timestamps
}
WAVE_SYNTH;

typedef WAVE_SYNTH  * HWAVESYNTH;// Handle to virtual wave synthesizer

//
// Handle to thread which called AIL_startup()
//
// This thread is suspended by MSS callback threads, to simulate DOS-style
// interrupt handler behavior
//

#ifdef IS_WIN32API

#ifdef NTAPI
extern HANDLE hAppThread;
#endif

#endif

//
// Background flag for timers
//

extern volatile int   AIL_bkgnd_flag;

//
// Global preference array
//

extern int          AIL_preference   [N_PREFS];

//
// DIG_DRIVER list
//

extern HDIGDRIVER     DIG_first;

//
// MDI_DRIVER list
//

extern HMDIDRIVER     MDI_first;

//
// Miscellaneous system services
//

#define FILE_READ_WITH_SIZE ((void *)(int)-1)

#ifndef NO_OLD_SYS_FUNCTIONS

#define MEM_alloc_lock AIL_mem_alloc_lock
#define MEM_free_lock  AIL_mem_free_lock
#define FILE_error     AIL_file_error
#define FILE_size      AIL_file_size
#define FILE_read      AIL_file_read
#define FILE_write     AIL_file_write


#endif

extern int __stdcall SP_ASI_DECODER_callback(UINT     user,
                                               void  *dest,
                                               int       bytes_requested,
                                               int       offset);

 void  * __stdcall AIL_mem_alloc_lock(UINT     size);
 void       __stdcall AIL_mem_free_lock (void  *ptr);

 int        __stdcall AIL_file_error   (void);

 int        __stdcall AIL_file_size    (char const   *filename);

 void  * __stdcall AIL_file_read    (char const    *filename,
                                           void  *dest);

 int        __stdcall AIL_file_write   (char const     *filename,
                                            void const   *buf,
                                            UINT       len);

 int        __stdcall AIL_WAV_file_write
                                           (char const     *filename,
                                            void const   *buf,
                                            UINT       len,
                                            int       rate,
                                            int       format);

 int        __stdcall AIL_file_append  (char const  *filename,
                                            void const  *buf, UINT len);


#ifdef ON_MAC_USE_FSS

 int        __stdcall AIL_file_fss_size(FSSpec const   *filename);

 void  * __stdcall AIL_file_fss_read(FSSpec const    *filename,
                                            void  *dest);

 int        __stdcall AIL_file_fss_write(FSSpec const     *filename,
                                             void const   *buf,
                                             UINT       len);

 int        __stdcall AIL_file_fss_attrib_write
                                            (FSSpec const     *filename,
                                             void const   *buf,
                                             UINT       len,
                                             UINT       type,
                                             UINT       creator );

 int        __stdcall AIL_WAV_file_fss_write
                                           (FSSpec const     *filename,
                                            void const   *buf,
                                            UINT       len,
                                            int       rate,
                                            int       format);

#endif



#ifdef __WATCOMC__

#pragma warning 14 10      // disable "no reference to symbol" warning on function args

void MSSBreakPoint();
#pragma aux MSSBreakPoint = "int 3";

#else
#ifdef IS_PS2
#else

#ifdef IS_WIN64
#define MSSBreakPoint() __debugbreak();
#else
#ifdef IS_WINDOWS
#define MSSBreakPoint() __asm {int 3}
#else
#define MSSBreakPoint() *(int*)0=0;
#endif


#endif
#endif
#endif

//
// Compiler-independent CRTL helper functions for PS2
// Exported here for use in demo programs as well as MSS itself
//

#ifdef IS_PS2
#else
                 
#ifdef IS_WATCOM
#define AIL_pow   powf
#define AIL_tan   tanf
#else
#define AIL_pow   pow
#define AIL_tan   tan
#endif

#define AIL_sin   sin
#define AIL_cos   cos
#define AIL_acos  acos
#define AIL_atan  atan
#define AIL_ceil  ceil
#define AIL_floor floor
#define AIL_fsqrt(arg) ((float) sqrt(arg))
#define AIL_fabs  fabs
#define AIL_log10 log10
#define AIL_log   log
#define AIL_frexpf(a1,a2) ((float) frexp(a1,a2))
#define AIL_ldexpf(a1,a2) ((float) ldexp(a1,a2))
#define AIL_exp   exp

#endif

DXDEC char * __stdcall AIL_ftoa(float val);

//
// High-level support services
//


//
// Process services
//

 HTIMER  __stdcall  AIL_register_timer            (AILTIMERCB  fn);

 UINT   __stdcall  AIL_set_timer_user            (HTIMER      timer,
                                                       UINT       user);

 void    __stdcall  AIL_set_timer_period          (HTIMER      timer,
                                                       UINT         microseconds);

 void    __stdcall  AIL_set_timer_frequency       (HTIMER      timer,
                                                       UINT         hertz);

 void    __stdcall  AIL_set_timer_divisor         (HTIMER      timer,
                                                       UINT         PIT_divisor);

 void    __stdcall  AIL_start_timer               (HTIMER      timer);
 void    __stdcall  AIL_start_all_timers          (void);

 void    __stdcall  AIL_stop_timer                (HTIMER      timer);
 void    __stdcall  AIL_stop_all_timers           (void);

 void    __stdcall  AIL_release_timer_handle      (HTIMER      timer);
 void    __stdcall  AIL_release_all_timers        (void);

#ifdef IS_WIN32



 HWND    __stdcall  AIL_HWND                      (void);

#else
#endif

//
// high-level digital services
//




DXDEC void __stdcall AIL_close_digital_driver( HDIGDRIVER dig );



#define AIL_MSS_version(str,len)        \
{                                       \
  HINSTANCE l=LoadLibrary(MSSDLLNAME);  \
  if ((UINT)l<=32)                     \
    *(str)=0;                           \
  else {                                \
    LoadString(l,1,str,len);            \
    FreeLibrary(l);                     \
  }                                     \
}

#if 0

 void         __stdcall AIL_waveOutClose          (HDIGDRIVER drvr);

 int          __stdcall AIL_digital_handle_release(HDIGDRIVER drvr);

 int          __stdcall AIL_digital_handle_reacquire
                                                  (HDIGDRIVER drvr);

DXDEC char *     __stdcall AIL_set_redist_directory(char const *dir);

 int          __stdcall AIL_background_CPU_percent (void);


 int          __stdcall AIL_digital_latency       (HDIGDRIVER dig);


DXDEC void          __stdcall AIL_set_speaker_configuration
                                                      (HDIGDRIVER       dig,
                                                       MSSVECTOR3D  *array,
                                                       int              n_channels,
                                                       float              falloff_power);

DXDEC MSSVECTOR3D  * 
                    __stdcall AIL_speaker_configuration
                                                      (HDIGDRIVER       dig,
                                                       int          *n_physical_channels,
                                                       int          *n_logical_channels,
                                                       float          *falloff_power,
                                                       MSS_MC_SPEC  *channel_spec);

DXDEC void          __stdcall AIL_set_listener_relative_receiver_array
                                                      (HDIGDRIVER             dig,
                                                       MSS_RECEIVER_LIST  *array,
                                                       int                    n_receivers);

DXDEC MSS_RECEIVER_LIST  * 
                    __stdcall AIL_listener_relative_receiver_array
                                                      (HDIGDRIVER dig,
                                                       int    *n_receivers);

 
int   AILEXPORT AIL_calculate_3D_channel_levels      (HDIGDRIVER                   dig, //)
                                                      float                      *channel_levels,
                                                      MSS_SPEAKER  const *  *speaker_array,
                                                      MSSVECTOR3D              *src_pos,
                                                      MSSVECTOR3D              *src_face,
                                                      MSSVECTOR3D              *src_up,
                                                      float                          src_inner_angle,
                                                      float                          src_outer_angle,
                                                      float                          src_outer_volume,
                                                      float                          src_max_dist,
                                                      float                          src_min_dist,
                                                      MSSVECTOR3D              *listen_pos, 
                                                      MSSVECTOR3D              *listen_face,
                                                      MSSVECTOR3D              *listen_up,  
                                                      float                          rolloff_factor,
                                                      MSSVECTOR3D              *doppler_velocity,
                                                      float                      *doppler_shift);
                                                      

 



 int          __stdcall AIL_set_sample_info       (HSAMPLE   S,
                                                       AILSOUNDINFO const  * info);

 int          __stdcall AIL_set_named_sample_file (HSAMPLE   S,
                                                       char const    *file_type_suffix,
                                                       void const  *file_image,
                                                       UINT       file_size,
                                                       int       block);

 HPROVIDER    __stdcall AIL_set_sample_processor  (HSAMPLE     S,
                                                       SAMPLESTAGE pipeline_stage,
                                                       HPROVIDER   provider);

 HPROVIDER    __stdcall AIL_set_digital_driver_processor
                                                      (HDIGDRIVER  dig,
                                                       DIGDRVSTAGE pipeline_stage,
                                                       HPROVIDER   provider);

 HPROVIDER    __stdcall AIL_sample_processor      (HSAMPLE     S,
                                                       SAMPLESTAGE pipeline_stage);

 HPROVIDER    __stdcall AIL_digital_driver_processor
                                                      (HDIGDRIVER  dig,
                                                       DIGDRVSTAGE pipeline_stage);

 void         __stdcall AIL_set_sample_adpcm_block_size
                                                     (HSAMPLE S,
                                                     UINT     blocksize);

 void         __stdcall AIL_set_sample_address    (HSAMPLE S,
                                                     void const    *start,
                                                     UINT     len);

 void         __stdcall AIL_start_sample          (HSAMPLE S);

 void         __stdcall AIL_stop_sample           (HSAMPLE S);

 void         __stdcall AIL_resume_sample         (HSAMPLE S);

 void         __stdcall AIL_end_sample            (HSAMPLE S);

 void         __stdcall AIL_set_sample_playback_rate
                                                     (HSAMPLE S,
                                                     int     playback_rate);


 void         __stdcall AIL_set_sample_volume_levels(HSAMPLE S,
                                                         float     left_level,
                                                         float     right_level);

 void         __stdcall AIL_set_sample_channel_levels (HSAMPLE        S,
                                                           MSS_SPEAKER  const *source_speaker_indexes,
                                                           MSS_SPEAKER  const *dest_speaker_indexes,
                                                           float          const *levels,
                                                           int                    n_levels);

 void         __stdcall AIL_set_sample_reverb_levels(HSAMPLE S,
                                                         float     dry_level,
                                                         float     wet_level);

 void         __stdcall AIL_set_sample_low_pass_cut_off(HSAMPLE S,
                                                            float     cut_off);

 void         __stdcall AIL_set_sample_loop_count (HSAMPLE S,
                                                       int     loop_count);

 void         __stdcall AIL_set_sample_loop_block (HSAMPLE S,
                                                       int     loop_start_offset,
                                                       int     loop_end_offset);

 int          __stdcall AIL_sample_loop_block     (HSAMPLE S,
                                                       int    *loop_start_offset,
                                                       int    *loop_end_offset);


 int          __stdcall AIL_sample_playback_rate  (HSAMPLE S);


 int          __stdcall AIL_sample_channel_count  (HSAMPLE S, UINT *mask);

 void         __stdcall AIL_sample_channel_levels (HSAMPLE                S,
                                                       MSS_SPEAKER  const *source_speaker_indexes,
                                                       MSS_SPEAKER  const *dest_speaker_indexes,
                                                       float                *levels,
                                                       int                    n_levels);

 void         __stdcall AIL_sample_volume_levels  (HSAMPLE  S,
                                                       float  *left_level,
                                                       float  *right_level);

 void         __stdcall AIL_sample_reverb_levels  (HSAMPLE  S,
                                                       float  *dry_level,
                                                       float  *wet_level);

 void         __stdcall AIL_sample_output_levels  (HSAMPLE                S,
                                                       MSS_SPEAKER  const *source_speaker_indexes,
                                                       MSS_SPEAKER  const *dest_speaker_indexes,
                                                       float                *levels,
                                                       int                    n_levels);

 float          __stdcall AIL_sample_low_pass_cut_off(HSAMPLE S);

 int          __stdcall AIL_sample_loop_count     (HSAMPLE S);

 void         __stdcall AIL_set_digital_master_volume_level
                                                     (HDIGDRIVER dig,
                                                      float        master_volume);

 float          __stdcall AIL_digital_master_volume_level (HDIGDRIVER dig);

 void         __stdcall AIL_set_sample_51_volume_pan( HSAMPLE S,
                                                          float     volume,
                                                          float     pan,
                                                          float     fb_pan,
                                                          float     center_level,
                                                          float     sub_level );

 void         __stdcall AIL_sample_51_volume_pan    ( HSAMPLE S,
                                                          float * volume,
                                                          float * pan,
                                                          float * fb_pan,
                                                          float * center_level,
                                                          float * sub_level );

 void         __stdcall AIL_set_sample_51_volume_levels( HSAMPLE S,
                                                             float     f_left_level,
                                                             float     f_right_level,
                                                             float     b_left_level,
                                                             float     b_right_level,
                                                             float     center_level,
                                                             float     sub_level );

 void         __stdcall AIL_sample_51_volume_levels    ( HSAMPLE S,
                                                             float * f_left_level,
                                                             float * f_right_level,
                                                             float * b_left_level,
                                                             float * b_right_level,
                                                             float * center_level,
                                                             float * sub_level );
#if !defined(IS_XBOX)

 void         __stdcall AIL_set_digital_master_reverb
                                                     (HDIGDRIVER dig,
                                                      float        reverb_decay_time,
                                                      float        reverb_predelay,
                                                      float        reverb_damping);

 void         __stdcall AIL_digital_master_reverb
                                                     (HDIGDRIVER dig,
                                                      float *   reverb_time,
                                                      float *   reverb_predelay,
                                                      float *   reverb_damping);

#endif

 void         __stdcall AIL_set_digital_master_reverb_levels
                                                     (HDIGDRIVER dig,
                                                      float        dry_level,
                                                      float        wet_level);

 void         __stdcall AIL_digital_master_reverb_levels
                                                     (HDIGDRIVER dig,
                                                      float  *  dry_level,
                                                      float  *  wet_level);


//
// low-level digital services
//

 int      __stdcall AIL_minimum_sample_buffer_size(HDIGDRIVER dig,
                                                     int         playback_rate,
                                                     int         format);

 int      __stdcall AIL_set_sample_buffer_count  (HSAMPLE S,
                                                      int     n_buffers);

 int      __stdcall AIL_sample_buffer_count      (HSAMPLE S);

 int      __stdcall AIL_sample_buffer_available (HSAMPLE S);

 int      __stdcall AIL_load_sample_buffer      (HSAMPLE S,
                                                     int     buff_num,
                                                     void const  *buffer,
                                                     UINT     len);

 void     __stdcall AIL_request_EOB_ASI_reset   (HSAMPLE S,
                                                     UINT     buff_num,
                                                     int     new_stream_position);

 int      __stdcall AIL_sample_buffer_info      (HSAMPLE S, //)
                                                     int     buff_num,
                                                     UINT      *pos,
                                                     UINT      *len,
                                                     int      *head,
                                                     int      *tail);

 UINT      __stdcall AIL_sample_granularity      (HSAMPLE S);

 void     __stdcall AIL_set_sample_position      (HSAMPLE S,
                                                      UINT     pos);

 UINT      __stdcall AIL_sample_position          (HSAMPLE S);

 AILSAMPLECB __stdcall AIL_register_SOB_callback
                                                    (HSAMPLE S,
                                                     AILSAMPLECB SOB);

 AILSAMPLECB __stdcall AIL_register_EOB_callback
                                                    (HSAMPLE S,
                                                     AILSAMPLECB EOB);


 AILFALLOFFCB __stdcall AIL_register_falloff_function_callback
                                                    (HSAMPLE S,
                                                     AILFALLOFFCB falloff_cb);

 void     __stdcall AIL_set_sample_user_data   (HSAMPLE S,
                                                    UINT     index,
                                                    int   value);

 int    __stdcall AIL_sample_user_data       (HSAMPLE S,
                                                    UINT     index);

 int      __stdcall AIL_active_sample_count    (HDIGDRIVER dig);


DXDEC int     __stdcall AIL_platform_property (void                 *object,
                                       MSS_PLATFORM_PROPERTY property,
                                       void              *before_value,
                                       void const        *new_value,
                                       void              *after_value);

#ifdef IS_WIN32API

 int      __stdcall AIL_set_direct_buffer_control (HSAMPLE S,
                                                     UINT      command);

 void     __stdcall AIL_get_DirectSound_info  (HSAMPLE              S,
                                                   AILLPDIRECTSOUND    *lplpDS,
                                                   AILLPDIRECTSOUNDBUFFER *lplpDSB);

#ifdef IS_WIN32
 int      __stdcall AIL_set_DirectSound_HWND(HDIGDRIVER dig, HWND wnd);
#endif 

#endif

 void     __stdcall AIL_set_sample_ms_position (HSAMPLE    S, //)
                                                    int        milliseconds);

 void     __stdcall AIL_sample_ms_position     (HSAMPLE    S, //)
                                                    int  *  total_milliseconds,
                                                    int  *  current_milliseconds);

//
// Digital input services
//

#if defined(IS_WINDOWS)

DXDEC HDIGINPUT __stdcall AIL_open_input             (AIL_INPUT_INFO  *info);

DXDEC void      __stdcall AIL_close_input            (HDIGINPUT         dig);

DXDEC AIL_INPUT_INFO  *
                __stdcall AIL_get_input_info         (HDIGINPUT         dig);

DXDEC int       __stdcall AIL_set_input_state        (HDIGINPUT         dig,
                                                    int               enable);
#endif


//
// High-level XMIDI services
//

DXDEC HMDIDRIVER __stdcall AIL_open_XMIDI_driver( UINT flags );

#define AIL_OPEN_XMIDI_NULL_DRIVER 1

DXDEC void __stdcall AIL_close_XMIDI_driver( HMDIDRIVER mdi );


#ifdef IS_WINDOWS

 int          __stdcall AIL_midiOutOpen(HMDIDRIVER  *drvr,
                                            LPHMIDIOUT  *lphMidiOut,
                                            int             dwDeviceID);

 void         __stdcall AIL_midiOutClose      (HMDIDRIVER mdi);

 int          __stdcall AIL_MIDI_handle_release
                                                 (HMDIDRIVER mdi);

 int          __stdcall AIL_MIDI_handle_reacquire
                                                 (HMDIDRIVER mdi);

#else


#endif

#endif

 HSEQUENCE    __stdcall AIL_allocate_sequence_handle
                                                     (HMDIDRIVER mdi);

 void         __stdcall AIL_release_sequence_handle
                                                     (HSEQUENCE S);

 int          __stdcall AIL_init_sequence         (HSEQUENCE S,
                                                     void const      *start,
                                                     int       sequence_num);

 void         __stdcall AIL_start_sequence        (HSEQUENCE S);

 void         __stdcall AIL_stop_sequence         (HSEQUENCE S);

 void         __stdcall AIL_resume_sequence       (HSEQUENCE S);

 void         __stdcall AIL_end_sequence          (HSEQUENCE S);

 void         __stdcall AIL_set_sequence_tempo    (HSEQUENCE S,
                                                       int       tempo,
                                                       int       milliseconds);

 void         __stdcall AIL_set_sequence_volume   (HSEQUENCE S,
                                                       int       volume,
                                                       int       milliseconds);

 void         __stdcall AIL_set_sequence_loop_count
                                                     (HSEQUENCE S,
                                                      int       loop_count);

 UINT          __stdcall AIL_sequence_status       (HSEQUENCE S);

 int          __stdcall AIL_sequence_tempo        (HSEQUENCE S);

 int          __stdcall AIL_sequence_volume       (HSEQUENCE S);

 int          __stdcall AIL_sequence_loop_count   (HSEQUENCE S);

 void         __stdcall AIL_set_XMIDI_master_volume
                                                     (HMDIDRIVER mdi,
                                                      int         master_volume);

 int          __stdcall AIL_XMIDI_master_volume   (HMDIDRIVER mdi);


//
// Low-level XMIDI services
//

 int      __stdcall AIL_active_sequence_count     (HMDIDRIVER mdi);

 int      __stdcall AIL_controller_value          (HSEQUENCE S,
                                                      int       channel,
                                                      int       controller_num);

 int      __stdcall AIL_channel_notes             (HSEQUENCE S,
                                                      int       channel);

 void     __stdcall AIL_sequence_position         (HSEQUENCE S,
                                                      int       *beat,
                                                      int       *measure);

 void     __stdcall AIL_branch_index              (HSEQUENCE  S,
                                                      UINT        marker);

 AILPREFIXCB __stdcall AIL_register_prefix_callback
                                                     (HSEQUENCE  S,
                                                      AILPREFIXCB callback);

 AILTRIGGERCB __stdcall AIL_register_trigger_callback
                                                     (HSEQUENCE  S,
                                                      AILTRIGGERCB callback);

 AILSEQUENCECB __stdcall AIL_register_sequence_callback
                                                     (HSEQUENCE  S,
                                                      AILSEQUENCECB callback);

 AILBEATCB __stdcall AIL_register_beat_callback   (HSEQUENCE  S,
                                                      AILBEATCB callback);

 AILEVENTCB __stdcall AIL_register_event_callback (HMDIDRIVER mdi,
                                                      AILEVENTCB callback);

 AILTIMBRECB __stdcall AIL_register_timbre_callback
                                                     (HMDIDRIVER mdi,
                                                      AILTIMBRECB callback);

 void     __stdcall AIL_set_sequence_user_data    (HSEQUENCE S,
                                                       UINT       index,
                                                       int     value);

 int    __stdcall AIL_sequence_user_data        (HSEQUENCE S,
                                                       UINT       index);

 void     __stdcall AIL_register_ICA_array        (HSEQUENCE S,
                                                      BYTE        *array);

 int      __stdcall AIL_lock_channel              (HMDIDRIVER mdi);

 void     __stdcall AIL_release_channel           (HMDIDRIVER mdi,
                                                      int         channel);

 void     __stdcall AIL_map_sequence_channel      (HSEQUENCE S,
                                                      int       seq_channel,
                                                      int       new_channel);

 int      __stdcall AIL_true_sequence_channel     (HSEQUENCE S,
                                                      int       seq_channel);

 void     __stdcall AIL_send_channel_voice_message
                                                     (HMDIDRIVER  mdi,
                                                      HSEQUENCE   S,
                                                      int         status,
                                                      int         data_1,
                                                      int         data_2);

 void     __stdcall AIL_send_sysex_message        (HMDIDRIVER mdi,
                                                       void const        *buffer);

 HWAVESYNTH
                __stdcall AIL_create_wave_synthesizer   (HDIGDRIVER dig,
                                                    HMDIDRIVER mdi,
                                                       void const        *wave_lib,
                                                       int         polyphony);

 void     __stdcall AIL_destroy_wave_synthesizer  (HWAVESYNTH W);

 void     __stdcall AIL_set_sequence_ms_position  (HSEQUENCE S, //)
                                                       int       milliseconds);

 void     __stdcall AIL_sequence_ms_position(HSEQUENCE S, //)
                                                 int  *total_milliseconds,
                                                 int  *current_milliseconds);



//
// red book functions
//

#if !defined(IS_XBOX) && !defined(IS_XENON) && !defined(IS_PS2) && !defined(IS_PS3)  && !defined(IS_WII)


typedef MSS_STRUCT _REDBOOK {
  UINT DeviceID;
  UINT paused;
  UINT pausedsec;
  UINT lastendsec;
} REDBOOK;

typedef MSS_STRUCT _REDBOOK * HREDBOOK;

#define REDBOOK_ERROR    0
#define REDBOOK_PLAYING  1
#define REDBOOK_PAUSED   2
#define REDBOOK_STOPPED  3


 HREDBOOK   __stdcall AIL_redbook_open(UINT which);

 HREDBOOK   __stdcall AIL_redbook_open_drive(int drive);

 void       __stdcall AIL_redbook_close(HREDBOOK hand);

 void       __stdcall AIL_redbook_eject(HREDBOOK hand);

 void       __stdcall AIL_redbook_retract(HREDBOOK hand);

 UINT        __stdcall AIL_redbook_status(HREDBOOK hand);

 UINT        __stdcall AIL_redbook_tracks(HREDBOOK hand);

 UINT        __stdcall AIL_redbook_track(HREDBOOK hand);

 void       __stdcall AIL_redbook_track_info(HREDBOOK hand,UINT tracknum,
                                                     UINT * startmsec,UINT * endmsec);

 UINT        __stdcall AIL_redbook_id(HREDBOOK hand);

 UINT        __stdcall AIL_redbook_position(HREDBOOK hand);

 UINT        __stdcall AIL_redbook_play(HREDBOOK hand,UINT startmsec, UINT endmsec);

 UINT        __stdcall AIL_redbook_stop(HREDBOOK hand);

 UINT        __stdcall AIL_redbook_pause(HREDBOOK hand);

 UINT        __stdcall AIL_redbook_resume(HREDBOOK hand);

 float        __stdcall AIL_redbook_volume_level(HREDBOOK hand);

 float        __stdcall AIL_redbook_set_volume_level(HREDBOOK hand, float volume);

#endif

  DXDEC UINT __stdcall AIL_ms_count(void);
  DXDEC UINT __stdcall AIL_us_count(void);






DXDEC 


DXDEC HSAMPLE  __stdcall AIL_stream_sample_handle(HSTREAM stream);

DXDEC int __stdcall AIL_service_stream(HSTREAM stream, int fillup);

DXDEC void __stdcall AIL_start_stream(HSTREAM stream);



DXDEC int __stdcall AIL_stream_loop_count(HSTREAM stream);

DXDEC void __stdcall AIL_set_stream_loop_count(HSTREAM stream, int count);

DXDEC void __stdcall AIL_set_stream_loop_block (HSTREAM S,
                                              int     loop_start_offset,
                                              int     loop_end_offset);

DXDEC int __stdcall AIL_stream_status(HSTREAM stream);

DXDEC void __stdcall AIL_set_stream_position(HSTREAM stream,int offset);

DXDEC int __stdcall AIL_stream_position(HSTREAM stream);

DXDEC void __stdcall AIL_stream_info(HSTREAM stream, int * datarate, int * sndtype, int * length, int * memory);

DXDEC AILSTREAMCB __stdcall AIL_register_stream_callback(HSTREAM stream, AILSTREAMCB callback);

DXDEC void __stdcall AIL_auto_service_stream(HSTREAM stream, int onoff);

DXDEC void     __stdcall AIL_set_stream_user_data   (HSTREAM S,
                                                   UINT     index,
                                                   int   value);

DXDEC int    __stdcall AIL_stream_user_data       (HSTREAM S,
                                                   UINT     index);



#ifdef ON_MAC_USE_FSS

typedef MSS_STRUCT MSS_FILE
{
  int file_type; // 0 = char*, 1 = FSSpec*
  void const * file;
} MSS_FILE;

#else

typedef char MSS_FILE;

#endif

typedef UINT  (__stdcall *AIL_file_open_callback)  (MSS_FILE const * Filename,
                                                        UINT * FileHandle);

typedef void (__stdcall *AIL_file_close_callback) (UINT FileHandle);

#define AIL_FILE_SEEK_BEGIN   0
#define AIL_FILE_SEEK_CURRENT 1
#define AIL_FILE_SEEK_END     2

typedef int  (__stdcall *AIL_file_seek_callback)  (UINT FileHandle,
                                                        int Offset,
                                                        UINT Type);

typedef UINT  (__stdcall *AIL_file_read_callback)  (UINT FileHandle,
                                                        void * Buffer,
                                                        UINT Bytes);

 void  __stdcall AIL_set_file_callbacks  (AIL_file_open_callback opencb,
                                              AIL_file_close_callback closecb,
                                              AIL_file_seek_callback seekcb,
                                              AIL_file_read_callback readcb);

#ifdef IS_32

typedef void * (__stdcall  *AIL_file_async_read_callback) (UINT FileHandle,
                                                                   void * Buffer,
                                                                   UINT Bytes);

typedef int (__stdcall *AIL_file_async_status_callback)  (void * async,
                                                               int wait,
                                                               UINT * BytesRead);

 void  __stdcall AIL_set_file_async_callbacks (AIL_file_open_callback opencb,
                                                   AIL_file_close_callback closecb,
                                                   AIL_file_seek_callback seekcb,
                                                   AIL_file_async_read_callback areadcb,
                                                   AIL_file_async_status_callback statuscb);

#endif

//
// High-level DLS functions
//

typedef MSS_STRUCT _DLSFILEID {
  int id;
  struct _DLSFILEID * next;
} DLSFILEID;

typedef MSS_STRUCT _DLSFILEID * HDLSFILEID;

typedef MSS_STRUCT _DLSDEVICE {
  VOIDFUNC * pGetPref;
  VOIDFUNC * pSetPref;
  VOIDFUNC * pMSSOpen;
  VOIDFUNC * pOpen;
  VOIDFUNC * pClose;
  VOIDFUNC * pLoadFile;
  VOIDFUNC * pLoadMem;
  VOIDFUNC * pUnloadFile;
  VOIDFUNC * pUnloadAll;
  VOIDFUNC * pGetInfo;
  VOIDFUNC * pCompact;
  VOIDFUNC * pSetAttr;
  int DLSHandle;
  UINT format;
  UINT buffer_size;
  void * buffer[2];
  HSAMPLE sample;
  HMDIDRIVER mdi;
  HDIGDRIVER dig;
  HDLSFILEID first;
  HMODULE lib;
} DLSDEVICE;

typedef MSS_STRUCT _DLSDEVICE * HDLSDEVICE;

typedef MSS_STRUCT _AILDLSINFO {
  char Description[128];
  int MaxDLSMemory;
  int CurrentDLSMemory;
  int LargestSize;
  int GMAvailable;
  int GMBankSize;
} AILDLSINFO;


 HDLSDEVICE __stdcall AIL_DLS_open(HMDIDRIVER mdi, HDIGDRIVER dig,
                                          char const  * libname,
                                          UINT flags, UINT rate, int bits, int channels);

//
// Parameters for the dwFlag used in DLSClose() and flags in AIL_DLS_close
//

#define RETAIN_DLS_COLLECTION   0x00000001
#define RETURN_TO_BOOTUP_STATE  0x00000002
#define RETURN_TO_GM_ONLY_STATE 0x00000004
#define DLS_COMPACT_MEMORY      0x00000008

 void   __stdcall AIL_DLS_close(HDLSDEVICE dls, UINT flags);

 HDLSFILEID __stdcall AIL_DLS_load_file(HDLSDEVICE dls, char const * filename, UINT flags);

 HDLSFILEID __stdcall AIL_DLS_load_memory(HDLSDEVICE dls, void const * memfile, UINT flags);

//
// other parameters for AIL_DLS_unload
//

#define AIL_DLS_UNLOAD_MINE 0
#define AIL_DLS_UNLOAD_ALL  ((HDLSFILEID)(UINT)(int)-1)

 void   __stdcall AIL_DLS_unload(HDLSDEVICE dls, HDLSFILEID dlsid);

 void   __stdcall AIL_DLS_compact(HDLSDEVICE dls);

 void   __stdcall AIL_DLS_get_info(HDLSDEVICE dls, AILDLSINFO * info, int * PercentCPU);

DXDEC HSAMPLE __stdcall AIL_DLS_sample_handle(HDLSDEVICE dls);

//
// Bankfile management
//

typedef MSS_STRUCT _SOUNDLIB
{
   struct _SOUNDLIB *next;

   struct _BANKFILE *B;
}
SOUNDLIB;

typedef MSS_STRUCT _SOUNDLIB  *HSOUNDLIB;

DXDEC void AILEXPORT AIL_close_library(HSOUNDLIB L);

DXDEC HSOUNDLIB AILEXPORT AIL_open_library(const char *filename,          
                                           int       embedded_at_offset);

DXDEC char * __stdcall AIL_library_resource_filename(HSOUNDLIB     lib,  
                                                 char const  *resource_name,
                                                 char           *filename_buffer,
                                                 int           filename_buffer_bytes);

//
// Quick-integration service functions and data types
//

typedef MSS_STRUCT
{
   UINT const  *data;
   int  size;
   int  type;
   void  *handle;
   int  status;
   void * next;
   int  speed;
   float  volume;
   float  extravol;
   float  dry;
   float  wet;
   float  cutoff;
   HDLSFILEID dlsid;
   void * dlsmem;
   void * dlsmemunc;
   int  milliseconds;
   int  length;
   int userdata;
}
AUDIO_TYPE;


#define QSTAT_DONE          1       // Data has finished playing
#define QSTAT_LOADED        2       // Data has been loaded, but not yet played
#define QSTAT_PLAYING       3       // Data is currently playing

typedef AUDIO_TYPE  * HAUDIO;        // Generic handle to any audio data type

#define AIL_QUICK_USE_WAVEOUT             2
#define AIL_QUICK_MIDI_AND_DLS            2
#define AIL_QUICK_DLS_ONLY                3
#define AIL_QUICK_MIDI_AND_VORTEX_DLS     4
#define AIL_QUICK_MIDI_AND_SONICVIBES_DLS 5

DXDEC int     __stdcall
#if defined(IS_WINDOWS)
                       AIL_quick_startup           (
#else
                       AIL_quick_startup_with_start(void*       startup,
#endif
                                                    int         use_digital,
                                                    int         use_MIDI,
                                                    UINT         output_rate,
                                                    int         output_bits,
                                                    int         output_channels);

DXDEC void    __stdcall AIL_quick_shutdown            (void);

DXDEC void    __stdcall AIL_quick_handles             (HDIGDRIVER * pdig,
                                                     HMDIDRIVER * pmdi,
                                                     HDLSDEVICE * pdls );

DXDEC HAUDIO  __stdcall AIL_quick_load                (char const    *filename);

#ifdef ON_MAC_USE_FSS
DXDEC HAUDIO  __stdcall AIL_quick_fss_load            (FSSpec const    *filename);
#endif

DXDEC HAUDIO  __stdcall AIL_quick_load_mem            (void const    *mem,
                                                     UINT    size);

DXDEC HAUDIO  __stdcall AIL_quick_load_named_mem      (void const    *mem,
                                                     char const    *filename,
                                                     UINT    size);

DXDEC HAUDIO  __stdcall AIL_quick_copy                (HAUDIO      audio);

DXDEC void    __stdcall AIL_quick_unload              (HAUDIO      audio);

DXDEC int     __stdcall AIL_quick_play                (HAUDIO      audio,
                                                    UINT         loop_count);

DXDEC void    __stdcall AIL_quick_halt                (HAUDIO      audio);

DXDEC int     __stdcall AIL_quick_status              (HAUDIO      audio);

DXDEC HAUDIO  __stdcall AIL_quick_load_and_play       (char const    *filename,
                                                    UINT         loop_count,
                                                    int         wait_request);

#ifdef ON_MAC_USE_FSS
DXDEC HAUDIO  __stdcall AIL_quick_fss_load_and_play  (FSSpec const    *filename,
                                                    UINT         loop_count,
                                                    int         wait_request);
#endif

DXDEC void   __stdcall AIL_quick_set_speed (HAUDIO audio, int speed);

DXDEC void   __stdcall AIL_quick_set_volume (HAUDIO audio, float volume, float extravol);

DXDEC void   __stdcall AIL_quick_set_reverb_levels (HAUDIO audio,
                                                  float    dry_level,
                                                  float    wet_level);

DXDEC void   __stdcall AIL_quick_set_low_pass_cut_off(HAUDIO S,
                                                    float cut_off);

DXDEC void   __stdcall AIL_quick_set_ms_position(HAUDIO audio,int milliseconds);

DXDEC int    __stdcall AIL_quick_ms_position(HAUDIO audio);

DXDEC int    __stdcall AIL_quick_ms_length(HAUDIO audio);


#define AIL_QUICK_XMIDI_TYPE        1
#define AIL_QUICK_DIGITAL_TYPE      2
#define AIL_QUICK_DLS_XMIDI_TYPE    3
#define AIL_QUICK_MPEG_DIGITAL_TYPE 4
#define AIL_QUICK_OGG_VORBIS_TYPE   5
#define AIL_QUICK_V12_VOICE_TYPE    6
#define AIL_QUICK_V24_VOICE_TYPE    7
#define AIL_QUICK_V29_VOICE_TYPE    8
#define AIL_QUICK_OGG_SPEEX_TYPE    9
#define AIL_QUICK_S8_VOICE_TYPE     10
#define AIL_QUICK_S16_VOICE_TYPE    11
#define AIL_QUICK_S32_VOICE_TYPE    12

DXDEC int    __stdcall AIL_quick_type(HAUDIO audio);

DXDEC void __stdcall AIL_save_sample_attributes(HSAMPLE   S,
                                              HSATTRIBS D);

DXDEC UINT __stdcall AIL_load_sample_attributes(HSAMPLE   S,
                                             HSATTRIBS D);



DXDEC int __stdcall AIL_size_processed_digital_audio(
                                 UINT             dest_rate,
                                 UINT             dest_format,
                                 int             num_srcs,
                                 AILMIXINFO const  * src);

DXDEC int __stdcall AIL_process_digital_audio(
                                 void        *dest_buffer,
                                 int             dest_buffer_size,
                                 UINT             dest_rate,
                                 UINT             dest_format,
                                 int             num_srcs,
                                 AILMIXINFO * src);

#define AIL_LENGTHY_INIT           0
#define AIL_LENGTHY_SET_PROPERTY   1
#define AIL_LENGTHY_UPDATE         2
#define AIL_LENGTHY_DONE           3



typedef int (__stdcall * AILCODECSETPROP)(char const * property,void const  * value);

DXDEC int __stdcall AIL_compress_ASI(AILSOUNDINFO const  * info, //)
                                   char const * filename_ext,
                                   void * * outdata,
                                   UINT * outsize,
                                   AILLENGTHYCB callback);



DXDEC int __stdcall AIL_compress_ADPCM(AILSOUNDINFO const  * info,
                                     void * * outdata, UINT * outsize);

DXDEC int __stdcall AIL_decompress_ADPCM(AILSOUNDINFO const  * info,
                                       void * * outdata, UINT * outsize);

DXDEC int __stdcall AIL_compress_DLS(void const * dls,
                                   char const * compression_extension,
                                   void * * mls, UINT * mlssize,
                                   AILLENGTHYCB callback);

DXDEC int __stdcall AIL_merge_DLS_with_XMI(void const * xmi, void const * dls,
                                         void * * mss, UINT * msssize);

 int __stdcall AIL_extract_DLS( void const  *source_image, //)
                                    UINT             source_size,
                                    void  *  *XMI_output_data,
                                    UINT         *XMI_output_size,
                                    void  *  *DLS_output_data,
                                    UINT         *DLS_output_size,
                                    AILLENGTHYCB   callback);

#define AILFILTERDLS_USINGLIST 1

DXDEC int __stdcall AIL_filter_DLS_with_XMI(void const * xmi, void const * dls,
                                         void * * dlsout, UINT * dlssize,
                                         int  flags, AILLENGTHYCB callback);

#define AILMIDITOXMI_USINGLIST 1
#define AILMIDITOXMI_TOLERANT  2

 int __stdcall AIL_MIDI_to_XMI       (void const *  MIDI,
                                         UINT        MIDI_size,
                                         void *  *XMIDI,
                                         UINT  *  XMIDI_size,
                                         int        flags);

#define AILDLSLIST_ARTICULATION 1
#define AILDLSLIST_DUMP_WAVS    2

#if defined(IS_WIN32)

 int          __stdcall AIL_list_DLS          (void const * DLS,
                                                   char * * lst,
                                                   UINT  * lst_size,
                                                   int       flags,
                                                   char   * title);

#define AILMIDILIST_ROLANDSYSEX 1
#define AILMIDILIST_ROLANDUN    2
#define AILMIDILIST_ROLANDAB    4

 int          __stdcall AIL_list_MIDI         (void const * MIDI,
                                                  UINT       MIDI_size,
                                                  char * * lst,
                                                  UINT  * lst_size,
                                                  int       flags);
#endif


DXDEC int __stdcall AIL_file_type_named(void const * data, char const * filename, UINT size);

DXDEC int __stdcall AIL_find_DLS       (void const *      data, UINT size,
                                      void * * xmi, UINT * xmisize,
                                      void * * dls, UINT * dlssize);
typedef MSS_STRUCT
{
   //
   // File-level data accessible to app
   //
   // This is valid after AIL_inspect_MP3() is called (even if the file contains no valid frames)
   //

   BYTE  *MP3_file_image;       // Original MP3_file_image pointer passed to AIL_inspect_MP3()
   int     MP3_image_size;       // Original MP3_image_size passed to AIL_inspect_MP3()

   BYTE  *ID3v2;                // ID3v2 tag, if not NULL
   int     ID3v2_size;           // Size of tag in bytes

   BYTE  *ID3v1;                // ID3v1 tag, if not NULL (always 128 bytes long if present)

   BYTE  *start_MP3_data;       // Pointer to start of data area in file (not necessarily first valid frame)
   BYTE  *end_MP3_data;         // Pointer to last valid byte in MP3 data area (before ID3v1 tag, if any)

   //
   // Information about current frame being inspected, valid if AIL_enumerate_MP3_frames() returns 
   // TRUE
   //

   int sample_rate;              // Sample rate in Hz (normally constant across all frames in file)
   int bit_rate;                 // Bits/second for current frame
   int channels_per_sample;      // 1 or 2
   int samples_per_frame;        // Always 576 or 1152 samples in each MP3 frame, depending on rate

   int byte_offset;              // Offset of frame from start_MP3_data (i.e., suitable for use as loop point)
   int next_frame_expected;      // Anticipated offset of next frame to be enumerated, if any
   int average_frame_size;       // Average source bytes per frame, determined solely by bit rate and sample rate
   int data_size;                // # of data-only bytes in this particular frame
   int header_size;              // 4 or 6 bytes, depending on CRC
   int side_info_size;           // Valid for layer 3 side info only
   int ngr;                      // Always 2 for MPEG1, else 1
   int main_data_begin;          // Always 0 in files with no bit reservoir
   int hpos;                     // Current bit position in header/side buffer

   int MPEG1;                    // Data copied directly from frame header, see ISO docs for info... 
   int MPEG25;                   
   int layer;                    
   int protection_bit;
   int bitrate_index;
   int sampling_frequency;
   int padding_bit;
   int private_bit;
   int mode;
   int mode_extension;
   int copyright;
   int original;
   int emphasis;

   //
   // LAME/Xing info tag data
   //

   int Xing_valid;
   int Info_valid;
   UINT header_flags;
   int frame_count;
   int byte_count;
   int VBR_scale;
   BYTE  TOC[100];
   int enc_delay;
   int enc_padding;

   //
   // Private (undocumented) data used during frame enumeration
   //

   BYTE  *ptr;
   int bytes_left;

   int check_valid;
   int check_MPEG1;
   int check_MPEG25;
   int check_layer;
   int check_protection_bit;
   int check_sampling_frequency;
   int check_mode;
   int check_copyright;
   int check_original;
}
MP3_INFO;

DXDEC void __stdcall AIL_inspect_MP3 (MP3_INFO  *inspection_state,
                                    BYTE        *MP3_file_image,
                                    int           MP3_image_size);

DXDEC int __stdcall AIL_enumerate_MP3_frames (MP3_INFO  *inspection_state);

//
// RAD room types - currently the same as EAX
//

enum
{
    ENVIRONMENT_GENERIC,                // factory default
    ENVIRONMENT_PADDEDCELL,
    ENVIRONMENT_ROOM,                   // standard environments
    ENVIRONMENT_BATHROOM,
    ENVIRONMENT_LIVINGROOM,
    ENVIRONMENT_STONEROOM,
    ENVIRONMENT_AUDITORIUM,
    ENVIRONMENT_CONCERTHALL,
    ENVIRONMENT_CAVE,
    ENVIRONMENT_ARENA,
    ENVIRONMENT_HANGAR,
    ENVIRONMENT_CARPETEDHALLWAY,
    ENVIRONMENT_HALLWAY,
    ENVIRONMENT_STONECORRIDOR,
    ENVIRONMENT_ALLEY,
    ENVIRONMENT_FOREST,
    ENVIRONMENT_CITY,
    ENVIRONMENT_MOUNTAINS,
    ENVIRONMENT_QUARRY,
    ENVIRONMENT_PLAIN,
    ENVIRONMENT_PARKINGLOT,
    ENVIRONMENT_SEWERPIPE,
    ENVIRONMENT_UNDERWATER,
    ENVIRONMENT_DRUGGED,
    ENVIRONMENT_DIZZY,
    ENVIRONMENT_PSYCHOTIC,

    ENVIRONMENT_COUNT           // total number of environments
};

//
// enumerated values for EAX
//

#ifndef EAX_H_INCLUDED

enum
{
    EAX_ENVIRONMENT_GENERIC,                // factory default
    EAX_ENVIRONMENT_PADDEDCELL,
    EAX_ENVIRONMENT_ROOM,              // standard environments
    EAX_ENVIRONMENT_BATHROOM,
    EAX_ENVIRONMENT_LIVINGROOM,
    EAX_ENVIRONMENT_STONEROOM,
    EAX_ENVIRONMENT_AUDITORIUM,
    EAX_ENVIRONMENT_CONCERTHALL,
    EAX_ENVIRONMENT_CAVE,
    EAX_ENVIRONMENT_ARENA,
    EAX_ENVIRONMENT_HANGAR,
    EAX_ENVIRONMENT_CARPETEDHALLWAY,
    EAX_ENVIRONMENT_HALLWAY,
    EAX_ENVIRONMENT_STONECORRIDOR,
    EAX_ENVIRONMENT_ALLEY,
    EAX_ENVIRONMENT_FOREST,
    EAX_ENVIRONMENT_CITY,
    EAX_ENVIRONMENT_MOUNTAINS,
    EAX_ENVIRONMENT_QUARRY,
    EAX_ENVIRONMENT_PLAIN,
    EAX_ENVIRONMENT_PARKINGLOT,
    EAX_ENVIRONMENT_SEWERPIPE,
    EAX_ENVIRONMENT_UNDERWATER,
    EAX_ENVIRONMENT_DRUGGED,
    EAX_ENVIRONMENT_DIZZY,
    EAX_ENVIRONMENT_PSYCHOTIC,

    EAX_ENVIRONMENT_COUNT           // total number of environments
};

#define EAX_REVERBMIX_USEDISTANCE (-1.0F)

#endif

#define MSS_BUFFER_HEAD (-1)

#if defined(IS_WIN32API)

//
// Auxiliary 2D interface calls
//

DXDEC HDIGDRIVER __stdcall AIL_primary_digital_driver  (HDIGDRIVER new_primary);

//
// 3D-related calls
//

 int      __stdcall AIL_room_type                (HDIGDRIVER dig);

 void     __stdcall AIL_set_room_type            (HDIGDRIVER dig,
                                                      int        room_type);

 float      __stdcall AIL_3D_rolloff_factor        (HDIGDRIVER dig);

 void     __stdcall AIL_set_3D_rolloff_factor    (HDIGDRIVER dig,
                                                      float       factor);

 float      __stdcall AIL_3D_doppler_factor        (HDIGDRIVER dig);

 void     __stdcall AIL_set_3D_doppler_factor    (HDIGDRIVER dig,
                                                      float       factor);

 float      __stdcall AIL_3D_distance_factor       (HDIGDRIVER dig);

 void     __stdcall AIL_set_3D_distance_factor   (HDIGDRIVER dig,
                                                      float       factor);

DXDEC void       __stdcall AIL_set_sample_obstruction  (HSAMPLE S,
                                                      float     obstruction);

DXDEC void       __stdcall AIL_set_sample_occlusion    (HSAMPLE S,
                                                      float     occlusion);

DXDEC void       __stdcall AIL_set_sample_exclusion    (HSAMPLE S,
                                                      float     exclusion);

DXDEC float        __stdcall AIL_sample_obstruction      (HSAMPLE S);

DXDEC float        __stdcall AIL_sample_occlusion        (HSAMPLE S);

DXDEC float        __stdcall AIL_sample_exclusion        (HSAMPLE S);

DXDEC void       __stdcall AIL_set_sample_3D_distances (HSAMPLE S,
                                                      float     max_dist,
                                                      float     min_dist,
                                                      int     auto_3D_wet_atten);


DXDEC void       __stdcall AIL_sample_3D_distances     (HSAMPLE S,
                                                      float  * max_dist,
                                                      float  * min_dist,
                                                      int  * auto_3D_wet_atten);

DXDEC void       __stdcall AIL_set_sample_3D_cone        (HSAMPLE S,
                                                        float     inner_angle,
                                                        float     outer_angle,
                                                        float     outer_volume_level);

DXDEC void       __stdcall AIL_sample_3D_cone            (HSAMPLE S,
                                                        float *  inner_angle,
                                                        float *  outer_angle,
                                                        float *  outer_volume_level);

DXDEC void       __stdcall AIL_set_sample_3D_position    (HSAMPLE obj,
                                                        float     X,
                                                        float     Y,
                                                        float     Z);

DXDEC void       __stdcall AIL_set_sample_3D_velocity    (HSAMPLE obj,
                                                        float     dX_per_ms,
                                                        float     dY_per_ms,
                                                        float     dZ_per_ms,
                                                        float     magnitude);

DXDEC void       __stdcall AIL_set_sample_3D_velocity_vector  (HSAMPLE obj,
                                                             float     dX_per_ms,
                                                             float     dY_per_ms,
                                                             float     dZ_per_ms);

DXDEC void       __stdcall AIL_set_sample_3D_orientation      (HSAMPLE obj,
                                                             float     X_face,
                                                             float     Y_face,
                                                             float     Z_face,
                                                             float     X_up,
                                                             float     Y_up,
                                                             float     Z_up);

DXDEC int        __stdcall AIL_sample_3D_position             (HSAMPLE  obj,
                                                             float  *X,
                                                             float  *Y,
                                                             float  *Z);

DXDEC void       __stdcall AIL_sample_3D_velocity             (HSAMPLE  obj,
                                                             float  *dX_per_ms,
                                                             float  *dY_per_ms,
                                                             float  *dZ_per_ms);

DXDEC void       __stdcall AIL_sample_3D_orientation          (HSAMPLE  obj,
                                                             float  *X_face,
                                                             float  *Y_face,
                                                             float  *Z_face,
                                                             float  *X_up,
                                                             float  *Y_up,
                                                             float  *Z_up);

DXDEC void       __stdcall AIL_update_sample_3D_position      (HSAMPLE obj,
                                                             float     dt_milliseconds);

DXDEC void       __stdcall AIL_set_listener_3D_position         (HDIGDRIVER dig,
                                                               float     X,
                                                               float     Y,
                                                               float     Z);

DXDEC void       __stdcall AIL_set_listener_3D_velocity         (HDIGDRIVER dig,
                                                               float     dX_per_ms,
                                                               float     dY_per_ms,
                                                               float     dZ_per_ms,
                                                               float     magnitude);

DXDEC void       __stdcall AIL_set_listener_3D_velocity_vector  (HDIGDRIVER dig,
                                                               float     dX_per_ms,
                                                               float     dY_per_ms,
                                                               float     dZ_per_ms);

DXDEC void       __stdcall AIL_set_listener_3D_orientation      (HDIGDRIVER dig,
                                                               float     X_face,
                                                               float     Y_face,
                                                               float     Z_face,
                                                               float     X_up,
                                                               float     Y_up,
                                                               float     Z_up);
                                                               
DXDEC void       __stdcall AIL_listener_3D_position             (HDIGDRIVER  dig,
                                                               float  *X,
                                                               float  *Y,
                                                               float  *Z);

DXDEC void       __stdcall AIL_listener_3D_velocity             (HDIGDRIVER  dig,
                                                               float  *dX_per_ms,
                                                               float  *dY_per_ms,
                                                               float  *dZ_per_ms);

DXDEC void       __stdcall AIL_listener_3D_orientation          (HDIGDRIVER  dig,
                                                               float  *X_face,
                                                               float  *Y_face,
                                                               float  *Z_face,
                                                               float  *X_up,
                                                               float  *Y_up,
                                                               float  *Z_up);

DXDEC void       __stdcall AIL_update_listener_3D_position      (HDIGDRIVER dig,
                                                               float     dt_milliseconds);

#endif

#ifdef MSS_FLT_SUPPORTED

//
// Filter result codes
//

typedef int FLTRESULT;

#define FLT_NOERR                   0   // Success -- no error
#define FLT_NOT_ENABLED             1   // FLT not enabled
#define FLT_ALREADY_STARTED         2   // FLT already started
#define FLT_INVALID_PARAM           3   // Invalid parameters used
#define FLT_INTERNAL_ERR            4   // Internal error in FLT driver
#define FLT_OUT_OF_MEM              5   // Out of system RAM
#define FLT_ERR_NOT_IMPLEMENTED     6   // Feature not implemented
#define FLT_NOT_FOUND               7   // FLT supported device not found
#define FLT_NOT_INIT                8   // FLT not initialized
#define FLT_CLOSE_ERR               9   // FLT not closed correctly

//############################################################################
//##                                                                        ##
//## Interface "MSS pipeline filter" (some functions shared by              ##
//## "MSS voice filter")                                                    ##
//##                                                                        ##
//############################################################################

typedef FLTRESULT (__stdcall  *FLT_STARTUP)(void);

typedef FLTRESULT (__stdcall  *FLT_SHUTDOWN)(void);

typedef char  *  (__stdcall  *FLT_ERROR)(void);

typedef HDRIVERSTATE (__stdcall  *FLT_OPEN_DRIVER) (HDIGDRIVER dig);

typedef FLTRESULT    (__stdcall  *FLT_CLOSE_DRIVER) (HDRIVERSTATE state);

typedef void         (__stdcall  *FLT_PREMIX_PROCESS) (HDRIVERSTATE driver);

typedef int          (__stdcall  *FLT_POSTMIX_PROCESS) (HDRIVERSTATE driver, void  *output_buffer);

//############################################################################
//##                                                                        ##
//## Interface "Pipeline filter sample services"                            ##
//##                                                                        ##
//############################################################################

typedef HSAMPLESTATE (__stdcall  * FLTSMP_OPEN_SAMPLE) (HDRIVERSTATE driver,
                                                         HSAMPLE      S);

typedef FLTRESULT    (__stdcall  * FLTSMP_CLOSE_SAMPLE) (HSAMPLESTATE state);

typedef void         (__stdcall  * FLTSMP_SAMPLE_PROCESS) (HSAMPLESTATE    state,
                                                            void  *      source_buffer,
                                                            void  *      dest_buffer, // may be the same as src
                                                            int             n_samples,
                                                            int             playback_rate,
                                                            int             is_stereo );

typedef int          (__stdcall  * FLTSMP_SAMPLE_PROPERTY) (HSAMPLESTATE    state,
                                                             HPROPERTY       property,
                                                             void *       before_value,
                                                             void const * new_value,
                                                             void *       after_value
                                                             );

//############################################################################
//##                                                                        ##
//## Interface "MSS output filter"                                          ##
//##                                                                        ##
//############################################################################

typedef int (__stdcall  * VFLT_ASSIGN_SAMPLE_VOICE) (HDRIVERSTATE driver,
                                                      HSAMPLE      S);

typedef void (__stdcall  * VFLT_RELEASE_SAMPLE_VOICE) (HDRIVERSTATE driver,
                                                        HSAMPLE      S);

typedef int (__stdcall  * VFLT_START_SAMPLE_VOICE) (HDRIVERSTATE driver,
                                                     HSAMPLE      S);

//############################################################################
//##                                                                        ##
//## Interface "Voice filter driver services"                               ##
//##                                                                        ##
//############################################################################

typedef int          (__stdcall  * VDRV_DRIVER_PROPERTY) (HDRIVERSTATE    driver,
                                                           HPROPERTY       property,
                                                           void *       before_value,
                                                           void const * new_value,
                                                           void *       after_value
                                                           );

typedef int          (__stdcall  * VDRV_FORCE_UPDATE)     (HDRIVERSTATE driver);

//############################################################################
//##                                                                        ##
//## Interface "Voice filter sample services"                               ##
//##                                                                        ##
//############################################################################

typedef int          (__stdcall  * VSMP_SAMPLE_PROPERTY) (HSAMPLE      S,
                                                           HPROPERTY       property,
                                                           void *       before_value,
                                                           void const * new_value,
                                                           void *       after_value
                                                           );

//
// Pipeline filter calls
//

DXDEC HPROVIDER  __stdcall AIL_digital_output_filter (HDIGDRIVER dig);

DXDEC int        __stdcall AIL_enumerate_filters  (HPROENUM   *next,
                                                 HPROVIDER  *dest,
                                                 char   *  *name);
DXDEC HDRIVERSTATE
                 __stdcall AIL_open_filter        (HPROVIDER  lib,
                                                 HDIGDRIVER dig);

DXDEC void       __stdcall AIL_close_filter       (HDRIVERSTATE filter);

DXDEC int        __stdcall AIL_find_filter        (char const  *name, 
                                                 HPROVIDER *ret);

DXDEC int        __stdcall AIL_enumerate_filter_properties
                                                (HPROVIDER                  lib,
                                                 HINTENUM  *             next,
                                                 RIB_INTERFACE_ENTRY  *  dest);

DXDEC int        __stdcall AIL_filter_property    (HPROVIDER  lib,
                                                 char const *   name,
                                                 void *       before_value,
                                                 void const * new_value,
                                                 void *       after_value
                                                 );

 int      __stdcall AIL_enumerate_output_filter_driver_properties
                                                (HPROVIDER                 lib,
                                                 HINTENUM  *            next,
                                                 RIB_INTERFACE_ENTRY  * dest);

 int     __stdcall AIL_output_filter_driver_property
                                                (HDIGDRIVER     dig,
                                                 char const  * name,
                                                 void *       before_value,
                                                 void const * new_value,
                                                 void *       after_value
                                                 );

 int      __stdcall AIL_enumerate_output_filter_sample_properties
                                                (HPROVIDER                 lib,
                                                 HINTENUM  *            next,
                                                 RIB_INTERFACE_ENTRY  * dest);

 int      __stdcall AIL_enumerate_filter_sample_properties
                                                (HPROVIDER                 lib,
                                                 HINTENUM  *            next,
                                                 RIB_INTERFACE_ENTRY  * dest);

DXDEC int       __stdcall AIL_enumerate_sample_stage_properties
                                                (HSAMPLE                    S,
                                                 SAMPLESTAGE                stage,
                                                 HINTENUM  *             next,
                                                 RIB_INTERFACE_ENTRY  *  dest);

 int      __stdcall AIL_sample_stage_property
                                                (HSAMPLE        S,
                                                 SAMPLESTAGE    stage,
                                                 char const  * name,
                                                 void *       before_value,
                                                 void const * new_value,
                                                 void *       after_value
                                                 );

#define AIL_filter_sample_property(S,name,beforev,newv,afterv) AIL_sample_stage_property((S),SP_FILTER_0,(name),(beforev),(newv),(afterv))

typedef MSS_STRUCT _FLTPROVIDER
{
   PROVIDER_PROPERTY               PROVIDER_property;

   FLT_STARTUP                     startup;
   FLT_ERROR                       error;
   FLT_SHUTDOWN                    shutdown;
   FLT_OPEN_DRIVER                 open_driver;
   FLT_CLOSE_DRIVER                close_driver;
   FLT_PREMIX_PROCESS              premix_process;
   FLT_POSTMIX_PROCESS             postmix_process;

   FLTSMP_OPEN_SAMPLE              open_sample;
   FLTSMP_CLOSE_SAMPLE             close_sample;
   FLTSMP_SAMPLE_PROCESS           sample_process;
   FLTSMP_SAMPLE_PROPERTY          sample_property;

   VFLT_ASSIGN_SAMPLE_VOICE        assign_sample_voice;
   VFLT_RELEASE_SAMPLE_VOICE       release_sample_voice;
   VFLT_START_SAMPLE_VOICE         start_sample_voice;

   VDRV_DRIVER_PROPERTY            driver_property;
   VDRV_FORCE_UPDATE               force_update;

   VSMP_SAMPLE_PROPERTY            output_sample_property;

   HDIGDRIVER   dig;
   HPROVIDER    provider;
   HDRIVERSTATE driver_state;
   int          provider_flags;

   struct _FLTPROVIDER  *next;
}
FLTPROVIDER;

//
// Values for "Flags" property exported by all MSS Pipeline Filter and MSS Output Filter 
// providers
// 

#define FPROV_ON_SAMPLES 0x0001        // Pipeline filter that operates on input samples (and is enumerated by AIL_enumerate_filters)
#define FPROV_ON_POSTMIX 0x0002        // Pipeline filter that operates on the post mixed output (capture filter)
#define FPROV_MATRIX     0x0004        // This is a matrix output filter (e.g., SRS/Dolby) 
#define FPROV_VOICE      0x0008        // This is a per-voice output filter (e.g., DirectSound 3D)
#define FPROV_3D         0x0010        // Output filter uses S3D substructure for positioning
#define FPROV_OCCLUSION  0x0020        // Output filter supports occlusion (doesn't need per-sample lowpass)
#define FPROV_EAX        0x0040        // Output filter supports EAX-compatible environmental reverb

#ifdef IS_WIN32

#define MSS_EAX_AUTO_GAIN   1
#define MSS_EAX_AUTOWAH     2
#define MSS_EAX_CHORUS      3
#define MSS_EAX_DISTORTION  4
#define MSS_EAX_ECHO        5
#define MSS_EAX_EQUALIZER   6
#define MSS_EAX_FLANGER     7
#define MSS_EAX_FSHIFTER    8
#define MSS_EAX_VMORPHER    9
#define MSS_EAX_PSHIFTER   10
#define MSS_EAX_RMODULATOR 11 
#define MSS_EAX_REVERB     12

typedef MSS_STRUCT EAX_SAMPLE_SLOT_VOLUME
{
  int Slot;       // 0, 1, 2, 3
  int Send;
  int SendHF;
  int Occlusion;
  float OcclusionLFRatio;
  float OcclusionRoomRatio;
  float OcclusionDirectRatio;
} EAX_SAMPLE_SLOT_VOLUME;

typedef MSS_STRUCT EAX_SAMPLE_SLOT_VOLUMES
{
  UINT NumVolumes;  // 0, 1, or 2
  EAX_SAMPLE_SLOT_VOLUME volumes[ 2 ];
} EAX_SAMPLE_SLOT_VOLUMES;

// Use this structure for EAX REVERB
typedef MSS_STRUCT EAX_REVERB
{
  int Effect;                  // set to MSS_EAX_REVERB
  int Volume;                  // -10000 to 0
  UINT Environment;             // one of the ENVIRONMENT_ enums
  float EnvironmentSize;         // environment size in meters
  float EnvironmentDiffusion;    // environment diffusion
  int Room;                    // room effect level (at mid frequencies)
  int RoomHF;                  // relative room effect level at high frequencies
  int RoomLF;                  // relative room effect level at low frequencies  
  float DecayTime;               // reverberation decay time at mid frequencies
  float DecayHFRatio;            // high-frequency to mid-frequency decay time ratio
  float DecayLFRatio;            // low-frequency to mid-frequency decay time ratio   
  int Reflections;             // early reflections level relative to room effect
  float ReflectionsDelay;        // initial reflection delay time
  float ReflectionsPanX;         // early reflections panning vector
  float ReflectionsPanY;         // early reflections panning vector
  float ReflectionsPanZ;         // early reflections panning vector
  int Reverb;                  // late reverberation level relative to room effect
  float ReverbDelay;             // late reverberation delay time relative to initial reflection
  float ReverbPanX;              // late reverberation panning vector
  float ReverbPanY;              // late reverberation panning vector
  float ReverbPanZ;              // late reverberation panning vector
  float EchoTime;                // echo time
  float EchoDepth;               // echo depth
  float ModulationTime;          // modulation time
  float ModulationDepth;         // modulation depth
  float AirAbsorptionHF;         // change in level per meter at high frequencies
  float HFReference;             // reference high frequency
  float LFReference;             // reference low frequency 
  float RoomRolloffFactor;       // like DS3D flRolloffFactor but for room effect
  UINT Flags;                   // modifies the behavior of properties
} EAX_REVERB;

// Use this structure for EAX AUTOGAIN
typedef MSS_STRUCT EAX_AUTOGAIN
{
  int Effect;      // set to MSS_EAX_AUTO_GAIN
  int Volume;      // -10000 to 0
  UINT OnOff;       // Switch Compressor on or off (1 or 0)
} EAX_AUTOGAIN;

// Use this structure for EAX AUTOWAH
typedef MSS_STRUCT EAX_AUTOWAH
{
   int Effect;        // set to MSS_EAX_AUTOWAH
   int Volume;        // -10000 to 0
   float AttackTime;    // Attack time (seconds)
   float ReleaseTime;   // Release time (seconds)
   int Resonance;     // Resonance (mB)
   int PeakLevel;     // Peak level (mB)
} EAX_AUTOWAH;

// Use this structure for EAX CHORUS
typedef MSS_STRUCT EAX_CHORUS
{
  int Effect;       // set to MSS_EAX_CHORUS
  int Volume;       // -10000 to 0
  UINT Waveform;     // Waveform selector - 0 = sinusoid, 1 = triangle
  int Phase;        // Phase (Degrees)
  float Rate;         // Rate (Hz)
  float Depth;        // Depth (0 to 1)
  float Feedback;     // Feedback (-1 to 1)
  float Delay;        // Delay (seconds)
} EAX_CHORUS;

// Use this structure for EAX DISTORTION
typedef MSS_STRUCT EAX_DISTORTION
{
  int Effect;        // set to MSS_EAX_DISTORTION
  int Volume;        // -10000 to 0
  float Edge;          // Controls the shape of the distortion (0 to 1)
  int Gain;          // Controls the post distortion gain (mB)
  float LowPassCutOff; // Controls the cut-off of the filter pre-distortion (Hz)
  float EQCenter;      // Controls the center frequency of the EQ post-distortion (Hz)
  float EQBandwidth;   // Controls the bandwidth of the EQ post-distortion (Hz)
} EAX_DISTORTION;

// Use this structure for EAX ECHO
typedef MSS_STRUCT EAX_ECHO
{
  int Effect;        // set to MSS_EAX_ECHO
  int Volume;        // -10000 to 0
  float Delay;         // Controls the initial delay time (seconds)
  float LRDelay;       // Controls the delay time between the first and second taps (seconds)
  float Damping;       // Controls a low-pass filter that dampens the echoes (0 to 1)
  float Feedback;      // Controls the duration of echo repetition (0 to 1)
  float Spread;        // Controls the left-right spread of the echoes
} EAX_ECHO;

// Use this structure for EAXEQUALIZER_ALLPARAMETERS
typedef MSS_STRUCT EAX_EQUALIZER
{
  int Effect;        // set to MSS_EAX_EQUALIZER
  int Volume;        // -10000 to 0
  int LowGain;       // (mB)
  float LowCutOff;     // (Hz)
  int Mid1Gain;      // (mB)
  float Mid1Center;    // (Hz)
  float Mid1Width;     // (octaves)
  float Mid2Gain;      // (mB)
  float Mid2Center;    // (Hz)
  float Mid2Width;     // (octaves)
  int HighGain;      // (mB)
  float HighCutOff;    // (Hz)
} EAX_EQUALIZER;

// Use this structure for EAX FLANGER
typedef MSS_STRUCT EAX_FLANGER
{
  int Effect;       // set to MSS_EAX_FLANGER
  int Volume;       // -10000 to 0
  UINT Waveform;     // Waveform selector - 0 = sinusoid, 1 = triangle
  int Phase;        // Phase (Degrees)
  float Rate;         // Rate (Hz)
  float Depth;        // Depth (0 to 1)
  float Feedback;     // Feedback (0 to 1)
  float Delay;        // Delay (seconds)
} EAX_FLANGER;


// Use this structure for EAX FREQUENCY SHIFTER
typedef MSS_STRUCT EAX_FSHIFTER
{
  int Effect;         // set to MSS_EAX_FSHIFTER
  int Volume;         // -10000 to 0
  float Frequency;      // (Hz)
  UINT LeftDirection;  // direction - 0 = down, 1 = up, 2 = off
  UINT RightDirection; // direction - 0 = down, 1 = up, 2 = off
} EAX_FSHIFTER;

// Use this structure for EAX VOCAL MORPHER
typedef MSS_STRUCT EAX_VMORPHER
{
  int Effect;                // set to MSS_EAX_VMORPHER
  int Volume;                // -10000 to 0
  UINT PhonemeA;              // phoneme: 0 to 29 - A E I O U AA AE AH AO EH ER IH IY UH UW B D G J K L M N P R S T V Z
  int PhonemeACoarseTuning;  // (semitones)
  UINT PhonemeB;              // phoneme: 0 to 29 - A E I O U AA AE AH AO EH ER IH IY UH UW B D G J K L M N P R S T V Z
  int PhonemeBCoarseTuning;  // (semitones)
  UINT Waveform;              // Waveform selector - 0 = sinusoid, 1 = triangle, 2 = sawtooth
  float Rate;                  // (Hz)
} EAX_VMORPHER;


// Use this structure for EAX PITCH SHIFTER
typedef MSS_STRUCT EAX_PSHIFTER
{
  int Effect;       // set to MSS_EAX_PSHIFTER
  int Volume;       // -10000 to 0
  int CoarseTune;   // Amount of pitch shift (semitones)
  int FineTune;     // Amount of pitch shift (cents)
} EAX_PSHIFTER;

// Use this structure for EAX RING MODULATOR
typedef MSS_STRUCT EAX_RMODULATOR
{
  int Effect;          // set to MSS_EAX_RMODULATOR
  int Volume;          // -10000 to 0
  float Frequency;       // Frequency of modulation (Hz)
  float HighPassCutOff;  // Cut-off frequency of high-pass filter (Hz)
  UINT Waveform;        // Waveform selector - 0 = sinusoid, 1 = triangle, 2 = sawtooth
} EAX_RMODULATOR;

#endif

#else

typedef MSS_STRUCT _FLTPROVIDER
{
  UINT junk;
} FLTPROVIDER;

#endif

#ifndef IS_PS2
#else

// round up to multiples of 16 for DMA alignment
#define SPR_IS_NEEDED  (((2 * 578         * sizeof(short)) + 15) & ~15)  // 2320
#define SPR_XR_NEEDED  (((2 * 32 * 18     * sizeof(float)) + 15) & ~15)  // 4608
#define SPR_LR_NEEDED  (((2 * 32 * 18     * sizeof(float)) + 15) & ~15)  // 4608
#define SPR_RES_NEEDED (((32 * 18         * sizeof(float)) + 15) & ~15)  // 2304
#define SPR_S_NEEDED   (((2 * 32 * 18     * sizeof(float)) + 15) & ~15)  // 4608
#define SPR_U_NEEDED   (((2 * 2 * 17 * 16 * sizeof(float)) + 15) & ~15)  // 4352

#define SPR_MEM        (0x70000000)
#define SPR_U_START    (SPR_MEM)
#define SPR_S_START    (SPR_U_START + SPR_U_NEEDED)
#define SPR_RES_START  (SPR_S_START + SPR_S_NEEDED)
#define SPR_LR_START   (SPR_RES_START + SPR_RES_NEEDED)
#define SPR_IS_START   (SPR_RES_START + SPR_RES_NEEDED)  // Shared with LR space (LR is larger)
#define END_SPR_MEM    (SPR_LR_START + SPR_LR_NEEDED)
#define SPR_BYTES_USED (END_SPR_MEM - SPR_MEM)

typedef enum
{
   MSS_DO_NOT_USE,
   MSS_USE_AND_SAVE,
   MSS_USE_WITHOUT_SAVING,
   MSS_USED = 100,               // Internal use only
}
MSS_RESOURCE_POLICY;

DXDEC MSS_RESOURCE_POLICY __stdcall AIL_set_scratchpad_policy(MSS_RESOURCE_POLICY policy);
DXDEC MSS_RESOURCE_POLICY __stdcall AIL_scratchpad_policy    (void);


#endif

*/

/*
//############################################################################
//##                                                                        ##
//##  Miles Sound System                                                    ##
//##                                                                        ##
//##  MSSDBG.C: Runtime debugging facilities for MSS API calls              ##
//##                                                                        ##
//##  16-bit protected-mode source compatible with MSC 7.0                  ##
//##  32-bit protected-mode source compatible with MSC 9.0                  ##
//##                                                                        ##
//##  Version 1.00 of 15-Feb-95: Derived from WAILDBG V1.01                 ##
//##          1.01 of 11-Oct-95: Added MEM_alloc/free_lock() debug trace    ##
//##          1.02 of 20-Jan-96: jkr: added all the win32 stuff             ##
//##          1.03 of 20-Mar-98: jkr: merged DOS and windows                ##
//##                                                                        ##
//##  Author: John Miles & Jeff Roberts                                     ##
//##                                                                        ##
//############################################################################
//##                                                                        ##
//##  Copyright (C) RAD Game Tools, Inc.                                    ##
//##                                                                        ##
//##  Contact RAD Game Tools at 425-893-4300 for technical support.         ##
//##                                                                        ##
//############################################################################

#if defined(_XENON) || (_XBOX_VER == 200) || defined(_XBOX)
#include <xtl.h>
#elif defined(_WIN32) || defined(_WIN64) || defined(WIN32) || defined(__NT__) || defined(__WIN32__)
#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#define STRICT
#include <windows.h>
#include <mmsystem.h>
#include <ks.h>
#pragma warning( disable : 4214)
#include <ksmedia.h>
#endif

#include "mss.h"
#include "imssapi.h"

#include <stdio.h>
#include <time.h>
#include <stdarg.h>
#include <math.h>

#ifdef IS_LINUX
#include <string.h>
#endif

#ifdef IS_PS2
#include <eekernel.h>
#endif

#ifdef IS_WII
#include <revolution/dvd.h>
#endif

//############################################################################
//##                                                                        ##
//## Locked static data                                                     ##
//##                                                                        ##
//############################################################################

U16 AIL_debug=0;

U16 AIL_sys_debug=0;

U16 AIL_indent=0;

U32 AIL_starttime;

#ifndef IS_DOS
AILTRACECB trace_callback;
#endif

#ifdef IS_WIN64

  #define PTRFMT "0x%I64X"
  #define SINTFMT "%I64d"
  #define HPFMT "%ld"

#else

  #define PTRFMT "0x%lX"
  #define SINTFMT "%ld"
  #define HPFMT "%ld"

#endif

#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX) || defined(IS_XBOX) || defined(IS_XENON) || defined(IS_PS3)

FILE *AIL_debugfile;
#define NL "\n"

#elif defined(IS_WII)

U32 AIL_debugfile;
#define NL "\n"

#else

U32 AIL_debugfile;
#define NL "\r\n"

#endif

#if defined(IS_WIN32API) || defined(IS_PS3) || defined(IS_MAC)

  #define IN_AIL  InMilesMutex()
  #define OUT_AIL OutMilesMutex()

  #define IN_AIL_NM
  #define OUT_AIL_NM

#else
  #define IN_AIL
  #define OUT_AIL

  #define IN_AIL_NM
  #define OUT_AIL_NM
#endif

#ifdef IS_32

  int AIL_didaninit=NO;

  #define DidAnInit() (AIL_didaninit!=NO)

  #define SetInit(val) AIL_didaninit=(val);

#else

  #define MAXTASKS 32             // Only 32 16-bit apps can be used at once

  static HTASK inittasks[MAXTASKS];

  static int numtasks=0;

  static U8 DidAnInit()          // See if this task has previously called
  {                              //   AIL_startup
    int i;
    HTASK t=GetCurrentTask();
    for(i=0;i<numtasks;i++)
      if (inittasks[i]==t)
        return(1);
    return(0);
  }

  static void SetInit(int val)   // Clear or set the task's init status
  {
    int i;
    HTASK t;

    if (val==NO) {
      t=GetCurrentTask();
      for(i=0;i<numtasks;i++)
        if (inittasks[i]==t) {
          AIL_memcpy((void*)&inittasks[i],(void*)&inittasks[i+1],(numtasks-i-1)*sizeof(inittasks[0]));
          --numtasks;
          break;
        }
    } else {
      if (numtasks>=MAXTASKS)
        MessageBox(0,"Too many MSS applications loaded at once.","Error from MSS",MB_OK);
      else
        inittasks[numtasks++]=GetCurrentTask();
    }
  }

#endif

//############################################################################
//##                                                                        ##
//## Debug trace macros and helper functions                                ##
//##                                                                        ##
//############################################################################

#define START  if (AIL_indent++, (AIL_debug && \
                  ((AIL_indent==1) || AIL_sys_debug) && \
                  (!AIL_low_background() && AIL_time_write())))

#define END    AIL_indent--

#define RESULT if (AIL_debug && ((AIL_indent==1) || AIL_sys_debug) && (!AIL_low_background()))

#define INDENT AIL_fprintf(AIL_debugfile,"%s%s",(char FAR *)indentspace,(char FAR *)indentdot+33-AIL_indent);

static char indentspace[]="               ";
static char indentdot[]="--------------------------------";

#define FTS AIL_ftoa

#ifdef IS_PS2

static void AIL_fprintf (U32 out, char const FAR *fmt, ...)
{
   va_list ap;

   va_start(ap, 
            fmt);

   if (trace_callback == NULL)
      {
      sceVprintf(fmt, 
                 ap);
      }
   else
      {
      C8 text[256];

      sceVsnprintf(text,
                   sizeof(text)-1,
                   fmt,
                   ap);
     text[sizeof(text)-1]=0;

      trace_callback(text,
                     AIL_indent);
      }

   va_end(ap);
}

#elif defined(IS_WII)

static void AIL_fprintf (U32 out, char const FAR *fmt, ...)
{
   C8 text[512];
   va_list ap;

   va_start(ap, 
            fmt);

   vsnprintf(text,
             sizeof(text)-1,
             fmt,
             ap);
   text[sizeof(text)-1]=0;

   if (trace_callback == NULL)
      {
      OSReport( text );
      }
      else
      {
      trace_callback(text,
                     AIL_indent);
      }

   va_end(ap);
}

#else

#ifndef IS_DOS
static void AIL_fprintf (FILE *out, char const FAR *fmt, ...)
{
   va_list ap;

   va_start(ap, 
            fmt);

   if (trace_callback == NULL)
      {
      vfprintf(out,
               fmt, 
               ap);
      }
   else
      {
      C8 text[256];

      #if defined(IS_MAC) || defined(IS_PS3) || defined(IS_WII)
      vsnprintf(text,
      #else
      _vsnprintf(text,
      #endif
                 sizeof(text)-1,
                 fmt,
                 ap);
      text[sizeof(text)-1]=0;

      trace_callback(text,
                     AIL_indent);
      }

   va_end(ap);
}
#endif

#endif

#ifdef IS_DOS

//############################################################################
//##                                                                        ##
//## Locked code                                                            ##
//##                                                                        ##
//############################################################################

#ifdef __WATCOMC__
#pragma aux AIL_debug "_*";
#pragma aux AIL_sys_debug "_*";
#pragma aux AIL_indent "_*";
#pragma aux AIL_starttime "_*";
#pragma aux AIL_debugfile "_*";
#pragma aux AIL_didaninit "_*";
#endif

#define LOCK(x) AIL_vmm_lock((void *) &(x),sizeof(x))

static S32 locked = 0;

void __pascal AILDEBUG_end(void);

void __pascal AILDEBUG_start(void)
{
   if (!locked)
      {
      AIL_vmm_lock_range(AILDEBUG_start, AILDEBUG_end);

      LOCK (AIL_indent);
      LOCK (AIL_debug);
      LOCK (AIL_sys_debug);
      LOCK (AIL_debugfile);

      LOCK (AIL_didaninit);
      LOCK (AIL_starttime);

      locked = 1;

      }
}

#endif
                                        
static void outresfloat(F32 val)
{
  INDENT;
  AIL_fprintf(AIL_debugfile,"Result = %s" NL,FTS(val));
}

static void outresint(U32 val)
{
  INDENT;
  AIL_fprintf(AIL_debugfile,"Result = %d" NL,val);
}

static void outresstr(C8 *val)
{
  INDENT;
  if (val == NULL)
      AIL_fprintf(AIL_debugfile,"Result = NULL (no error)" NL);
   else
      AIL_fprintf(AIL_debugfile,"Result = %s" NL,val);
}

static void outreshex(U32 val)
{
  INDENT;
  AIL_fprintf(AIL_debugfile,"Result = 0x%X" NL,val);
}

static void outresuint(UINTa val)
{
  INDENT;
  AIL_fprintf(AIL_debugfile,"Result = " SINTFMT NL,val); 
}

static void outresuhex(UINTa val)
{
  INDENT;
  AIL_fprintf(AIL_debugfile,"Result = " PTRFMT NL,val); 
}

DXDEF C8 * AILCALL AIL_ftoa(F32 fval)
{
#define N_STRINGS_IN_POOL     16
#define MAX_POOL_STRING_BYTES 32

   //
   // Maintain pool of strings so that AIL_ftoa() can be called multiple
   // times without invalidating previous results
   // (e.g., in a printf() statement)
   //
   // Background and foreground contexts need separate string pools
   // to avoid collisions
   //

   static C8  f_pool[N_STRINGS_IN_POOL][MAX_POOL_STRING_BYTES];
   static S32 f_cur = 0;

   static C8  b_pool[N_STRINGS_IN_POOL][MAX_POOL_STRING_BYTES];
   static S32 b_cur = 0;

#ifdef IS_PS2
   S32 irq = DI();
#endif

   C8 *result;

   if (AIL_API_background())
      {
      result = b_pool[b_cur];
      b_cur = (b_cur + 1) % N_STRINGS_IN_POOL;
      }
   else
      {
      result = f_pool[f_cur];
      f_cur = (f_cur + 1) % N_STRINGS_IN_POOL;
      }

#ifdef IS_PS2
   if (irq) EI();
   
   // 
   // GCC under PS2 cannot handle floats in vararg functions, so
   // we have to implement our own float-to-string handler
   //

#define FRACT_DIGITS 4

   {
   S32 s,i;
   S32 fixed;
   C8 buffer[MAX_POOL_STRING_BYTES];
   C8 *d,*r;

   s = (fval < 0.0F);
   if (s) fval = -fval;

   fixed = (S32) ((AIL_pow(10.0F,FRACT_DIGITS) * fval) + 0.5F);

   d = buffer;

   for (i=0; i < MAX_POOL_STRING_BYTES-3; i++)
      {
      *d++ = (fixed % 10) + '0';
      fixed /= 10;

      if (i == (FRACT_DIGITS-1))
         {
         *d++ = '.';
         }

      if ((!fixed) && (i >= (FRACT_DIGITS)))
         {
         break;
         }
      }

   if (s) 
      {
      *d++ = '-';
      }

   r = result;
   while (d > buffer)
      {
      *r++ = *--d;
      }
   *r = 0;
   }
#else

   sprintf(result,"%f",fval);

#endif

   return result;
}

#if defined(IS_WIN32) || defined(IS_PS2) || defined(IS_MAC) || defined(IS_WII)

//
// Open a matrix- or output-filter provider
//

S32 MC_open_output_filter(C8        *name, //)
                          HDIGDRIVER driver,
                          S32        is_matrix_filter)
{
   HPROVIDER HP;

   if ((is_matrix_filter) && (!(driver->hw_format & DIG_F_16BITS_MASK)))
      {
      AIL_set_error("Matrix filters require 16-bit PCM output format");
      return 0;
      }

   if (!AIL_find_filter(name, 
                       &HP))
      {
      AIL_set_error("Required output filter not found");
      return 0;
      }

   if (!AIL_open_filter(HP,
                        driver))
      {
      return 0;
      }

   return 1;
}

#endif
      
#ifdef IS_WIN32

  static U32 doquery=0;   // jkr: use queryperformance instead of timegettime?

  static LONGLONG qpfreq; // frequency of the high-perf timer

  static LONGLONG qpfirst;// first read of queryperformance

  static U32 lasttime;
  static U32 lastrad;
  static U32 radadj=0;

  static U32 lasttimeu;
  static U32 lastradu;
  static U32 radadju=0;

  // jkr: this function is called when the 32 bit DLL is loaded

  BOOL Win32Init(HINSTANCE hinstDll)
  {
    if (QueryPerformanceFrequency((LARGE_INTEGER*)&qpfreq))
    {
      doquery=1;
      QueryPerformanceCounter((LARGE_INTEGER*)&qpfirst);
      lastrad=0;
      lasttime=GetTickCount();
      lastradu=0;
      lasttimeu=lasttime;
    }

    return( TRUE );
  }

//
// we don't display these calls in the debug file because they are
// called so frequently
//

DXDEF U32 AILEXPORT AIL_ms_count()
{
    LONGLONG count;
    if (doquery)
    {
      U32 ret, time;
      S32 deltatime,deltarad;

      // read the new times...
      QueryPerformanceCounter((LARGE_INTEGER*)&count);
      time = GetTickCount();
      ret = radadj + ( (U32)(((count-qpfirst)*((LONGLONG)1000))/qpfreq) );

      // see how much each has moved
      deltatime = time - lasttime;
      deltarad = ret - lastrad;

      // check the query against GetTickCount to make sure it hasn't
      //   jumped ahead...

      if ( abs( deltatime-deltarad ) > 200 )
      {
        deltatime -= deltarad;
        radadj += deltatime;
        ret += deltatime;
      }

      // now check to see if it went backwards?
      if ( ( ( U32 ) ( ret - lastrad ) ) > 0xc0000000 )
      {
        // yup, just return the old timer value then
        return( lastrad );
      }

      lasttime = time;
      lastrad = ret;
      return(ret);

    } else
      return( timeGetTime() );
}

DXDEF U32 AILEXPORT AIL_us_count()
{
    LONGLONG count;
    if (doquery) {
      U32 ret, time;
      S32 deltatime,deltarad;

      // read the new times...
      QueryPerformanceCounter((LARGE_INTEGER*)&count);
      time = GetTickCount();
      ret = radadju + ( (U32)(((count-qpfirst)*((LONGLONG)1000000L))/qpfreq) );

      // see how much each has moved
      deltatime = (time - lasttimeu)*1000;
      deltarad = ret - lastradu;

      // check the query against GetTickCount to make sure it hasn't
      //   jumped ahead...

      if ( abs( deltatime-deltarad ) > 200000 )
      {
        deltatime -= deltarad;
        radadju += deltatime;
        ret += deltatime;
      }

      // now check to see if it went backwards?
      if ( ( ( U32 ) ( ret - lastradu ) ) > 0xc0000000 )
      {
        // yup, just return the old timer value then
        return( lastradu );
      }

      lasttimeu = time;
      lastradu = ret;
      return(ret);
    } else
      return( timeGetTime()*1000 );
}

#endif

#if defined(IS_LINUX)

#include <sys/time.h>

  //
  // we don't display these calls in the debug file because they are
  // called so frequently
  //

  static long long start = 0;

  DXDEF U32 AILEXPORT AIL_ms_count()
  {
    long long now;
    struct timeval tv;
    gettimeofday(&tv, NULL);
    now = tv.tv_sec;
    now *= 1000000;
    now += tv.tv_usec;
    if ( !start ) {
        start = now;
    }
    return (U32)((now - start) / 1000);
  }

  DXDEF U32 AILEXPORT AIL_us_count()
  {
    long long now;
    struct timeval tv;
    gettimeofday(&tv, NULL);
    now = tv.tv_sec;
    now *= 1000000;
    now += tv.tv_usec;
    if ( !start ) {
        start = now;
    }
    return (U32)(now - start);
  }

#endif

// jkr: these macros expand to normal function calls

#define ailcall0(header,func,type)                                    \
          header##_API_##func()

#define ailcall1(header,func,type,param1)                             \
          header##_API_##func(param1)

#define ailcall2(header,func,type,param1,param2)                      \
          header##_API_##func(param1,param2)

#define ailcall3(header,func,type,param1,param2,param3)               \
          header##_API_##func(param1,param2,param3)

#define ailcall4(header,func,type,param1,param2,param3,param4)        \
          header##_API_##func(param1,param2,param3,param4)

#define ailcall5(header,func,type,param1,param2,param3,param4,param5) \
          header##_API_##func(param1,param2,param3,param4,param5)

#define ailcall6(header,func,type,param1,param2,param3,param4,param5,param6) \
          header##_API_##func(param1,param2,param3,param4,param5,param6)

#define ailcall7(header,func,type,param1,param2,param3,param4,param5,param6,param7) \
          header##_API_##func(param1,param2,param3,param4,param5,param6,param7)

#define ailcall14(header,func,type,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13,param14) \
          header##_API_##func(param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13,param14)

#define ailcall15(header,func,type,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13,param14,param15) \
          header##_API_##func(param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13,param14,param15)

#define ailcall16(header,func,type,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13,param14,param15,param16) \
          header##_API_##func(param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13,param14,param15,param16)

#define dsailcall0 ailcall0
#define dsailcall1 ailcall1
#define dsailcall2 ailcall2
#define dsailcall3 ailcall3
#define dsailcall4 ailcall4
#define dsailcall5 ailcall5
#define dsailcall6 ailcall6
#define dsailcall7 ailcall7
#define dsailcall14 ailcall14
#define dsailcall15 ailcall15
#define dsailcall16 ailcall16

#if defined(IS_MAC) || defined(IS_LINUX) || defined(IS_XBOX) || defined(IS_XENON) || defined(IS_PS2) || defined(IS_PS3) || defined(IS_WII)

static void AIL_get_debug_file(char* fname)
{
  #if defined(IS_XBOX) || defined(IS_XENON)
    AIL_strcpy( fname, "d:\\mss_debug_log.txt" );
  #else
    #if defined(IS_PS3)
      AIL_strcpy( fname, "/dev_hdd0/mss_debug_log.txt" );
    #else
      #if defined(IS_PS2)
        AIL_strcpy( fname, "host0:\\mss_debug_log.txt" );
      #elif defined(IS_WII)
        AIL_strcpy( fname, "/mss_debug_log.txt" );
     #else
        AIL_strcpy( fname, MSS_Directory);
        AIL_strcat(fname,"mssdebug.log");
     #endif
    #endif
  #endif
}

#endif

static S32    AIL_low_background                (void)
{
  return(AIL_API_background());
}

//############################################################################
//##                                                                        ##
//## Write timestamp to output file or device                               ##
//##                                                                        ##
//############################################################################

S32 AILCALL AIL_time_write(void)
{
   U32 t;
   U16 h,m,s,c;

   t = AIL_ms_count()-AIL_starttime;

   h = (U16)(t/3600000L);
   m = (U16)((t/60000L)%60L);
   s = (U16)((t/1000L)%60L);
   c = (U16)(t%1000L);

   if (AIL_indent == 1)
      {
      AIL_fprintf(AIL_debugfile,"[%.02d:%.02d:%.02d.%.03d] %s",h,m,s,c,(char FAR *)indentdot+33-AIL_indent);
      }
   else
      AIL_fprintf(AIL_debugfile,"%s%s", (char FAR *)indentspace,(char FAR *)indentdot+33-AIL_indent);

   return 1;
}

//############################################################################
//##                                                                        ##
//## AIL_MMX_available()                                                    ##
//##                                                                        ##
//############################################################################

#ifdef IS_X86

DXDEF U32 AILEXPORT AIL_MMX_available(void)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_MMX_available()" NL);
      }

   IN_AIL_NM;

   if (!AIL_get_preference(AIL_ENABLE_MMX_SUPPORT))
      {
      result = 0;
      }
   else
      {
      result = MSS_MMX_available();
      }

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

#endif

//############################################################################
//##                                                                        ##
//## AIL_debug_printf()                                                     ##
//##                                                                        ##
//############################################################################

#ifndef IS_MAC

DXDEF void
#if !defined(IS_LINUX) && !defined(IS_PS2) && !defined(IS_PS3) && !defined(IS_WII) 
__cdecl
#endif
AIL_debug_printf(C8 const FAR *fmt, ...)
{
  #ifdef _DEBUG
   char work_string[768];

   va_list ap;

   va_start(ap,
            fmt);

#if defined(IS_XBOX) || defined(IS_DOS)
   vsprintf(work_string,
#else
   vsnprintf(work_string,768,
#endif
             fmt,
             ap);

   va_end  (ap);

#if defined(IS_WINDOWS) || defined(IS_XBOX) || defined(IS_XENON)
   OutputDebugString(work_string);
#else
#if SHOW_DEBUG_TRACE || defined(IS_WII)
   printf(work_string);
#endif
#endif

#if DEBUG_LOGFILE
   static S32 init = 0;

   if (init == 0)
      {
      _unlink("debug.log");
      init = 1;
      }

   FILE FAR*out = fopen("debug.log","a+t");
   fprintf(out,"%s",work_string);
   fclose(out);
#endif
#endif
}

#else

extern void DebugStr( unsigned char const * );

DXDEF void AIL_debug_printf(C8 const FAR *fmt, ...)
{
  #ifdef _DEBUG
   char work_string[256];

   va_list ap;

   va_start(ap,
            fmt);

   work_string[0] = (char) vsnprintf(work_string + 1, 255,fmt,ap);
   work_string[255]=0;
   
   va_end  (ap);
   DebugStr( (U8*) work_string );
  #endif
}
#endif

#ifndef IS_DOS

//############################################################################
DXDEC  AILTRACECB AILCALL AIL_register_trace_callback (AILTRACECB cb, 
                                                       S32        level)
{
   AILTRACECB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_trace_callback(" PTRFMT ",0x%ld)" NL,
         cb, level);
      }

   IN_AIL;

   result         = trace_callback;
   trace_callback = cb;

   switch (level)
      {
      case 0: 
         AIL_debug      = FALSE;
         AIL_sys_debug  = FALSE;
         break;

      case 1:
         AIL_debug      = TRUE;
         AIL_sys_debug  = FALSE;
         break;

      case 2:
         AIL_debug      = TRUE;
         AIL_sys_debug  = TRUE;
         break;
      }

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

#endif

#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX) || defined(IS_XBOX) || defined(IS_XENON) || defined(IS_PS2) || defined(IS_PS3) || defined(IS_WII)

//############################################################################
DXDEF S32    AILEXPORT AIL_startup                   (void)
{
   static char filename[256];
   static S8 sysdebug[8];
#ifndef IS_PS2
   static time_t elapstime;
   static struct tm *adjtime;
   static char loctime[32];
#endif

   if (DidAnInit())
     return(0);

   SetInit(YES);

#ifdef IS_WIN32API

   //
   // Get handle to application process thread that called AIL_startup(),
   // so that callback functions can suspend it
   //

   DuplicateHandle(GetCurrentProcess(),
                   GetCurrentThread(),
                   GetCurrentProcess(),
                  &hAppThread,
                   0,
                   FALSE,
                   DUPLICATE_SAME_ACCESS);

#endif

   //
   // Get profile string for debug script, and enable debug mode
   // if script filename valid
   //

   AIL_debug     = 0;
   AIL_sys_debug = 0;

#if defined(IS_MAC) || defined(IS_LINUX) || defined(IS_XBOX) || defined(IS_XENON) || defined(IS_PS2) || defined(IS_PS3) || defined(IS_WII)

  AIL_get_debug_file( filename );

  #if defined(IS_WII)
    if (DVDConvertPathToEntrynum(filename)!=-1)
      AIL_debugfile=1;
    else
      AIL_debugfile=0;
  #elif defined(IS_PS2) 
     {
     U32 hand = 0;
     AIL_debugfile = MSS_open(filename, &hand);

     if (AIL_debugfile)
        {
        MSS_close(hand);
        }
     }
  #else
    AIL_debugfile = fopen(filename,"r");
    if(AIL_debugfile)
    {
      fclose(AIL_debugfile);
      AIL_debugfile = fopen(filename,"wt");       // Debug file is used as a journaling target
      AIL_sys_debug = 0;
    }
  #endif

#else

   GetProfileString("MSS", "MSSDebug", "None", filename, sizeof(filename));

   if (!AIL_strnicmp(filename, "None", 4))
      {
      ailcall0(AIL,startup,void);
      return(1);
      }

   GetProfileString("MSS", "MSSSysDebug", "0", sysdebug, sizeof(sysdebug));

   if (AIL_strnicmp(sysdebug, "0", 1))
      {
      AIL_sys_debug = 1;
      }

   //
   // Open script file and set "debug" flag
   //

   AIL_debugfile = fopen(filename,"wt");        // Debug file is used only as a flag

#endif

   //
   // if there is no debug file opened, then just startup normally
   //

   if (AIL_debugfile == 0)
      {
      ailcall0(AIL,startup,void);
      return(1);
      }

   //
   // Write header to script file
   //

#if defined(IS_PS2) || defined(IS_WII)

   AIL_fprintf(AIL_debugfile,
               "-------------------------------------------------------------------------------" NL
               "Miles Sound System usage script generated by MSS V"MSS_VERSION"" NL
               "-------------------------------------------------------------------------------" NL NL);
#else

   time(&elapstime);
   adjtime = localtime(&elapstime);
   AIL_strcpy(loctime,asctime(adjtime));
   loctime[24] = 0;

   AIL_fprintf(AIL_debugfile,
               "-------------------------------------------------------------------------------" NL
               "Miles Sound System usage script generated by MSS V"MSS_VERSION"" NL
               "Start time: %s" NL
               "-------------------------------------------------------------------------------" NL NL
               ,(U8 FAR *) loctime);
#endif

   //
   // Initialize API
   //

   ailcall0(AIL,startup,void);

   AIL_starttime=AIL_ms_count();

   AIL_debug  = 1;

   AIL_indent = 1;
   AIL_time_write();
   AIL_indent = 0;

   AIL_fprintf(AIL_debugfile,"AIL_startup()" NL);

   return(1);
}

#endif

//############################################################################

DXDEF
void    AILEXPORT  AIL_shutdown                  (void)
{
   if (DidAnInit())
     SetInit(NO);

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_shutdown()" NL);
      }

   IN_AIL;

   ailcall0(AIL,shutdown,void);

   OUT_AIL;        

#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX) || defined(IS_XBOX) || defined(IS_XENON) || defined(IS_PS3)
   if (AIL_debug && (AIL_debugfile != 0))
      fclose(AIL_debugfile);
#endif

   END;

   #ifdef IS_WIN32API

   if (hAppThread)
   {
     CloseHandle(hAppThread);     //avoid debug kernel message
     hAppThread=0;
   }

   #endif

}

//############################################################################

#ifdef IS_WIN32

DXDEF
HWND AILEXPORT AIL_HWND(void)
{
   HWND result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_HWND()" NL);
      }

   IN_AIL_NM;

   result = AIL_API_HWND();

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

#else

#ifdef IS_PS2

//############################################################################
DXDEF 
MSS_RESOURCE_POLICY AILEXPORT AIL_set_scratchpad_policy (MSS_RESOURCE_POLICY policy)
{
   MSS_RESOURCE_POLICY result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_scratchpad_policy(%d)" NL,policy);
      }

   IN_AIL_NM;

   result = ailcall1(AIL,set_scratchpad_policy,MSS_RESOURCE_POLICY,policy);

   OUT_AIL_NM;

   RESULT
      {
      outresint((S32) result);
      }

   END;

   return result;
}

//############################################################################
DXDEF 
MSS_RESOURCE_POLICY AILEXPORT AIL_scratchpad_policy (void)
{
   MSS_RESOURCE_POLICY result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_scratchpad_policy()" NL);
      }

   IN_AIL_NM;

   result = ailcall0(AIL,scratchpad_policy,MSS_RESOURCE_POLICY);

   OUT_AIL_NM;

   RESULT
      {
      outresint((S32) result);
      }

   END;

   return result;
}

#endif

#endif

//############################################################################

DXDEF
void FAR * AILEXPORT AIL_mem_alloc_lock(UINTa size)
{
   void FAR *result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_mem_alloc_lock(" SINTFMT ")" NL,size);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,mem_alloc_lock,void FAR*,size);

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void AILEXPORT AIL_mem_free_lock(void FAR *ptr)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_mem_free_lock(" PTRFMT ")" NL,ptr);
      }

   IN_AIL_NM;

   ailcall1(AIL,mem_free_lock,void,ptr);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
SINTa    AILEXPORT  AIL_set_preference            (U32       number, //)
                                                   SINTa     value)
{
   SINTa result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_preference(%ld," SINTFMT ")" NL,number,value);
      }

   IN_AIL;

   result=ailcall2(AIL,set_preference,SINTa,number,value);

   OUT_AIL;

   RESULT
      {
      outresuint((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
char FAR * AILEXPORT  AIL_last_error               (void)
{
   char FAR *result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_last_error()" NL);
      }

   IN_AIL_NM;

   result=ailcall0(AIL,last_error,char FAR*);

   OUT_AIL_NM;

   RESULT
      {
      outresstr(result);
      }

   END;

   return (char FAR*) result;
}

//############################################################################

DXDEF
void AILEXPORT  AIL_set_error               (char const FAR* error_msg)
{
   START
      {
      if (error_msg == NULL)
         {
         AIL_fprintf(AIL_debugfile,"AIL_set_error(NULL)" NL);
         }
      else
         {
         AIL_fprintf(AIL_debugfile,"AIL_set_error(%s)" NL, error_msg);
         }
      }

   IN_AIL_NM;

   ailcall1(AIL,set_error,void,error_msg);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_delay                     (S32        intervals)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_delay(%ld)" NL,intervals);
      }

   IN_AIL_NM;

#ifdef IS_WIN32API

   Sleep(intervals * 100 / 6);

#else

   AIL_API_delay(intervals);

#endif

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
S32    AILEXPORT  AIL_background                (void)
{
   S32 result;

   IN_AIL_NM;

   result=AIL_low_background();

   OUT_AIL_NM;

   return result;
}

//############################################################################

DXDEF
HTIMER  AILEXPORT  AIL_register_timer            (AILTIMERCB    callback_fn)
{
   HTIMER result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_timer(" PTRFMT ")" NL,callback_fn);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,register_timer,HTIMER,callback_fn);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
UINTa AILEXPORT  AIL_set_timer_user            (HTIMER      timer, //)
                                                UINTa       user)
{
   UINTa result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_timer_user(%ld," SINTFMT ")" NL,timer,user);
      }

   IN_AIL;

   result=ailcall2(AIL,set_timer_user,UINTa,timer,user);

   OUT_AIL;

   RESULT
      {
      outresuint((UINTa) result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_set_timer_period          (HTIMER      timer, //)
                                               U32         microseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_timer_period(%ld,%lu)" NL,timer,microseconds);
      }

   IN_AIL;

   ailcall2(AIL,set_timer_period,void,timer,microseconds);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_set_timer_frequency       (HTIMER      timer, //)
                                               U32         hertz)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_timer_frequency(%ld,%lu)" NL,timer,hertz);
      }

   IN_AIL;

   ailcall2(AIL,set_timer_frequency,void,timer,hertz);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_set_timer_divisor         (HTIMER      timer, //)
                                               U32         PIT_divisor)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_timer_divisor(%ld,%lu)" NL,timer,PIT_divisor);
      }

   IN_AIL;

   ailcall2(AIL,set_timer_divisor,void,timer,PIT_divisor);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_start_timer               (HTIMER      timer)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_start_timer(%lu)" NL,timer);
      }

   IN_AIL;

   ailcall1(AIL,start_timer,void,timer);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_start_all_timers          (void)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_start_all_timers()" NL);
      }

   IN_AIL;

   ailcall0(AIL,start_all_timers,void);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_stop_timer                (HTIMER      timer)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stop_timer(%lu)" NL,timer);
      }

   IN_AIL;

   ailcall1(AIL,stop_timer,void,timer);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_stop_all_timers           (void)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stop_all_timers()" NL);
      }

   IN_AIL;

   ailcall0(AIL,stop_all_timers,void);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_release_timer_handle      (HTIMER      timer)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_release_timer_handle(%lu)" NL,timer);
      }

   IN_AIL;

   ailcall1(AIL,release_timer_handle,void,timer);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void    AILEXPORT  AIL_release_all_timers        (void)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_release_all_timers()" NL);
      }

   IN_AIL;

   ailcall0(AIL,release_all_timers,void);

   OUT_AIL;

   END;
}

//############################################################################

#ifdef IS_WIN32

//
// (Not currently exported as a separate API call -- accessible only 
// via AIL_open_digital_driver())
//

static HDIGDRIVER AIL_open_multichannel_driver( U32         frequency,
                                                S32         bits,
                                                MSS_MC_SPEC channel_spec,
                                                U32         flags )
{
   HDIGDRIVER result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_open_digital_driver(%ld, %ld, %ld, 0x%lX)" NL,
                                                         frequency,
                                                         bits,
                                                         channel_spec,
                                                         flags);
      }

   IN_AIL;

#ifdef IS_WIN32
   {
   WAVEFORMATEXTENSIBLE wf;
   S32                  ds_emulated_ok = 0;
   MSSVECTOR3D          SPKR[MAX_SPEAKERS];

   AIL_memset(&wf,
               0,
               sizeof(wf));

   wf.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;    
   wf.Format.cbSize     = 22;     
   wf.SubFormat         = KSDATAFORMAT_SUBTYPE_PCM;

   if (channel_spec == MSS_MC_USE_SYSTEM_CONFIG)
      {
      if (!get_system_speaker_configuration(&channel_spec))
         {
         goto bail;
         }
      }

   switch (channel_spec)
      {
      //
      // Turn channel_spec into wf.Format.nChannels, wf.dwChannelMask, default speaker positions
      //
      // Low byte = # of physical (DAC) channels
      // High byte = # of logical (source) channels
      //
      // Formats that use voice-output filters (like DirectSound3D) should specify zero logical channels,
      // to indicate that the core output system is not used
      //
      
      case MSS_MC_DIRECTSOUND3D:
      case MSS_MC_EAX2:
      case MSS_MC_EAX3:
      case MSS_MC_EAX4:
         {
         AIL_set_preference(DIG_DS_DSBCAPS_CTRL3D, TRUE);
         AIL_set_preference(DIG_USE_WAVEOUT,       FALSE);
         AIL_set_preference(DIG_DS_USE_PRIMARY,    NO);

         wf.Format.nChannels = 0x0;
         break;
         }

      case MSS_MC_MONO:
         {
         wf.Format.nChannels = 0x101;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT;
         break;
         }

      case MSS_MC_STEREO:
         {
         wf.Format.nChannels = 0x202;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT + 
                            SPEAKER_FRONT_RIGHT;

         SPKR[0].x = -0.7F; SPKR[0].y = 0.0F; SPKR[0].z = 0.7F;
         SPKR[1].x =  0.7F; SPKR[1].y = 0.0F; SPKR[1].z = 0.7F;
         break;
         }

      case MSS_MC_HEADPHONES:   
         {
         wf.Format.nChannels = 0x202;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT + 
                            SPEAKER_FRONT_RIGHT;

         SPKR[0].x = -1.0F; SPKR[0].y = 0.0F; SPKR[0].z = 0.0F;
         SPKR[1].x =  1.0F; SPKR[1].y = 0.0F; SPKR[1].z = 0.0F;
         break;
         }

      case MSS_MC_40_DTS:
         {
         wf.Format.nChannels = 0x402;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT    + 
                            SPEAKER_FRONT_RIGHT;
         break;
         }

      case MSS_MC_40_DISCRETE:           
         {
         wf.Format.nChannels = 0x404;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT  + 
                            SPEAKER_FRONT_RIGHT + 
                            SPEAKER_BACK_LEFT   +
                            SPEAKER_BACK_RIGHT;
         break;
         }

      case MSS_MC_DOLBY_SURROUND:  
         {
         wf.Format.nChannels = 0x302;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT    + 
                            SPEAKER_FRONT_RIGHT;
         break;
         }

      case MSS_MC_SRS_CIRCLE_SURROUND:
         {
         wf.Format.nChannels = 0x702;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT    + 
                            SPEAKER_FRONT_RIGHT;
         break;
         }

      case MSS_MC_51_DTS:
         {
         wf.Format.nChannels = 0x602;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT    + 
                            SPEAKER_FRONT_RIGHT;
         break;
         }

      case MSS_MC_51_DISCRETE:  
         {
         wf.Format.nChannels = 0x606;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT    + 
                            SPEAKER_FRONT_RIGHT   + 
                            SPEAKER_FRONT_CENTER  +
                            SPEAKER_LOW_FREQUENCY +
                            SPEAKER_BACK_LEFT     +
                            SPEAKER_BACK_RIGHT;
         break;
         }

      case MSS_MC_61_DISCRETE:  
         {
         wf.Format.nChannels = 0x707;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT    + 
                            SPEAKER_FRONT_RIGHT   + 
                            SPEAKER_FRONT_CENTER  +
                            SPEAKER_LOW_FREQUENCY +
                            SPEAKER_BACK_LEFT     +
                            SPEAKER_BACK_RIGHT    + 
                            SPEAKER_BACK_CENTER;
         break;
         }

      case MSS_MC_71_DISCRETE:           
         {
         wf.Format.nChannels = 0x808;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT    + 
                            SPEAKER_FRONT_RIGHT   + 
                            SPEAKER_FRONT_CENTER  +
                            SPEAKER_LOW_FREQUENCY +
                            SPEAKER_BACK_LEFT     +
                            SPEAKER_BACK_RIGHT    +
                            SPEAKER_SIDE_LEFT     +
                            SPEAKER_SIDE_RIGHT;
         break;
         }

      case MSS_MC_81_DISCRETE:
         {
         wf.Format.nChannels = 0x909;

         wf.dwChannelMask = SPEAKER_FRONT_LEFT    + 
                            SPEAKER_FRONT_RIGHT   + 
                            SPEAKER_FRONT_CENTER  +
                            SPEAKER_LOW_FREQUENCY +
                            SPEAKER_BACK_LEFT     +
                            SPEAKER_BACK_RIGHT    +
                            SPEAKER_BACK_CENTER   +
                            SPEAKER_SIDE_LEFT     +
                            SPEAKER_SIDE_RIGHT;
         break;
         }

      default:
         {
         //
         // Invalid enum submitted
         //

         AIL_set_error("Unsupported output format (MSS_MC_SPEC enum required)");
         goto bail;
         }
      }

   wf.Format.nSamplesPerSec       = (frequency) ? frequency : 22050;
   wf.Format.wBitsPerSample       = (bits == 8) ? 8 : 16;
   wf.Samples.wValidBitsPerSample = wf.Format.wBitsPerSample;

   wf.Format.nBlockAlign          = (wf.Format.nChannels == 0) ? 
                                     2 : ((wf.Format.wBitsPerSample / 8) * (wf.Format.nChannels & 0xff));

   wf.Format.nAvgBytesPerSec      = (U32) wf.Format.nSamplesPerSec * (U32) wf.Format.nBlockAlign;

   if ((flags & AIL_OPEN_DIGITAL_FORCE_PREFERENCE) == 0)
      {
      AIL_set_preference(DIG_USE_WAVEOUT,FALSE);
      }

   if (AIL_API_waveOutOpen(&result,0,(LPWAVEFORMAT) &wf) == 0)
      {
      if ((result->emulated_ds) && ((flags & AIL_OPEN_DIGITAL_FORCE_PREFERENCE) == 0))
         {
         OutMilesMutex();
         AIL_API_waveOutClose(result);
         InMilesMutex();

         ds_emulated_ok = 1;
         goto trywaveout;
         }
      }
   else
      {
      if ((flags & AIL_OPEN_DIGITAL_FORCE_PREFERENCE) == 0)
         {
trywaveout:
         AIL_set_preference(DIG_USE_WAVEOUT,TRUE);
tryagain:
         if (AIL_API_waveOutOpen(&result, 0, (LPWAVEFORMAT) &wf))
            {
            // if waveout failed, but emulated suceeded, go back to directsound

            if ( ds_emulated_ok )
               {
               ds_emulated_ok=0;
               AIL_set_preference(DIG_USE_WAVEOUT,FALSE);
               goto tryagain;
               }

            result=0;
            }
         }
      else  
         {
         result=0;
         }
      }

   //
   // Perform format-specific operations (such as filter installation) after 
   // driver has been created
   //

   if (result != 0)
      {
      S32 filter_OK = 1;

      result->channel_spec = channel_spec;

      switch (channel_spec)
         {
         case MSS_MC_STEREO:
         case MSS_MC_HEADPHONES:
            {
            //
            // Use SPKR array to distinguish between speakers and headphones,
            // keeping the default falloff power established when the driver was opened 
            //
            // Otherwise, the default speaker positions are set up at driver-allocation time
            //

            AIL_set_speaker_configuration(result, 
                                          SPKR, 
                                          2, 
                                          result->D3D.falloff_power);
            break;
            }

         case MSS_MC_40_DTS:
         case MSS_MC_51_DTS:
            {
            //
            // DTS requires its own postmix filter
            // (PS2 only for now, but hook installed for future use...)
            //

            filter_OK = MC_open_output_filter("DTS 4.0/5.1 Matrix Filter", result, TRUE);
            break;
            }

         case MSS_MC_SRS_CIRCLE_SURROUND:
            {
            //
            // SRS requires its own postmix filter
            //

            filter_OK = MC_open_output_filter("SRS Circle Surround 6.1 Matrix Filter", result, TRUE);
            break;
            }

         case MSS_MC_DOLBY_SURROUND:
            {
            //
            // Dolby Surround requires its own postmix filter
            //

            filter_OK = MC_open_output_filter("Dolby Surround Matrix Filter", result, TRUE);
            break;
            }

         //
         // Each DirectSound3D provider maintains its own secondary buffer for each voice
         // via a per-voice output filter
         //

         case MSS_MC_DIRECTSOUND3D:
            {
            filter_OK = MC_open_output_filter("DirectSound3D Hardware Support", result, FALSE);
            break;
            }

         case MSS_MC_EAX2:
            {
            filter_OK = MC_open_output_filter("Creative Labs EAX 2 (TM)", result, FALSE);
            break;
            }

         case MSS_MC_EAX3:
            {
            filter_OK = MC_open_output_filter("Creative Labs EAX 3 (TM)", result, FALSE);
            break;
            }

         case MSS_MC_EAX4:
            {
            filter_OK = MC_open_output_filter("Creative Labs EAX 4 (TM)", result, FALSE);
            break;
            }
         }

      if (!filter_OK)
         {
         //
         // A required matrix filter or voice filter could not be associated with the driver
         // Close the driver handle and return failure
         //

         OutMilesMutex();
         AIL_API_waveOutClose(result);
         InMilesMutex();

         result = 0;
         }
      }

bail:
   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;
   }
#endif

   return result;
}

#endif


//############################################################################

#ifdef IS_PS2

//
// (Not currently exported as a separate API call -- accessible only 
// via AIL_open_digital_driver())
//

static HDIGDRIVER AIL_open_multichannel_driver( U32         frequency,
                                                S32         bits,
                                                MSS_MC_SPEC channel_spec,
                                                U32         flags )
{
   HDIGDRIVER result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_open_digital_driver(%ld, %ld, %ld, 0x%lX)" NL,
                                                         frequency,
                                                         bits,
                                                         channel_spec,
                                                         flags);
      }

   IN_AIL;

   {
   MSSVECTOR3D SPKR[MAX_SPEAKERS];

   if ((channel_spec == MSS_MC_USE_SYSTEM_CONFIG) || (channel_spec == MSS_MC_MONO))
      {
      channel_spec = MSS_MC_STEREO;
      }

   switch (channel_spec)
      {
      case MSS_MC_STEREO:
         {
         SPKR[0].x = -0.7F; SPKR[0].y = 0.0F; SPKR[0].z = 0.7F;
         SPKR[1].x =  0.7F; SPKR[1].y = 0.0F; SPKR[1].z = 0.7F;
         break;
         }

      case MSS_MC_HEADPHONES:   
         {
         SPKR[0].x = -1.0F; SPKR[0].y = 0.0F; SPKR[0].z = 0.0F;
         SPKR[1].x =  1.0F; SPKR[1].y = 0.0F; SPKR[1].z = 0.0F;
         break;
         }

      case MSS_MC_DOLBY_SURROUND:  
         {
         break;
         }

      case MSS_MC_SRS_CIRCLE_SURROUND:
         {
         break;
         }

      case MSS_MC_40_DTS:
      case MSS_MC_51_DTS:
         {
         flags |= AIL_OPEN_DIGITAL_USE_IOP_CORE0;
         break;
         }

      default:
         {
         //
         // Invalid enum submitted
         //

         AIL_set_error("Unsupported output format (MSS_MC_SPEC enum required)");
         goto bail;
         }
      }

   result = AIL_API_open_digital_driver( frequency, bits, channel_spec, flags );

   //
   // Perform format-specific operations (such as filter installation) after 
   // driver has been created
   //

   if (result != 0)
      {
      S32 filter_OK = 1;

      result->channel_spec = channel_spec;

      switch (channel_spec)
         {
         case MSS_MC_STEREO:
         case MSS_MC_HEADPHONES:
            {
            //
            // Use SPKR array to distinguish between speakers and headphones,
            // keeping the default falloff power established when the driver was opened 
            //
            // Otherwise, the default speaker positions are set up at driver-allocation time
            //

            AIL_set_speaker_configuration(result, 
                                          SPKR, 
                                          2, 
                                          result->D3D.falloff_power);
            break;
            }

         case MSS_MC_40_DTS:
         case MSS_MC_51_DTS:
            {
            //
            // DTS requires its own postmix filter
            //

            filter_OK = MC_open_output_filter("DTS 4.0/5.1 Matrix Filter", result, TRUE);
            break;
            }

         case MSS_MC_SRS_CIRCLE_SURROUND:
            {
            //
            // SRS requires its own postmix filter
            //

            filter_OK = MC_open_output_filter("SRS Circle Surround 6.1 Matrix Filter", result, TRUE);
            break;
            }

         case MSS_MC_DOLBY_SURROUND:
            {
            //
            // Dolby Surround requires its own postmix filter
            //

            filter_OK = MC_open_output_filter("Dolby Surround Matrix Filter", result, TRUE);
            break;
            }

         default:
            break;
         }

      if (!filter_OK)
         {
         //
         // A required matrix filter or voice filter could not be associated with the driver
         // Close the driver handle and return failure
         //

         OutMilesMutex();
         AIL_API_close_digital_driver(result);
         InMilesMutex();

         result = 0;
         }
      }

bail:
   OUT_AIL;

   RESULT
      {
      outreshex((U32)result);
      }

   END;
   }

   return result;
}

#endif

//############################################################################

DXDEF HDIGDRIVER AILEXPORT AIL_open_digital_driver( U32 frequency,
                                                    S32 bits,
                                                    S32 channels,
                                                    U32 flags )
{
   HDIGDRIVER result;
   
#if defined(IS_XENON)
   channels = MSS_MC_51_DISCRETE;   // TODO: headphone support on Xenon
#endif

#if defined(IS_WIN32) || defined(IS_PS2)

   //
   // In Win32, channel specs beyond mono/stereo require the creation of a 
   // WAVEFORMATEXTENSIBLE structure rather than a PCMWAVEFORMAT structure.  For maximum
   // compatibility with older/legacy systems, we'll avoid calling the MC-compatible
   // version for channel specs that don't require it
   //
   // PS2 also goes through this path for SRS/Dolby/DTS support
   //

   if (channels > 2)
      {
      result = AIL_open_multichannel_driver(frequency,
                                            bits,
                              (MSS_MC_SPEC) channels,
                                            flags);
      return result;
      }
#endif

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_open_digital_driver(%ld, %ld, %ld, 0x%lX)" NL,
                                                             frequency,
                                                             bits,
                                                             channels,
                                                             flags);
      }

   IN_AIL;

   #if defined(IS_WINDOWS)

   {
     PCMWAVEFORMAT wf;

     #ifdef IS_WIN32
     S32 ds_emulated_ok = 0;
     #endif

     wf.wf.wFormatTag      = WAVE_FORMAT_PCM;
     wf.wf.nChannels       = (channels==1)?1:2;
     wf.wf.nSamplesPerSec  = (frequency)?frequency:22050;
     wf.wBitsPerSample     = (bits==8)?8:16;

     wf.wf.nBlockAlign     = (wf.wBitsPerSample / 8) * wf.wf.nChannels;
     wf.wf.nAvgBytesPerSec = (U32)wf.wf.nSamplesPerSec * (U32)wf.wf.nBlockAlign;

     if ((flags&AIL_OPEN_DIGITAL_FORCE_PREFERENCE)==0)
       AIL_set_preference(DIG_USE_WAVEOUT,FALSE);

     if (AIL_API_waveOutOpen(&result,0,(LPWAVEFORMAT)&wf)==0)
     {
       #ifdef IS_WIN32
       if ((result->emulated_ds) && ((flags&AIL_OPEN_DIGITAL_FORCE_PREFERENCE)==0))
       {
         OutMilesMutex();
         AIL_API_waveOutClose(result);
         InMilesMutex();
         ds_emulated_ok = 1;
         goto trywaveout;
       }
       #endif
     }
     else
     {
       #ifdef IS_WIN32
       if ((flags&AIL_OPEN_DIGITAL_FORCE_PREFERENCE)==0)
       {
        trywaveout:
         AIL_set_preference(DIG_USE_WAVEOUT,TRUE);
        tryagain:
         if (AIL_API_waveOutOpen(&result,0,(LPWAVEFORMAT)&wf))
         {
           // if waveout failed, but emulated suceeded, go back to directsound
           if ( ds_emulated_ok )
           {
             ds_emulated_ok=0;
             AIL_set_preference(DIG_USE_WAVEOUT,FALSE);
             goto tryagain;
           }
           result=0;
         }
       }
       else
       #endif
         result=0;
     }
   }

   #else

   #ifdef IS_DOS

   if ((flags&AIL_OPEN_DIGITAL_FORCE_PREFERENCE)==0)
   {
     if (frequency)
       AIL_set_preference(DIG_HARDWARE_SAMPLE_RATE, frequency );
     if (bits)
       AIL_set_preference(DIG_USE_16_BITS, (bits>8)?TRUE:FALSE );
     if (channels)
       AIL_set_preference(DIG_USE_STEREO, (channels>1)?TRUE:FALSE );
   }
   if (AIL_install_DIG_INI(&result)!=AIL_INIT_SUCCESS)
     result=0;

   #else

   // Mac, PS3, Xbox, Xenon, Linux, Wii

   result = AIL_API_open_digital_driver( frequency, bits, channels, flags );

#ifdef IS_WII
   if ( result )
   {
     if ( result->logical_channels_per_sample == 3 )
     {
       if ( ! MC_open_output_filter("Dolby Surround Matrix Filter", result, TRUE) )
         goto wii_flt_err;
     } 
     else if ( result->logical_channels_per_sample == 7 )
     {
       if ( ! MC_open_output_filter("SRS Circle Surround 6.1 Matrix Filter", result, TRUE) )
       {
        wii_flt_err:
         AIL_API_close_digital_driver(result);
         result = 0;
       }
     } 
   }      
#endif


   #endif
   #endif

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   if (result)
      {
      result->channel_spec = (MSS_MC_SPEC)channels;
      }

   return result;
}

DXDEF void AILEXPORT AIL_close_digital_driver( HDIGDRIVER dig )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_close_digital_driver(" PTRFMT ")" NL,dig);
      }

   IN_AIL_NM;

   #ifdef IS_WINDOWS
   dsailcall1(AIL,waveOutClose,void,dig);
   #else
   #ifdef IS_DOS
   AIL_API_uninstall_DIG_driver(dig)
   #else
   AIL_API_close_digital_driver(dig);
   #endif
   #endif

   OUT_AIL_NM;

   END;
}

#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX)

//############################################################################

DXDEF
S32         AILEXPORT AIL_digital_handle_release      (HDIGDRIVER dig)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_handle_release(" PTRFMT ")" NL,dig);
      }

   IN_AIL;

   result=dsailcall1(AIL,digital_handle_release,S32,dig);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_digital_handle_reacquire    (HDIGDRIVER dig)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_handle_reacquire(" PTRFMT ")" NL,dig);
      }

   IN_AIL;

   result=dsailcall1(AIL,digital_handle_reacquire,S32,dig);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF HDIGINPUT AILEXPORT AIL_open_input          (AIL_INPUT_INFO FAR *info)
{
   HDIGINPUT result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_open_input(" PTRFMT ")" NL,info);
      }

   IN_AIL;

   result=ailcall1(AIL,open_input,HDIGINPUT,info);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF void      AILEXPORT AIL_close_input         (HDIGINPUT         dig)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_close_input(" PTRFMT ")" NL,dig);
      }

   IN_AIL;

   ailcall1(AIL,close_input,void,dig);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF AIL_INPUT_INFO FAR * AILEXPORT AIL_get_input_info (HDIGINPUT         dig)
{
   AIL_INPUT_INFO FAR *result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_get_input_info(" PTRFMT ")" NL,dig);
      }

   IN_AIL;

   result=ailcall1(AIL,get_input_info,AIL_INPUT_INFO FAR *,dig);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF S32       AILEXPORT AIL_set_input_state     (HDIGINPUT         dig, //)
                                                   S32               enable)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_input_state(" PTRFMT ",%ld)" NL,dig,enable);
      }

   IN_AIL;

   result=ailcall2(AIL,set_input_state,S32,dig,enable);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

#endif

//############################################################################

DXDEF
S32         AILEXPORT AIL_digital_CPU_percent(HDIGDRIVER dig)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_CPU_percent(" PTRFMT ")" NL,dig);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,digital_CPU_percent,S32,dig);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_background_CPU_percent(void)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_background_CPU_percent()" NL);
      }

   IN_AIL_NM;

#ifdef IS_DOS
   result = 0;
#else
   result=ailcall0(AIL,background_CPU_percent,S32);
#endif

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_digital_latency(HDIGDRIVER dig)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_latency(" PTRFMT ")" NL,dig);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,digital_latency,S32,dig);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
HSAMPLE      AILEXPORT AIL_allocate_sample_handle (HDIGDRIVER dig)

{
   HSAMPLE result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_allocate_sample_handle(" PTRFMT ")" NL,dig);
      }

   IN_AIL;

   result=dsailcall1(AIL,allocate_sample_handle,HSAMPLE,dig);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_release_sample_handle (HSAMPLE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_release_sample_handle(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   dsailcall1(AIL,release_sample_handle,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_init_sample        (HSAMPLE S, //)
                                               S32     format,
                                               U32     flags)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_init_sample(" PTRFMT ",%ld,%lu)" NL,S,format,flags);
      }

   IN_AIL;

   dsailcall3(AIL,init_sample,void,S,format,flags);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
S32        AILEXPORT AIL_set_sample_file        (HSAMPLE   S, //)
                                              void const FAR *file_image,
                                              S32       block)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_file(" PTRFMT "," PTRFMT ",%ld)" NL,S,file_image,block);
      }

   IN_AIL;

   result=ailcall3(AIL,set_sample_file,S32,S,file_image,block);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32        AILEXPORT AIL_set_sample_info     (HSAMPLE   S, //)
                                              AILSOUNDINFO const FAR *info )
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_info(" PTRFMT "," PTRFMT ")" NL,S,info);
      }

   IN_AIL;

   result=ailcall2(AIL,set_sample_info,S32,S,info);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32          AILEXPORT AIL_set_named_sample_file (HSAMPLE   S, //)
                                                  C8   const FAR *file_type_suffix,
                                                  void const FAR *file_image,
                                                  U32       file_size,
                                                  S32       block)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_named_sample_file(" PTRFMT ",%s," PTRFMT ",%ld,%ld)" NL,
         S,file_type_suffix,file_image,file_size,block);
      }

   IN_AIL;

   result=ailcall5(AIL,set_named_sample_file,U32,S,file_type_suffix,file_image,file_size,block);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void        AILEXPORT AIL_save_sample_attributes (HSAMPLE   S, //)
                                                  HSATTRIBS D)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_save_sample_attributes(" PTRFMT "," PTRFMT ")" NL,S,D);
      }

   IN_AIL;

   ailcall2(AIL,save_sample_attributes,void,S,D);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
U32        AILEXPORT AIL_load_sample_attributes (HSAMPLE   S, //)
                                                 HSATTRIBS D)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_load_sample_attributes(" PTRFMT "," PTRFMT ")" NL,S,D);
      }

   IN_AIL;

   result=ailcall2(AIL,load_sample_attributes,S32,S,D);

   OUT_AIL;

   RESULT
      {
      outresint(result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
HPROVIDER    AILEXPORT AIL_set_sample_processor  (HSAMPLE     S, //)
                                                SAMPLESTAGE pipeline_stage,
                                                HPROVIDER   provider)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_processor(" PTRFMT ",%ld," HPFMT ")" NL,
         S,pipeline_stage,provider);
      }

   IN_AIL;

   result=ailcall3(AIL,set_sample_processor,HPROVIDER,S,pipeline_stage,provider);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
HPROVIDER    AILEXPORT AIL_sample_processor  (HSAMPLE     S, //)
                                            SAMPLESTAGE pipeline_stage)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_processor(" PTRFMT ",%ld)" NL,
         S,pipeline_stage);
      }

   IN_AIL;

   result=ailcall2(AIL,sample_processor,HPROVIDER,S,pipeline_stage);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
HPROVIDER    AILEXPORT AIL_set_digital_driver_processor  (HDIGDRIVER  dig, //)
                                                        DIGDRVSTAGE pipeline_stage,
                                                        HPROVIDER   provider)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_digital_driver_processor(" PTRFMT ",%ld," HPFMT ")" NL,
         dig,pipeline_stage,provider);
      }

   IN_AIL;

   result=ailcall3(AIL,set_digital_driver_processor,HPROVIDER,dig,pipeline_stage,provider);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
HPROVIDER  AILEXPORT AIL_digital_driver_processor  (HDIGDRIVER  dig, //)
                                                    DIGDRVSTAGE pipeline_stage)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_driver_processor(" PTRFMT ",%ld)" NL,
         dig,pipeline_stage);
      }

   IN_AIL;

   result=ailcall2(AIL,digital_driver_processor,HPROVIDER,dig,pipeline_stage);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_address    (HSAMPLE S, //)
                                               void const    FAR *start,
                                               U32     len)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_address(" PTRFMT "," PTRFMT ",%lu)" NL,S,start,len);
      }

   IN_AIL;

   dsailcall3(AIL,set_sample_address,void,S,start,len);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_adpcm_block_size(HSAMPLE S, //)
                                                       U32 blocksize)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_adpcm_block_size(" PTRFMT ",%lu)" NL,S,blocksize);
      }

   IN_AIL;

   dsailcall2(AIL,set_sample_adpcm_block_size,void,S,blocksize);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_start_sample          (HSAMPLE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_start_sample(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   dsailcall1(AIL,start_sample,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_stop_sample           (HSAMPLE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stop_sample(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   dsailcall1(AIL,stop_sample,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_resume_sample         (HSAMPLE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_resume_sample(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   dsailcall1(AIL,resume_sample,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_end_sample            (HSAMPLE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_end_sample(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   dsailcall1(AIL,end_sample,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_playback_rate//()
                                             (HSAMPLE S,
                                              S32     playback_rate)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_playback_rate(" PTRFMT ",%ld)" NL,S,playback_rate);
      }

   IN_AIL;

   dsailcall2(AIL,set_sample_playback_rate,void,S,playback_rate);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  
void         AILEXPORT AIL_set_sample_channel_levels (HSAMPLE S, //)
                                                      F32 FAR *level_array,
                                                      S32  n_levels)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_channel_levels(" PTRFMT "," PTRFMT ",%d)" NL,S,level_array,n_levels);
      }

   IN_AIL;

   dsailcall3(AIL,set_sample_channel_levels,void,S,level_array,n_levels);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  
F32 FAR *    AILEXPORT AIL_sample_channel_levels (HSAMPLE  S, //)
                                                  S32 FAR *n_levels)
{                         
   F32 FAR *result = NULL;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_channel_levels(" PTRFMT "," PTRFMT ")" NL,S,n_levels);
      }

   IN_AIL;

   result = dsailcall2(AIL,sample_channel_levels,F32 FAR *,S,n_levels);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF  
void         AILEXPORT AIL_set_speaker_configuration (HDIGDRIVER dig, //)
                                                      MSSVECTOR3D FAR *speaker_positions,
                                                      S32  n_channels,
                                                      F32  falloff_power)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_speaker_configuration(" PTRFMT "," PTRFMT ",%ld,%s)" NL,dig,speaker_positions,n_channels,FTS(falloff_power));
      }

   IN_AIL;        

   dsailcall4(AIL,set_speaker_configuration,void,dig,speaker_positions,n_channels,falloff_power);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  
MSSVECTOR3D FAR *    AILEXPORT AIL_speaker_configuration (HDIGDRIVER       dig, //)
                                                          S32         FAR *n_physical_channels,
                                                          S32         FAR *n_logical_channels,
                                                          F32         FAR *falloff_power,
                                                          MSS_MC_SPEC FAR *channel_spec)
{
   MSSVECTOR3D FAR *result = NULL;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_speaker_configuration(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,dig,n_physical_channels,n_logical_channels,falloff_power,channel_spec);
      }

   IN_AIL;

   result = dsailcall5(AIL,speaker_configuration,MSSVECTOR3D FAR *,dig,n_physical_channels,n_logical_channels,falloff_power,channel_spec);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF  
void AILEXPORT AIL_set_listener_relative_receiver_array (HDIGDRIVER               dig, //)
                                                         MSS_RECEIVER_LIST FAR *array,
                                                         S32                      n_receivers)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_listener_relative_receiver_array(" PTRFMT "," PTRFMT ",%d)" NL,dig,array,n_receivers);
      }

   IN_AIL;

   dsailcall3(AIL,set_listener_relative_receiver_array,void,dig,array,n_receivers);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  
MSS_RECEIVER_LIST FAR *    AILEXPORT AIL_listener_relative_receiver_array (HDIGDRIVER  dig, //)
                                                                           S32 FAR    *n_receivers)
{                                     
   MSS_RECEIVER_LIST FAR *result = NULL;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_listener_relative_receiver_array(" PTRFMT "," PTRFMT ")" NL,dig,n_receivers);
      }

   IN_AIL;

   result = dsailcall2(AIL,listener_relative_receiver_array,MSS_RECEIVER_LIST FAR *,dig,n_receivers);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

#ifndef IS_XBOX

//############################################################################
DXDEF  
void  AILEXPORT AIL_set_speaker_reverb_levels (HDIGDRIVER dig, //)
                                               F32       *wet_array,
                                               F32       *dry_array,
                                               S32        n_levels)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_speaker_reverb_levels(" PTRFMT "," PTRFMT "," PTRFMT ",%d)" NL,dig,wet_array,dry_array,n_levels);
      }

   IN_AIL;

   dsailcall4(AIL,set_speaker_reverb_levels,void,dig,wet_array,dry_array,n_levels);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  
void    AILEXPORT AIL_speaker_reverb_levels (HDIGDRIVER     dig, //)
                                             S32 FAR       *n_levels,
                                             F32 FAR * FAR *wet_array,
                                             F32 FAR * FAR *dry_array)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_speaker_reverb_levels(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,dig,n_levels,wet_array,dry_array);
      }

   IN_AIL;

   dsailcall4(AIL,speaker_reverb_levels,void,dig,n_levels,wet_array,dry_array);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  
S32   AILEXPORT AIL_calculate_3D_channel_levels      (HDIGDRIVER       dig, //)
                                                      F32         FAR *channel_levels,
                                                      MSSVECTOR3D FAR *src_pos,
                                                      MSSVECTOR3D FAR *src_face,
                                                      MSSVECTOR3D FAR *src_up,
                                                      F32              src_inner_angle,
                                                      F32              src_outer_angle,
                                                      F32              src_outer_volume,
                                                      F32              src_max_dist,
                                                      F32              src_min_dist,
                                                      MSSVECTOR3D FAR *listen_pos, 
                                                      MSSVECTOR3D FAR *listen_face,
                                                      MSSVECTOR3D FAR *listen_up,  
                                                      F32              rolloff_factor,
                                                      MSSVECTOR3D FAR *doppler_velocity,
                                                      F32             *doppler_shift)
{
   S32 result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_calculate_3D_channel_levels(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ",%s,%s,%s,%s,%s," PTRFMT "," PTRFMT "," PTRFMT ",%s," PTRFMT ",%s)" NL,
         dig,
         channel_levels,    
         src_pos,           
         src_face,          
         src_up,            
         FTS(src_inner_angle),   
         FTS(src_outer_angle),   
         FTS(src_outer_volume),  
         FTS(src_max_dist),      
         FTS(src_min_dist),      
         listen_pos,        
         listen_face,       
         listen_up,         
         FTS(rolloff_factor),
         doppler_velocity,
         doppler_shift);
      }

   IN_AIL;

   result = dsailcall16(AIL,calculate_3D_channel_levels,S32,
         dig,
         channel_levels,    
         src_pos,           
         src_face,          
         src_up,            
         src_inner_angle,   
         src_outer_angle,   
         src_outer_volume,  
         src_max_dist,      
         src_min_dist,      
         listen_pos,        
         listen_face,       
         listen_up,         
         rolloff_factor,
         doppler_velocity,
         doppler_shift);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

#endif

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_volume_pan  (HSAMPLE S, //)
                                                   F32     volume,
                                                   F32     pan)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_volume_pan(" PTRFMT ",%s,%s)" NL,S,FTS(volume),FTS(pan));
      }

   IN_AIL;

   dsailcall3(AIL,set_sample_volume_pan,void,S,volume,pan);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_51_volume_pan  (HSAMPLE S, //)
                                                      F32     volume,
                                                      F32     pan,
                                                      F32     fb_pan,
                                                      F32     center_level,
                                                      F32     sub_level )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_51_volume_pan(" PTRFMT ",%s,%s,%s,%s,%s)" NL,S,FTS(volume),FTS(pan),FTS(fb_pan),FTS(center_level),FTS(sub_level));
      }

   IN_AIL;

   dsailcall6(AIL,set_sample_51_volume_pan,void,S,volume,pan,fb_pan,center_level,sub_level);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_sample_51_volume_pan  (HSAMPLE   S, //)
                                                  F32 FAR * volume,
                                                  F32 FAR * pan,
                                                  F32 FAR * fb_pan,
                                                  F32 FAR * center_level,
                                                  F32 FAR * sub_level )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_51_volume_pan(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,volume,pan,fb_pan,center_level,sub_level);
      }

   IN_AIL_NM;

   dsailcall6(AIL,sample_51_volume_pan,void,S,volume,pan,fb_pan,center_level,sub_level);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_51_volume_levels (HSAMPLE S, //)
                                                        F32     f_left_level,
                                                        F32     f_right_level,
                                                        F32     b_left_level,
                                                        F32     b_right_level,
                                                        F32     center_level,
                                                        F32     sub_level )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_51_volume_levels(" PTRFMT ",%s,%s,%s,%s,%s)" NL,S,FTS(f_left_level),FTS(f_right_level),FTS(b_left_level),FTS(b_right_level),FTS(center_level),FTS(sub_level));
      }

   IN_AIL;

   dsailcall7(AIL,set_sample_51_volume_levels,void,S,f_left_level,f_right_level,b_left_level,b_right_level,center_level,sub_level);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_sample_51_volume_levels (HSAMPLE   S, //)
                                                    F32 FAR * f_left_level,
                                                    F32 FAR * f_right_level,
                                                    F32 FAR * b_left_level,
                                                    F32 FAR * b_right_level,
                                                    F32 FAR * center_level,
                                                    F32 FAR * sub_level )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_51_volume_levels(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,f_left_level,f_right_level,b_left_level,b_right_level,center_level,sub_level);
      }

   IN_AIL_NM;

   dsailcall7(AIL,sample_51_volume_levels,void,S,f_left_level,f_right_level,b_left_level,b_right_level,center_level,sub_level);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_volume_levels  (HSAMPLE S, //)
                                                      F32     left_level,
                                                      F32     right_level)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_volume_levels(" PTRFMT ",%s,%s)" NL,S,FTS(left_level),FTS(right_level));
      }

   IN_AIL;

   dsailcall3(AIL,set_sample_volume_levels,void,S,left_level,right_level);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_reverb_levels  (HSAMPLE S, //)
                                                      F32     dry_level,
                                                      F32     wet_level)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_reverb_levels(" PTRFMT ",%s,%s)" NL,S,FTS(dry_level),FTS(wet_level));
      }

   IN_AIL;

   dsailcall3(AIL,set_sample_reverb_levels,void,S,dry_level,wet_level);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_low_pass_cut_off(HSAMPLE S, //)
                                                       F32     cut_off)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_low_pass_cut_off(" PTRFMT ",%s)" NL,S,FTS(cut_off));
      }

   IN_AIL;

   dsailcall2(AIL,set_sample_low_pass_cut_off,void,S,cut_off);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_loop_count (HSAMPLE S, //)
                                               S32     loop_count)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_loop_count(" PTRFMT ",%ld)" NL,S,loop_count);
      }

   IN_AIL;

   dsailcall2(AIL,set_sample_loop_count,void,S,loop_count);

   OUT_AIL;

   END;
}


//############################################################################

DXDEF
void         AILEXPORT AIL_set_sample_loop_block (HSAMPLE S,
                                                     S32     loop_start_offset,
                                                     S32     loop_end_offset)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_loop_block(" PTRFMT ",%ld,%ld)" NL,S,loop_start_offset,loop_end_offset);
      }

   IN_AIL;

   dsailcall3(AIL,set_sample_loop_block,void,S,loop_start_offset,loop_end_offset);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF
S32        AILEXPORT AIL_sample_loop_block   (HSAMPLE S, //)
                                              S32    *loop_start_offset,
                                              S32    *loop_end_offset)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_loop_block(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          S,loop_start_offset,loop_end_offset);
      }

   IN_AIL_NM;

   result = AIL_API_sample_loop_block(S,loop_start_offset,loop_end_offset);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}


//############################################################################

DXDEF
U32        AILEXPORT AIL_sample_status         (HSAMPLE S)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_status(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=dsailcall1(AIL,sample_status,U32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_sample_playback_rate  (HSAMPLE S)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_playback_rate(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=dsailcall1(AIL,sample_playback_rate,S32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_sample_volume_pan         (HSAMPLE S,F32 FAR*volume, F32 FAR* pan)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_volume_pan(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,volume,pan);
      }

   IN_AIL_NM;

   dsailcall3(AIL,sample_volume_pan,void,S,volume,pan);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_sample_volume_levels  (HSAMPLE S, //)
                                                  F32 FAR * left_level,
                                                  F32 FAR * right_level)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_volume_levels(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,left_level,right_level);
      }

   IN_AIL_NM;

   dsailcall3(AIL,sample_volume_levels,void,S,left_level,right_level);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_sample_reverb_levels  (HSAMPLE S, //)
                                                  F32 FAR * dry_level,
                                                  F32 FAR * wet_level)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_reverb_levels(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,dry_level,wet_level);
      }

   IN_AIL_NM;

   dsailcall3(AIL,sample_reverb_levels,void,S,dry_level,wet_level);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
F32         AILEXPORT AIL_sample_low_pass_cut_off(HSAMPLE S )
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_low_pass_cut_off(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=dsailcall1(AIL,sample_low_pass_cut_off,F32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresfloat(result);
      }

   END;

   return( result );
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_sample_loop_count     (HSAMPLE S)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_loop_count(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=dsailcall1(AIL,sample_loop_count,S32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_set_digital_master_volume_level
                                             (HDIGDRIVER dig,
                                              F32        master_volume)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_digital_master_volume_level(" PTRFMT ",%s)" NL,dig,FTS(master_volume));
      }

   IN_AIL;

   dsailcall2(AIL,set_digital_master_volume_level,void,dig,master_volume);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
F32     AILEXPORT AIL_digital_master_volume_level   (HDIGDRIVER dig)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_master_volume_level(" PTRFMT ")" NL,dig);
      }

   IN_AIL;

   result=dsailcall1(AIL,digital_master_volume_level,F32,dig);

   OUT_AIL;

   RESULT
      {
      outresfloat((F32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_digital_master_reverb_levels  (HDIGDRIVER dig, //)
                                                              F32     dry_level,
                                                              F32     wet_level)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_digital_master_reverb_levels(" PTRFMT ",%s,%s)" NL,dig,FTS(dry_level),FTS(wet_level));
      }

   IN_AIL;

   dsailcall3(AIL,set_digital_master_reverb_levels,void,dig,dry_level,wet_level);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_digital_master_reverb_levels  (HDIGDRIVER dig, //)
                                                          F32 FAR * dry_level,
                                                          F32 FAR * wet_level)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_master_reverb_levels(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,dig,dry_level,wet_level);
      }

   IN_AIL;

   dsailcall3(AIL,digital_master_reverb_levels,void,dig,dry_level,wet_level);

   OUT_AIL;

   END;
}

#ifndef IS_XBOX

//############################################################################

DXDEF
void         AILEXPORT AIL_set_digital_master_reverb         (HDIGDRIVER dig, //)
                                                              F32     reverb_time,
                                                              F32     reverb_predelay,
                                                              F32     reverb_damping)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_digital_master_reverb(" PTRFMT ",%s,%s,%s)" NL,dig,FTS(reverb_time),FTS(reverb_predelay),FTS(reverb_damping));
      }

   IN_AIL_NM;

   dsailcall4(AIL,set_digital_master_reverb,void,dig,reverb_time,reverb_predelay,reverb_damping);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_digital_master_reverb (HDIGDRIVER dig, //)
                                                  F32 FAR * reverb_time,
                                                  F32 FAR * reverb_predelay,
                                                  F32 FAR * reverb_damping)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_master_reverb(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,dig,reverb_time,reverb_predelay,reverb_damping);
      }

   IN_AIL_NM;

   dsailcall4(AIL,digital_master_reverb,void,dig,reverb_time,reverb_predelay,reverb_damping);

   OUT_AIL_NM;

   END;
}

#endif

//############################################################################

DXDEF
S32     AILEXPORT AIL_minimum_sample_buffer_size  (HDIGDRIVER dig, //)
                                                S32        playback_rate,
                                                S32        format)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_minimum_sample_buffer_size(" PTRFMT ",%ld,%ld)" NL,dig,
         playback_rate,format);
      }

   IN_AIL;

   result=dsailcall3(AIL,minimum_sample_buffer_size,S32,dig,playback_rate,format);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_sample_buffer_ready       (HSAMPLE S)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_buffer_ready(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=dsailcall1(AIL,sample_buffer_ready,S32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_load_sample_buffer        (HSAMPLE S, //)
                                               U32     buff_num,
                                               void const FAR *buffer,
                                               U32     len)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_load_sample_buffer(" PTRFMT ",%ld," PTRFMT ",%lu)" NL,
         S,buff_num,buffer,len);
      }

   IN_AIL;

   dsailcall4(AIL,load_sample_buffer,void,S,buff_num,buffer,len);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_sample_buffer_info      (HSAMPLE S, //)
            U32     FAR *pos0,
            U32     FAR *len0,
            U32     FAR *pos1,
            U32     FAR *len1)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_buffer_info(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
                 S,pos0,len0,pos1,len1);
      }

   IN_AIL;

   result=dsailcall5(AIL,sample_buffer_info,S32,S,pos0,len0,pos1,len1);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}


//############################################################################

DXDEF
U32    AILEXPORT AIL_sample_granularity        (HSAMPLE S)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_granularity(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=dsailcall1(AIL,sample_granularity,U32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_set_sample_position       (HSAMPLE S, //)
                                               U32     pos)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_position(" PTRFMT ",%lu)" NL,S,pos);
      }

   IN_AIL;

   dsailcall2(AIL,set_sample_position,void,S,pos);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
U32    AILEXPORT AIL_sample_position           (HSAMPLE S)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_position(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=dsailcall1(AIL,sample_position,U32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILSAMPLECB AILEXPORT AIL_register_SOB_callback     (HSAMPLE S, //)
                                                    AILSAMPLECB SOB)
{
   AILSAMPLECB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_SOB_callback(" PTRFMT "," PTRFMT ")" NL,S,SOB);
      }

   IN_AIL;

   result=dsailcall2(AIL,register_SOB_callback,AILSAMPLECB,S,SOB);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILSAMPLECB AILEXPORT AIL_register_EOB_callback     (HSAMPLE S, //)
                                                    AILSAMPLECB EOB)
{
   AILSAMPLECB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_EOB_callback(" PTRFMT "," PTRFMT ")" NL,S,EOB);
      }

   IN_AIL;

   result=dsailcall2(AIL,register_EOB_callback,AILSAMPLECB,S,EOB);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILSAMPLECB AILEXPORT AIL_register_EOS_callback     (HSAMPLE S, //)
                                                    AILSAMPLECB EOS)
{
   AILSAMPLECB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_EOS_callback(" PTRFMT "," PTRFMT ")" NL,S,EOS);
      }

   IN_AIL;

   result=dsailcall2(AIL,register_EOS_callback,AILSAMPLECB,S,EOS);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILFALLOFFCB AILEXPORT AIL_register_falloff_function_callback     (HSAMPLE S, //)
                                                                  AILFALLOFFCB falloff_cb)
{
   AILFALLOFFCB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_falloff_function_callback(" PTRFMT "," PTRFMT ")" NL,S,falloff_cb);
      }

   IN_AIL;

   result=dsailcall2(AIL,register_falloff_function_callback,AILFALLOFFCB,S,falloff_cb);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_set_sample_user_data      (HSAMPLE S, //)
                                                  U32     index,
                                                  SINTa   value)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_user_data(" PTRFMT ",%ld," SINTFMT ")" NL,S,index,value);
      }

   IN_AIL_NM;

   dsailcall3(AIL,set_sample_user_data,void,S,index,value);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
SINTa   AILEXPORT AIL_sample_user_data          (HSAMPLE S, //)
                                                 U32     index)
{
   SINTa result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_user_data(" PTRFMT ",%lu)" NL,S,index);
      }

   IN_AIL_NM;

   result=dsailcall2(AIL,sample_user_data,SINTa,S,index);

   OUT_AIL_NM;

   RESULT
      {
      outresuint((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_active_sample_count       (HDIGDRIVER dig)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_active_sample_count(" PTRFMT ")" NL,dig);
      }

   IN_AIL;

   result=dsailcall1(AIL,active_sample_count,S32,dig);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

void     AILEXPORT AIL_digital_configuration     (HDIGDRIVER dig,
                                               S32   FAR *rate,
                                               S32   FAR *format,
                                               char  FAR *config)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_configuration(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         dig,rate,format,config);
      }

   IN_AIL;

   dsailcall4(AIL,digital_configuration,void,dig,rate,format,config);

   OUT_AIL;

   END;
}


#if defined(IS_WIN32) || defined(IS_XBOX)

//############################################################################

DXDEF
void     AILEXPORT AIL_get_DirectSound_info (HSAMPLE              S,
                                          LPVOID               *lplpDS,
                                          LPVOID               *lplpDSB)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_get_DirectSound_info(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         S,lplpDS,lplpDSB);
      }

   IN_AIL;

#ifdef IS_WINDOWS
   if (AIL_preference[DIG_USE_WAVEOUT]==NO)
#endif
      {
      AIL_API_get_DirectSound_info(S,lplpDS,lplpDSB);
      }

   OUT_AIL;

   END;
}

#endif

#ifdef IS_WINDOWS

//############################################################################

DXDEF
S32      AILEXPORT  AIL_set_DirectSound_HWND(HDIGDRIVER dig, HWND wnd)
{
   S32 result=0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_DirectSound_HWND(" PTRFMT ", " PTRFMT ")" NL,dig,wnd);
      }

   IN_AIL;

   if (AIL_preference[DIG_USE_WAVEOUT]==NO)
      {
      result=AIL_API_set_DirectSound_HWND(dig,wnd);
      }

   OUT_AIL;

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}

#endif


//############################################################################

DXDEF HMDIDRIVER AILEXPORT AIL_open_XMIDI_driver( U32 flags )
{
   HMDIDRIVER result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_open_XMIDI_driver(0x%lX)" NL,
                                                       flags);
      }

   IN_AIL;

   #ifdef IS_WINDOWS

   if (AIL_midiOutOpen(&result,0,(flags&AIL_OPEN_XMIDI_NULL_DRIVER)?(U32)MIDI_NULL_DRIVER:(U32)MIDI_MAPPER))
     result=0;

   #else

   #ifdef IS_DOS

   if (flags&AIL_OPEN_XMIDI_NULL_DRIVER)
   {
     result=AIL_install_MDI_driver_file("NULL.MDI",0);
   }
   else
   {
     if (AIL_install_MDI_INI(&result)!=AIL_INIT_SUCCESS)
       result=0;
   }

   #else

   result = AIL_API_open_XMIDI_driver( flags );

   #endif

   #endif

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}


DXDEF void AILEXPORT AIL_close_XMIDI_driver( HMDIDRIVER mdi )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_close_XMIDI_driver(" PTRFMT ")" NL,mdi);
      }

   IN_AIL;

   #ifdef IS_WINDOWS
   dsailcall1(AIL,midiOutClose,void,mdi);
   #else
   #ifdef IS_DOS
   AIL_API_uninstall_MDI_driver(mdi)
   #else
   AIL_API_close_XMIDI_driver( mdi );
   #endif
   #endif

   OUT_AIL;

   END;
}


#ifdef IS_WINDOWS

DXDEF
S32          AILEXPORT AIL_midiOutOpen      (HMDIDRIVER FAR *drvr,   //)
                                             LPHMIDIOUT FAR *lphMidiOut,
                                             S32         dwDeviceID)

{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_midiOutOpen(" PTRFMT ", " PTRFMT ", %d)" NL,drvr,lphMidiOut,dwDeviceID);
      }

   IN_AIL;

   result=ailcall3(AIL,midiOutOpen,S32,drvr,lphMidiOut,(U32)dwDeviceID);

   OUT_AIL;

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_midiOutClose      (HMDIDRIVER mdi)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_midiOutClose(" PTRFMT ")" NL,mdi);
      }

   IN_AIL;

   ailcall1(AIL,midiOutClose,void,mdi);

   OUT_AIL;

   END;
}

#endif

//############################################################################

#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX)

DXDEF
S32         AILEXPORT AIL_MIDI_handle_release     (HMDIDRIVER mdi)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_MIDI_handle_release(" PTRFMT ")" NL,mdi);
      }

   IN_AIL;

   result=ailcall1(AIL,MIDI_handle_release,S32,mdi);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_MIDI_handle_reacquire     (HMDIDRIVER mdi)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_MIDI_handle_reacquire(" PTRFMT ")" NL,mdi);
      }

   IN_AIL;

   result=ailcall1(AIL,MIDI_handle_reacquire,S32,mdi);

   OUT_AIL;

   RESULT
      {
      INDENT;

      AIL_fprintf(AIL_debugfile,"Result = %lu" NL,result);
      }

   END;

   return result;
}

#endif

//############################################################################

DXDEF
HSEQUENCE    AILEXPORT AIL_allocate_sequence_handle//()
                                             (HMDIDRIVER mdi)
{
   HSEQUENCE result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_allocate_sequence_handle(" PTRFMT ")" NL,mdi);
      }

   IN_AIL;

   result=ailcall1(AIL,allocate_sequence_handle,HSEQUENCE,mdi);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_release_sequence_handle//()
                                             (HSEQUENCE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_release_sequence_handle(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   ailcall1(AIL,release_sequence_handle,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_init_sequence         (HSEQUENCE S, //)
                                              void const FAR *start,
                                              S32       sequence_num)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_init_sequence(" PTRFMT "," PTRFMT ",%ld)" NL,S,start,sequence_num);
      }

   IN_AIL;

   result=ailcall3(AIL,init_sequence,S32,S,start,sequence_num);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_start_sequence        (HSEQUENCE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_start_sequence(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   ailcall1(AIL,start_sequence,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_stop_sequence         (HSEQUENCE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stop_sequence(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   ailcall1(AIL,stop_sequence,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_resume_sequence       (HSEQUENCE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_resume_sequence(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   ailcall1(AIL,resume_sequence,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_end_sequence          (HSEQUENCE S)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_end_sequence(" PTRFMT ")" NL,S);
      }

   IN_AIL;

   ailcall1(AIL,end_sequence,void,S);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sequence_tempo    (HSEQUENCE S, //)
                                               S32       tempo,
                                               S32       milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sequence_tempo(" PTRFMT ",%ld,%ld)" NL,S,tempo,
         milliseconds);
      }

   IN_AIL;

   ailcall3(AIL,set_sequence_tempo,void,S,tempo,milliseconds);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sequence_volume   (HSEQUENCE S, //)
                                               S32       volume,
                                               S32       milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sequence_volume(" PTRFMT ",%ld,%ld)" NL,S,volume,
         milliseconds);
      }

   IN_AIL_NM;

   ailcall3(AIL,set_sequence_volume,void,S,volume,milliseconds);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void         AILEXPORT AIL_set_sequence_loop_count (HSEQUENCE S, //)
                                                 S32       loop_count)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sequence_loop_count(" PTRFMT ",%ld)" NL,S,loop_count);
      }

   IN_AIL_NM;

   ailcall2(AIL,set_sequence_loop_count,void,S,loop_count);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
U32        AILEXPORT AIL_sequence_status       (HSEQUENCE S)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sequence_status(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,sequence_status,U32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_sequence_tempo        (HSEQUENCE S)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sequence_tempo(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,sequence_tempo,S32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_sequence_volume       (HSEQUENCE S)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sequence_volume(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,sequence_volume,S32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32         AILEXPORT AIL_sequence_loop_count   (HSEQUENCE S)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sequence_loop_count(" PTRFMT ")" NL,S);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,sequence_loop_count,S32,S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_set_XMIDI_master_volume
                                             (HMDIDRIVER mdi,
                                              S32        master_volume)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_XMIDI_master_volume(" PTRFMT ",%d)" NL,mdi,master_volume);
      }

   IN_AIL;

   ailcall2(AIL,set_XMIDI_master_volume,void,mdi,master_volume);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_XMIDI_master_volume   (HMDIDRIVER mdi)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_XMIDI_master_volume(" PTRFMT ")" NL,mdi);
      }

   IN_AIL;

   result=ailcall1(AIL,XMIDI_master_volume,S32,mdi);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_active_sequence_count     (HMDIDRIVER mdi)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_active_sequence_count(" PTRFMT ")" NL,mdi);
      }

   IN_AIL;

   result=ailcall1(AIL,active_sequence_count,S32,mdi);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_controller_value          (HSEQUENCE S, //)
                                              S32       channel,
                                              S32       controller_num)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_controller_value(" PTRFMT ",%ld,%ld)" NL,S,channel,
         controller_num);
      }

   IN_AIL_NM;

   result=ailcall3(AIL,controller_value,S32,S,channel,controller_num);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_channel_notes             (HSEQUENCE S, //)
                                              S32       channel)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_channel_notes(" PTRFMT ",%ld)" NL,S,channel);
      }

   IN_AIL;

   result=ailcall2(AIL,channel_notes,S32,S,channel);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_sequence_position         (HSEQUENCE S, //)
                                                  S32       FAR *beat,
                                                  S32       FAR *measure)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sequence_position(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,beat,measure);
      }

   IN_AIL_NM;

   ailcall3(AIL,sequence_position,void,S,beat,measure);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_branch_index              (HSEQUENCE S, //)
                                               U32       marker)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_branch_index(" PTRFMT ",%lu)" NL,S,marker);
      }

   IN_AIL;

   ailcall2(AIL,branch_index,void,S,marker);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
AILPREFIXCB AILEXPORT AIL_register_prefix_callback  (HSEQUENCE S, //)
                                               AILPREFIXCB   callback)
{
   AILPREFIXCB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_prefix_callback(" PTRFMT "," PTRFMT ")" NL,
         S,callback);
      }

   IN_AIL;

   result=ailcall2(AIL,register_prefix_callback,AILPREFIXCB,S,callback);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILTRIGGERCB AILEXPORT AIL_register_trigger_callback (HSEQUENCE S, //)
                                              AILTRIGGERCB   callback)
{
   AILTRIGGERCB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_trigger_callback(" PTRFMT "," PTRFMT ")" NL,
         S,callback);
      }

   IN_AIL;

   result=ailcall2(AIL,register_trigger_callback,AILTRIGGERCB,S,callback);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILSEQUENCECB AILEXPORT AIL_register_sequence_callback(HSEQUENCE S, //)
                                              AILSEQUENCECB   callback)
{
   AILSEQUENCECB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_sequence_callback(" PTRFMT "," PTRFMT ")" NL,
         S,callback);
      }

   IN_AIL;

   result=ailcall2(AIL,register_sequence_callback,AILSEQUENCECB,S,callback);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILBEATCB AILEXPORT AIL_register_beat_callback(HSEQUENCE S, //)
                                              AILBEATCB   callback)
{
   AILBEATCB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_beat_callback(" PTRFMT "," PTRFMT ")" NL,
         S,callback);
      }

   IN_AIL;

   result=ailcall2(AIL,register_beat_callback,AILBEATCB,S,callback);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILEVENTCB AILEXPORT AIL_register_event_callback   (HMDIDRIVER mdi, //)
                                                    AILEVENTCB callback)
{
   AILEVENTCB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_event_callback(" PTRFMT "," PTRFMT ")" NL,
         mdi,callback);
      }

   IN_AIL;

   result=ailcall2(AIL,register_event_callback,AILEVENTCB,mdi,callback);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
AILTIMBRECB AILEXPORT AIL_register_timbre_callback  (HMDIDRIVER mdi, //)
                                                     AILTIMBRECB callback)
{
   AILTIMBRECB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_timbre_callback(" PTRFMT "," PTRFMT ")" NL,
         mdi, callback);
      }

   IN_AIL;

   result=ailcall2(AIL,register_timbre_callback,AILTIMBRECB,mdi,callback);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_set_sequence_user_data    (HSEQUENCE S, //)
                                                  U32       index,
                                                  SINTa     value)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sequence_user_data(" PTRFMT ",%ld," SINTFMT ")" NL,S,index,value);
      }

   IN_AIL_NM;

   ailcall3(AIL,set_sequence_user_data,void,S,index,value);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
SINTa   AILEXPORT AIL_sequence_user_data        (HSEQUENCE S, //)
                                                 U32       index)
{
   SINTa result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sequence_user_data(" PTRFMT ",%lu)" NL,S,index);
      }

   IN_AIL_NM;

   result=ailcall2(AIL,sequence_user_data,SINTa,S,index);

   OUT_AIL_NM;

   RESULT
      {
      outresuint((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_register_ICA_array        (HSEQUENCE   S,  //)
                                                  U8     FAR *array)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_ICA_array(" PTRFMT "," PTRFMT ")" NL,S,array);
      }

   IN_AIL;

   ailcall2(AIL,register_ICA_array,void,S,array);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_lock_channel              (HMDIDRIVER mdi)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_lock_channel(" PTRFMT ")" NL,mdi);
      }

   IN_AIL;

   result=ailcall1(AIL,lock_channel,S32,mdi);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_release_channel           (HMDIDRIVER mdi, //)
                                               S32        channel)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_release_channel(" PTRFMT ",%ld)" NL,mdi,channel);
      }

   IN_AIL;

   ailcall2(AIL,release_channel,void,mdi,channel);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_map_sequence_channel      (HSEQUENCE S, //)
                                               S32       seq_channel,
                                               S32       new_channel)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_map_sequence_channel(" PTRFMT ",%ld,%ld)" NL,
         S,seq_channel,new_channel);
      }

   IN_AIL;

   ailcall3(AIL,map_sequence_channel,void,S,seq_channel,new_channel);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
S32     AILEXPORT AIL_true_sequence_channel     (HSEQUENCE S, //)
                                              S32       seq_channel)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_true_sequence_channel(" PTRFMT ",%ld)" NL,
         S,seq_channel);
      }

   IN_AIL;

   result=ailcall2(AIL,true_sequence_channel,S32,S,seq_channel);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_send_channel_voice_message(HMDIDRIVER mdi, //)
                                               HSEQUENCE  S,
                                               S32        status,
                                               S32        data_1,
                                               S32        data_2)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_send_channel_voice_message(" PTRFMT "," PTRFMT ",0x%lX,0x%lX,0x%lX)" NL,
         mdi,S,status,data_1,data_2);
      }

   IN_AIL;

   ailcall5(AIL,send_channel_voice_message,void,mdi,S,status,data_1,data_2);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_send_sysex_message        (HMDIDRIVER mdi, //)
                                               void const FAR *buffer)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_send_sysex_message(" PTRFMT "," PTRFMT ")" NL,mdi,buffer);
      }

   IN_AIL;

   ailcall2(AIL,send_sysex_message,void,mdi,buffer);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
HWAVESYNTH    AILEXPORT AIL_create_wave_synthesizer   (HDIGDRIVER dig,  //)
                                                    HMDIDRIVER mdi,
                                                    void const FAR *wave_lib,
                                                    S32        polyphony)
{
   HWAVESYNTH result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_create_wave_synthesizer(" PTRFMT "," PTRFMT "," PTRFMT ",%ld)" NL,
         dig,mdi,wave_lib,polyphony);
      }

   IN_AIL;

   result=ailcall4(AIL,create_wave_synthesizer,HWAVESYNTH,dig,mdi,wave_lib,polyphony);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
void     AILEXPORT AIL_destroy_wave_synthesizer  (HWAVESYNTH W)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_destroy_wave_synthesizer(" PTRFMT ")" NL,W);
      }

   IN_AIL;

   ailcall1(AIL,destroy_wave_synthesizer,void,W);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
S32        AILEXPORT AIL_file_error  (void)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_error()" NL);
      }

   IN_AIL_NM;

   result=ailcall0(AIL,file_error,S32);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
S32        AILEXPORT AIL_file_size   (char const FAR   *filename)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_size(%s)" NL,filename);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,file_size,S32,filename);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void FAR * AILEXPORT AIL_file_read   (char const FAR   *filename,
                                          void FAR *dest)
{
   void FAR * result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_read(%s," PTRFMT ")" NL,filename,dest);
      }

   IN_AIL_NM;

   result=ailcall2(AIL,file_read,void FAR*,filename,dest);

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
S32        AILEXPORT AIL_file_write  (char const FAR   *filename,
                                          void const FAR *buf,
                                          U32       len)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_write(%s," PTRFMT ",%ld)" NL,filename,buf,len);
      }

   IN_AIL_NM;

   result=ailcall3(AIL,file_write,S32,filename,buf,len);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
S32        AILEXPORT AIL_WAV_file_write  (char const FAR   *filename,
                                          void const FAR *buf,
                                          U32       len,
                                          S32       rate,
                                          S32       format)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_WAV_file_write(%s," PTRFMT ",%ld,%ld,%ld)" NL,filename,buf,len,rate,format);
      }

   IN_AIL_NM;

   result=ailcall5(AIL,WAV_file_write,S32,filename,buf,len,rate,format);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


#ifdef ON_MAC_USE_FSS
//############################################################################

DXDEF
S32        AILEXPORT AIL_file_fss_size   (FSSpec const *spec)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_fss_size(" PTRFMT ")" NL,spec);
      }

   IN_AIL;

   result=ailcall1(AIL,file_fss_size,S32,spec);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void FAR * AILEXPORT AIL_file_fss_read   (FSSpec const *spec,
                                          void FAR *dest)
{
   void FAR * result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_fss_read(" PTRFMT "," PTRFMT ")" NL,spec,dest);
      }

   IN_AIL;

   result=ailcall2(AIL,file_fss_read,void FAR*,spec,dest);

   OUT_AIL;

   RESULT
      {
      outreshex((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
S32        AILEXPORT AIL_file_fss_write  (FSSpec const *spec,
                                          void const FAR *buf,
                                          U32       len)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_fss_write(" PTRFMT "," PTRFMT ",%ld)" NL,spec,buf,len);
      }

   IN_AIL;

   result=ailcall3(AIL,file_fss_write,S32,spec,buf,len);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}

//############################################################################

DXDEF
S32        AILEXPORT AIL_file_fss_attrib_write  (FSSpec const *spec,
                                          void FAR const *buf,
                                          U32       len,
                                          U32 creator,
                                          U32 type )
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_fss_attrib_write(" PTRFMT "," PTRFMT ",%ld,%ld,%ld)" NL,spec,buf,len,creator,type);
      }

   IN_AIL;

   result=ailcall5(AIL,file_fss_attrib_write,S32,spec,buf,len,creator,type);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
S32        AILEXPORT AIL_WAV_file_fss_write  (FSSpec const *spec,
                                          void const FAR *buf,
                                          U32       len,
                                          S32       rate,
                                          S32       format)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_WAV_file_fss_write(" PTRFMT "," PTRFMT ",%ld,%ld,%ld)" NL,spec,buf,len,rate,format);
      }

   IN_AIL;

   result=ailcall5(AIL,WAV_file_fss_write,S32,spec,buf,len,rate,format);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}
#endif

//############################################################################

#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX) || defined(IS_XBOX) || defined(IS_XENON) || defined(IS_PS2) || defined(IS_PS3) || defined(IS_WII)

DXDEF
void       AILEXPORT AIL_serve()
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_serve()" NL);
      }

   IN_AIL_NM;

   ailcall0(AIL,serve,void);

   OUT_AIL_NM;

   END;
}

#endif

#if !defined(IS_XBOX) && !defined(IS_XENON) && !defined(IS_PS2) && !defined(IS_PS3) && !defined(IS_WII)

//############################################################################

DXDEF
HREDBOOK AILEXPORT AIL_redbook_open(U32 which)
{
   HREDBOOK result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_open(%ld)" NL,which);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_open,HREDBOOK,which);

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
HREDBOOK AILEXPORT
#ifdef IS_MAC
AIL_redbook_open_volume(char const* drive)
#else
AIL_redbook_open_drive(S32 drive)
#endif
{
   HREDBOOK result;

   START
      {
      AIL_fprintf(AIL_debugfile,
#ifdef IS_MAC
      "AIL_redbook_open_volume(%s:)" NL
#else
      "AIL_redbook_open_drive(%c:)" NL
#endif
      ,drive);
      }

   IN_AIL_NM;

#ifdef IS_MAC
   result=ailcall1(AIL,redbook_open_volume,HREDBOOK,drive);
#else
   result=ailcall1(AIL,redbook_open_drive,HREDBOOK,drive);
#endif

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void AILEXPORT AIL_redbook_close(HREDBOOK hand)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_close(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   ailcall1(AIL,redbook_close,void,hand);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
void AILEXPORT AIL_redbook_eject(HREDBOOK hand)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_eject(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   ailcall1(AIL,redbook_eject,void,hand);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
void AILEXPORT AIL_redbook_retract(HREDBOOK hand)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_retract(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   ailcall1(AIL,redbook_retract,void,hand);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_status(HREDBOOK hand)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_status(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_status,U32,hand);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_tracks(HREDBOOK hand)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_tracks(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_tracks,U32,hand);

   OUT_AIL_NM;

   RESULT
      {
      INDENT;

      AIL_fprintf(AIL_debugfile,"Result = %ld" NL,result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_track(HREDBOOK hand)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_track(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_track,U32,hand);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void AILEXPORT AIL_redbook_track_info(HREDBOOK hand,U32 tracknum,U32 FAR* startmsec,U32 FAR* endmsec)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_track_info(" PTRFMT ", %ld, " PTRFMT ", " PTRFMT ")" NL,hand,tracknum,startmsec,endmsec);
      }

   IN_AIL_NM;

   ailcall4(AIL,redbook_track_info,void,hand,tracknum,startmsec,endmsec);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_id(HREDBOOK hand)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_id(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_id,U32,hand);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_position(HREDBOOK hand)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_position(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_position,U32,hand);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_play(HREDBOOK hand,U32 startmsec, U32 endmsec)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_play(" PTRFMT ", %ld, %ld)" NL,hand,startmsec,endmsec);
      }

   IN_AIL_NM;

   result=ailcall3(AIL,redbook_play,U32,hand,startmsec,endmsec);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_stop(HREDBOOK hand)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_stop(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_stop,U32,hand);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_pause(HREDBOOK hand)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_pause(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_pause,U32,hand);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
U32 AILEXPORT AIL_redbook_resume(HREDBOOK hand)
{
   U32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_resume(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_resume,U32,hand);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
F32 AILEXPORT AIL_redbook_volume_level(HREDBOOK hand)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_volume_level(" PTRFMT ")" NL,hand);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,redbook_volume_level,F32,hand);

   OUT_AIL_NM;

   RESULT
      {
      outresfloat(result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
F32 AILEXPORT AIL_redbook_set_volume_level(HREDBOOK hand, F32 volume)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_redbook_set_volume(" PTRFMT ",%s)" NL,hand,FTS(volume));
      }

   IN_AIL_NM;

   result=ailcall2(AIL,redbook_set_volume_level,F32,hand,volume);

   OUT_AIL_NM;

   RESULT
      {
      outresfloat(result);
      }
   END;

   return result;
}

#endif

//############################################################################

#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX) || defined(IS_XBOX) || defined(IS_XENON) || defined(IS_PS2) || defined(IS_PS3) || defined(IS_WII)

DXDEF
S32   AILEXPORT AIL_quick_startup             (S32         use_digital,
                                               S32         use_MIDI,
                                               U32         output_rate,
                                               S32         output_bits,
                                               S32         output_channels)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_startup(%ld, %ld, %ld, %ld, %ld )" NL,use_digital,use_MIDI,output_rate,output_bits,output_channels);
      }

   IN_AIL_NM;

   result=ailcall5(AIL,quick_startup,S32,use_digital,use_MIDI,output_rate,output_bits,output_channels);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}

#endif

//############################################################################

DXDEF
void   AILEXPORT AIL_quick_shutdown            (void)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_shutdown()" NL);
      }

   IN_AIL_NM;

   ailcall0(AIL,quick_shutdown,void);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
void   AILEXPORT AIL_quick_handles ( HDIGDRIVER FAR* pdig, HMDIDRIVER FAR* pmdi, HDLSDEVICE FAR* pdls )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_handles( " PTRFMT ", " PTRFMT " , " PTRFMT " )" NL,pdig,pmdi,pdls);
      }

   IN_AIL_NM;

   ailcall3(AIL,quick_handles,void,pdig,pmdi,pdls);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
HAUDIO  AILEXPORT AIL_quick_load                (char const   FAR *filename)
{
   HAUDIO result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_load(%s)" NL,filename);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,quick_load,HAUDIO,filename);

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


//############################################################################

#ifdef ON_MAC_USE_FSS

DXDEF
HAUDIO  AILEXPORT AIL_quick_fss_load            (FSSpec const   FAR *filename)
{
   HAUDIO result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_fss_load(" PTRFMT ")" NL,filename);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,quick_fss_load,HAUDIO,filename);

   OUT_AIL_NM;

   RESULT
      {
      outreshex((U32)result);
      }
   END;

   return result;
}

#endif

//############################################################################

DXDEF
HAUDIO  AILEXPORT AIL_quick_load_mem            (void const   FAR *mem,
                                                 U32    size)
{
   HAUDIO result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_load_mem(" PTRFMT ",%lu)" NL,mem,size);
      }

   IN_AIL_NM;

   result=ailcall2(AIL,quick_load_mem,HAUDIO,mem,size);

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
HAUDIO  AILEXPORT AIL_quick_load_named_mem      (void const   FAR *mem,
                                                 char const   FAR *filename,
                                                 U32    size)
{
   HAUDIO result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_load_mem(" PTRFMT ",%s%lu)" NL,mem,filename,size);
      }

   IN_AIL_NM;

   result=ailcall3(AIL,quick_load_named_mem,HAUDIO,mem,filename,size);

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
HAUDIO  AILEXPORT AIL_quick_copy                (HAUDIO audio)
{
   HAUDIO result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_copy(" PTRFMT ")" NL,audio);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,quick_copy,HAUDIO,audio);

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void    AILEXPORT AIL_quick_unload              (HAUDIO      audio)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_unload(" PTRFMT ")" NL,audio);
      }

   IN_AIL_NM;

   ailcall1(AIL,quick_unload,void,audio);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
S32    AILEXPORT AIL_quick_play                (HAUDIO      audio,
                                                U32         loop_count)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_play(" PTRFMT ", %ld)" NL,audio,loop_count);
      }

   IN_AIL_NM;

   result=ailcall2(AIL,quick_play,S32,audio,loop_count);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void   AILEXPORT AIL_quick_halt                (HAUDIO      audio)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_halt(" PTRFMT ")" NL,audio);
      }

   IN_AIL_NM;

   ailcall1(AIL,quick_halt,void,audio);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
S32     AILEXPORT AIL_quick_status              (HAUDIO      audio)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_status(" PTRFMT ")" NL,audio);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,quick_status,S32,audio);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
HAUDIO  AILEXPORT AIL_quick_load_and_play       (char const  FAR *filename,
                                                 U32         loop_count,
                                                 S32         wait_request)
{
   HAUDIO result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_load_and_play(%s, %ld, %ld)" NL,filename,loop_count,wait_request);
      }

   IN_AIL_NM;

   result=ailcall3(AIL,quick_load_and_play,HAUDIO,filename,loop_count,wait_request);

   OUT_AIL_NM;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}

//############################################################################

#ifdef ON_MAC_USE_FSS

DXDEF
HAUDIO  AILEXPORT AIL_quick_fss_load_and_play   (FSSpec const  FAR *filename,
                                                 U32         loop_count,
                                                 S32         wait_request)
{
   HAUDIO result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_fss_load_and_play(" PTRFMT ", %ld, %ld)" NL,filename,loop_count,wait_request);
      }

   IN_AIL_NM;

   result=ailcall3(AIL,quick_fss_load_and_play,HAUDIO,filename,loop_count,wait_request);

   OUT_AIL_NM;

   RESULT
      {
      outreshex((U32)result);
      }
   END;

   return result;
}

#endif

//############################################################################

DXDEF
void   AILEXPORT AIL_quick_set_speed     (HAUDIO      audio, S32 speed)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_set_speed(" PTRFMT ", %ld)" NL,audio,speed);
      }

   IN_AIL_NM;

   ailcall2(AIL,quick_set_speed,void,audio,speed);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
void   AILEXPORT AIL_quick_set_volume (HAUDIO audio, F32 volume, F32 extravol)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_set_volume(" PTRFMT ", %s, %s)" NL,audio,FTS(volume),FTS(extravol));
      }

   IN_AIL_NM;

   ailcall3(AIL,quick_set_volume,void,audio,volume,extravol);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
void   AILEXPORT AIL_quick_set_reverb_levels (HAUDIO audio,
                                              F32     dry_level,
                                              F32     wet_level)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_set_reverb_levels(" PTRFMT ", %s, %s)" NL,audio,FTS(dry_level),FTS(wet_level));
      }

   IN_AIL_NM;

   ailcall3(AIL,quick_set_reverb_levels,void,audio,dry_level,wet_level);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
void   AILEXPORT AIL_quick_set_low_pass_cut_off(HAUDIO audio,
                                                F32    cut_off)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_set_low_pass_cut_off(" PTRFMT ", %s)" NL,audio,FTS(cut_off));
      }

   IN_AIL_NM;

   ailcall2(AIL,quick_set_low_pass_cut_off,void,audio,cut_off);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
HSTREAM AILEXPORT AIL_open_stream(HDIGDRIVER dig, char const FAR* filename, S32 stream_mem)
{
   HSTREAM result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_open_stream(" PTRFMT ", %s, %ld)" NL,dig,filename,stream_mem);
      }

   IN_AIL;

   result=ailcall3(AIL,open_stream,HSTREAM,dig,filename,stream_mem);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void AILEXPORT AIL_close_stream(HSTREAM stream)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_close_stream(" PTRFMT ")" NL,stream);
      }

   IN_AIL;

   ailcall1(AIL,close_stream,void,stream);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
HSAMPLE AILEXPORT AIL_stream_sample_handle(HSTREAM stream)
{
   HSAMPLE result = NULL;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stream_sample_handle(" PTRFMT ")" NL,stream);
      }

   if (stream != NULL)
      result = stream->samp;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}

//############################################################################

DXDEF
S32 AILEXPORT AIL_service_stream(HSTREAM stream, S32 fillup)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_service_stream(" PTRFMT ", %ld)" NL,stream,fillup);
      }

   IN_AIL_NM;

   result=ailcall2(AIL,service_stream,S32,stream,fillup);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void AILEXPORT AIL_start_stream(HSTREAM stream)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_start_stream(" PTRFMT ")" NL,stream);
      }

   IN_AIL;

   ailcall1(AIL,start_stream,void,stream);

   OUT_AIL;

   END;
}


//############################################################################

DXDEF
void AILEXPORT AIL_pause_stream(HSTREAM stream, S32 onoff)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_pause_stream(" PTRFMT ", %ld)" NL,stream,onoff);
      }

   IN_AIL;

   ailcall2(AIL,pause_stream,void,stream,onoff);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
S32 AILEXPORT AIL_stream_loop_count(HSTREAM stream)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stream_loop_count(" PTRFMT ")" NL,stream);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,stream_loop_count,S32,stream);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void         AILEXPORT AIL_set_stream_loop_block (HSTREAM S,
                                                     S32     loop_start_offset,
                                                     S32     loop_end_offset)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_stream_loop_block(" PTRFMT ",%ld,%ld)" NL,S,loop_start_offset,loop_end_offset);
      }

   IN_AIL_NM;

   ailcall3(AIL,set_stream_loop_block,void,S,loop_start_offset,loop_end_offset);

   OUT_AIL_NM;

   END;
}



//############################################################################

DXDEF
void AILEXPORT AIL_set_stream_loop_count(HSTREAM stream, S32 count)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_stream_loop_count(" PTRFMT ", %ld)" NL,stream,count);
      }

   IN_AIL_NM;

   ailcall2(AIL,set_stream_loop_count,void,stream,count);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
S32 AILEXPORT AIL_stream_status(HSTREAM stream)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stream_status(" PTRFMT ")" NL,stream);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,stream_status,S32,stream);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void AILEXPORT AIL_set_stream_position(HSTREAM stream,S32 offset)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_stream_position(" PTRFMT ", %ld)" NL,stream,offset);
      }

   IN_AIL;

   ailcall2(AIL,set_stream_position,void,stream,offset);

   OUT_AIL;

   END;
}


//############################################################################

DXDEF
S32 AILEXPORT AIL_stream_position(HSTREAM stream)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stream_position(" PTRFMT ")" NL,stream);
      }

   IN_AIL;

   result=ailcall1(AIL,stream_position,S32,stream);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }
   END;

   return result;
}


//############################################################################

DXDEF
void AILEXPORT AIL_stream_info(HSTREAM stream, S32 FAR* datarate, S32 FAR* sndtype, S32 FAR* length, S32 FAR* memory)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stream_info(" PTRFMT ", " PTRFMT ", " PTRFMT ", " PTRFMT ", " PTRFMT ")" NL,stream,datarate,sndtype,length,memory);
      }

   IN_AIL_NM;

   ailcall5(AIL,stream_info,void,stream,datarate,sndtype,length,memory);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
AILSTREAMCB AILEXPORT AIL_register_stream_callback(HSTREAM stream, AILSTREAMCB callback)
{
   AILSTREAMCB result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_register_stream_callback(" PTRFMT ", " PTRFMT ")" NL,stream,callback);
      }

   IN_AIL;

   result=ailcall2(AIL,register_stream_callback,AILSTREAMCB,stream,callback);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}


#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX) || defined(IS_XBOX) || defined(IS_XENON) || defined(IS_PS2) || defined(IS_PS3) || defined(IS_WII)

//############################################################################

DXDEF
void AILEXPORT AIL_auto_service_stream(HSTREAM stream, S32 onoff)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_auto_service_stream(" PTRFMT ", %ld)" NL,stream,onoff);
      }

   IN_AIL;

   ailcall2(AIL,auto_service_stream,void,stream,onoff);

   OUT_AIL;

   END;
}


#endif

//############################################################################

DXDEF
void     AILEXPORT AIL_set_stream_user_data      (HSTREAM S, //)
                                                  U32     index,
                                                  SINTa   value)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_stream_user_data(" PTRFMT ",%ld," SINTFMT ")" NL,S,index,value);
      }

   IN_AIL_NM;

   ailcall3(AIL,set_stream_user_data,void,S,index,value);

   OUT_AIL_NM;

   END;
}


//############################################################################

DXDEF
SINTa    AILEXPORT AIL_stream_user_data          (HSTREAM S, //)
                                                  U32     index)
{
   SINTa result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stream_user_data(" PTRFMT ",%lu)" NL,S,index);
      }

   IN_AIL_NM;

   result=ailcall2(AIL,stream_user_data,SINTa,S,index);

   OUT_AIL_NM;

   RESULT
      {
      outresuint((UINTa)result);
      }

   END;

   return result;
}


//############################################################################

DXDEF
HDLSDEVICE AILEXPORT AIL_DLS_open(HMDIDRIVER mdi, HDIGDRIVER dig,
#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX)
                                  char const FAR* dls,
#else
                                  AILSTATICDLS const FAR* dls,
#endif
                                  U32 flags, U32 rate, S32 bits, S32 channels)
{
   HDLSDEVICE result;

   START
      {
#if defined(IS_WINDOWS) || defined(IS_MAC) || defined(IS_LINUX)
      AIL_fprintf(AIL_debugfile,"AIL_DLS_open(" PTRFMT "," PTRFMT ",%s,%lX,%lu,%lu,%lu)" NL,mdi,dig,dls,flags,rate,bits,channels);
#else
      AIL_fprintf(AIL_debugfile,"AIL_DLS_open(" PTRFMT "," PTRFMT "," PTRFMT ",%lX,%lu,%lu,%lu)" NL,mdi,dig,dls,flags,rate,bits,channels);
#endif
      }

   IN_AIL;

   result=ailcall7(AIL,DLS_open,HDLSDEVICE,mdi,dig,dls,flags,rate,bits,channels);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}


//############################################################################

DXDEF
void   AILEXPORT  AIL_DLS_close(HDLSDEVICE dls, U32 flags)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_DLS_close(" PTRFMT ",%ld)" NL,dls,flags);
      }

   IN_AIL;

   ailcall2(AIL,DLS_close,void,dls,flags);

   OUT_AIL;

   END;
}


//############################################################################

DXDEF
HDLSFILEID AILEXPORT AIL_DLS_load_file(HDLSDEVICE dls, char const FAR* filename, U32 flags)
{
   HDLSFILEID result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_DLS_load_file(" PTRFMT ",%s,%lu)" NL,dls,filename,flags);
      }

   IN_AIL;

   result=ailcall3(AIL,DLS_load_file,HDLSFILEID,dls,filename,flags);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}


//############################################################################

DXDEF
HDLSFILEID AILEXPORT AIL_DLS_load_memory(HDLSDEVICE dls, void const FAR* memfile, U32 flags)
{
   HDLSFILEID result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_DLS_load_memory(" PTRFMT "," PTRFMT ",%lu)" NL,dls,memfile,flags);
      }

   IN_AIL;

   result=ailcall3(AIL,DLS_load_memory,HDLSFILEID,dls,memfile,flags);

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}


//############################################################################

DXDEF
void   AILEXPORT AIL_DLS_unload(HDLSDEVICE dls, HDLSFILEID dlsid)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_DLS_unload(" PTRFMT "," PTRFMT ")" NL,dls,dlsid);
      }

   IN_AIL;

   ailcall2(AIL,DLS_unload,void,dls,dlsid);

   OUT_AIL;

   END;
}


//############################################################################

DXDEF
void   AILEXPORT AIL_DLS_compact(HDLSDEVICE dls)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_DLS_compact(" PTRFMT ")" NL,dls);
      }

   IN_AIL;

   ailcall1(AIL,DLS_compact,void,dls);

   OUT_AIL;

   END;
}


//############################################################################

DXDEF
HSAMPLE AILEXPORT AIL_DLS_sample_handle(HDLSDEVICE dls)
{
   HSAMPLE result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_DLS_sample_handle(" PTRFMT ")" NL,dls);
      }

  if (dls) {
    // are we using the soft-synthesizer
#if !defined(IS_MAC) && !defined(IS_LINUX)
#ifdef IS_WINDOWS
    if (dls->lib==0)
#else
    if(dls->DOSname==0)
#endif
#endif
    {
      result = dls->sample;
    }
  }

   RESULT
      {
      outresuhex((UINTa)result);
      }
   END;

   return result;
}

//############################################################################

DXDEF
void   AILEXPORT AIL_DLS_get_info(HDLSDEVICE dls, AILDLSINFO FAR* info, S32 FAR* PercentCPU)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_DLS_get_info(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,dls,info,PercentCPU);
      }

   IN_AIL_NM;

   ailcall3(AIL,DLS_get_info,void,dls,info,PercentCPU);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void AILEXPORT    AIL_set_sequence_ms_position  (HSEQUENCE S, //)
                                                 S32       milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sequence_ms_position(" PTRFMT ",%lu)" NL,S,milliseconds);
      }

   IN_AIL;

   ailcall2(AIL,set_sequence_ms_position,void,S,milliseconds);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void AILEXPORT AIL_sequence_ms_position(HSEQUENCE S, //)
                                  S32 FAR    *total_milliseconds,
                                  S32 FAR    *current_milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sequence_ms_position(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,total_milliseconds,current_milliseconds);
      }

   IN_AIL_NM;

   ailcall3(AIL,sequence_ms_position,void,S,total_milliseconds,current_milliseconds);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void AILEXPORT AIL_sample_ms_position(HSAMPLE S, //)
                                  S32 FAR    *total_milliseconds,
                                  S32 FAR    *current_milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_ms_position(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,total_milliseconds,current_milliseconds);
      }

   IN_AIL_NM;

   dsailcall3(AIL,sample_ms_position,void,S,total_milliseconds,current_milliseconds);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void AILEXPORT    AIL_set_sample_ms_position  (HSAMPLE S, //)
                                               S32       milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_ms_position(" PTRFMT ",%lu)" NL,S,milliseconds);
      }

   IN_AIL;

   dsailcall2(AIL,set_sample_ms_position,void,S,milliseconds);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void AILEXPORT AIL_stream_ms_position(HSTREAM S, //)
                                  S32 FAR    *total_milliseconds,
                                  S32 FAR    *current_milliseconds)
{
    START
      {
      AIL_fprintf(AIL_debugfile,"AIL_stream_ms_position(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,S,total_milliseconds,current_milliseconds);
      }

   IN_AIL_NM;

   ailcall3(AIL,stream_ms_position,void,S,total_milliseconds,current_milliseconds);

   OUT_AIL_NM;

   END;
}

//############################################################################

DXDEF
void AILEXPORT    AIL_set_stream_ms_position  (HSTREAM S, //)
                                               S32       milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_stream_ms_position(" PTRFMT ",%lu)" NL,S,milliseconds);
      }

   IN_AIL;

   ailcall2(AIL,set_stream_ms_position,void,S,milliseconds);

   OUT_AIL;

   END;
}

//############################################################################

DXDEF
void AILEXPORT AIL_quick_set_ms_position(HAUDIO audio,S32 milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_set_ms_position(" PTRFMT ",%lu)" NL,audio,milliseconds);
      }

   IN_AIL;

   ailcall2(AIL,quick_set_ms_position,void,audio,milliseconds);

   OUT_AIL;

   END;
}


//############################################################################

DXDEF
S32 AILEXPORT AIL_quick_ms_position(HAUDIO audio)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_ms_position(" PTRFMT ")" NL,audio);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,quick_ms_position,S32,audio);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################

DXDEF
S32 AILEXPORT AIL_quick_ms_length(HAUDIO audio)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_ms_length(" PTRFMT ")" NL,audio);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,quick_ms_length,S32,audio);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}


//############################################################################

DXDEF
S32 AILEXPORT AIL_quick_type(HAUDIO audio)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_quick_type(" PTRFMT ")" NL,audio);
      }

   IN_AIL_NM;

   result=ailcall1(AIL,quick_type,S32,audio);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

DXDEF
S32  AILEXPORT      AIL_MIDI_to_XMI       (void const FAR* MIDI,
                                           U32       MIDI_size,
                                           void FAR* FAR*XMIDI,
                                           U32  FAR* XMIDI_size,
                                           S32       flags)

{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_MIDI_to_XMI(" PTRFMT ",%lu," PTRFMT "," PTRFMT ",0x%lX)" NL,
         MIDI, MIDI_size,XMIDI,XMIDI_size,flags);
      }

   IN_AIL_NM;

   result=AIL_API_MIDI_to_XMI(MIDI, MIDI_size,XMIDI,XMIDI_size,flags);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

  return( result);

}


DXDEF
S32 AILEXPORT AIL_compress_ADPCM(AILSOUNDINFO const FAR* info,
                              void FAR* FAR* outdata, U32 FAR* outsize)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_compress_ADPCM(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         info,outdata,outsize);
      }

   IN_AIL_NM;

   result=AIL_API_compress_ADPCM(info,outdata,outsize);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

  return( result);

}

DXDEF
S32 AILEXPORT AIL_decompress_ADPCM(AILSOUNDINFO const FAR* info,
                                void FAR* FAR* outdata, U32 FAR* outsize)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_decompress_ADPCM(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         info,outdata,outsize);
      }

   IN_AIL_NM;

   result=AIL_API_decompress_ADPCM(info,outdata,outsize);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

  return( result);

}


DXDEF
S32 AILEXPORT AIL_WAV_info(void const FAR* data, AILSOUNDINFO FAR* info)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_WAV_info(" PTRFMT "," PTRFMT ")" NL,
         data,info);
      }

   IN_AIL_NM;

   result=AIL_API_WAV_info(data,info);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

  return( result);

}

DXDEF
S32 AILEXPORT AIL_file_type(void const FAR* data, U32 size)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_type(" PTRFMT ",%lu)" NL,
         data,size);
      }

   IN_AIL_NM;

   result=AIL_API_file_type(data,size);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

  return( result);

}

DXDEF
S32 AILEXPORT AIL_file_type_named(void const FAR* data, char const FAR * filename,U32 size)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_file_type(" PTRFMT ",%s,%lu)" NL,
         data,filename,size);
      }

   IN_AIL_NM;

   result=AIL_API_file_type_named(data,filename,size);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

  return( result);

}

DXDEF
S32 AILEXPORT AIL_find_DLS    (void const FAR* data, U32 size,
                               void FAR* FAR*xmi, U32 FAR* xmisize,
                               void FAR* FAR*dls, U32 FAR* dlssize)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_find_DLS_in_XMI(" PTRFMT ",%lu," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         data,size,xmi,xmisize,dls,dlssize);
      }

   IN_AIL_NM;

   result=AIL_API_find_DLS(data,size,xmi,xmisize,dls,dlssize);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

  return( result);

}

//############################################################################

DXDEF
S32 AILEXPORT AIL_extract_DLS      (void const FAR       *source_image, //)
                                     U32             source_size,
                                     void FAR * FAR *XMI_output_data,
                                     U32  FAR       *XMI_output_size,
                                     void FAR * FAR *DLS_output_data,
                                     U32  FAR       *DLS_output_size,
                                     AILLENGTHYCB    callback)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_DLS_extract_image(" PTRFMT ",0x%lu," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         source_image,source_size,XMI_output_data,XMI_output_size,DLS_output_data,DLS_output_size,callback);
      }

   IN_AIL_NM;

   result=AIL_API_extract_DLS(source_image,source_size,XMI_output_data,XMI_output_size,DLS_output_data,DLS_output_size,callback);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;

}

#if defined(IS_WIN32API) || defined(IS_MAC) || defined(IS_LINUX) || defined(IS_DOS) || defined(IS_PS2) || defined(IS_PS3) || defined(IS_WII)

extern HDIGDRIVER primary_digital_driver;   // from msswo.cpp

//############################################################################
DXDEF HDIGDRIVER AILEXPORT AIL_primary_digital_driver  (HDIGDRIVER new_primary)
{
   HDIGDRIVER result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_primary_digital_driver(" PTRFMT ")" NL,
         new_primary);
      }

   IN_AIL;

   result = primary_digital_driver;

   if (new_primary != NULL)
      {
      primary_digital_driver = new_primary;
      }

   OUT_AIL;

   RESULT
      {
      outresuhex((UINTa)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF
void       AILEXPORT AIL_set_sample_obstruction (HSAMPLE S, //)
                                                 F32     obstruction)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_obstruction(" PTRFMT ",%s)" NL,
          S, FTS(obstruction));
      }

   IN_AIL;

   AIL_API_set_sample_obstruction(S, obstruction);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF
void       AILEXPORT AIL_set_sample_exclusion (HSAMPLE S, //)
                                               F32     exclusion)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_exclusion(" PTRFMT ",%s)" NL,
          S, FTS(exclusion));
      }

   IN_AIL;

   AIL_API_set_sample_exclusion(S, exclusion);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF
void       AILEXPORT AIL_set_sample_occlusion   (HSAMPLE S, //)
                                                 F32     occlusion)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_occlusion(" PTRFMT ",%s)" NL,
          S, FTS(occlusion));
      }

   IN_AIL;

   AIL_API_set_sample_occlusion(S, occlusion);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF
void       AILEXPORT AIL_set_sample_3D_cone        (HSAMPLE S, //)
                                                    F32       inner_angle,
                                                    F32       outer_angle,
                                                    F32       outer_volume)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_3D_cone(" PTRFMT ",%s,%s,%s)" NL,
          S, FTS(inner_angle),FTS(outer_angle),FTS(outer_volume));
      }

   IN_AIL;

   AIL_API_set_sample_3D_cone(S, inner_angle,outer_angle,outer_volume);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF
F32        AILEXPORT AIL_sample_obstruction (HSAMPLE S)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_obstruction(" PTRFMT ")" NL,
          S);
      }

   IN_AIL_NM;

   result = AIL_API_sample_obstruction(S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)(S32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF
F32        AILEXPORT AIL_sample_exclusion (HSAMPLE S)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_exclusion(" PTRFMT ")" NL,
          S);
      }

   IN_AIL_NM;

   result = AIL_API_sample_exclusion(S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)(S32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF
F32        AILEXPORT AIL_sample_occlusion   (HSAMPLE S)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_occlusion(" PTRFMT ")" NL,
          S);
      }

   IN_AIL_NM;

   result = AIL_API_sample_occlusion(S);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)(S32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF
void       AILEXPORT AIL_sample_3D_cone        (HSAMPLE S,
                                                F32 FAR* inner_angle,
                                                F32 FAR* outer_angle,
                                                F32 FAR* outer_volume)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_3D_cone(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          S,inner_angle,outer_angle,outer_volume);
      }

   IN_AIL_NM;

   AIL_API_sample_3D_cone(S,inner_angle,outer_angle,outer_volume);

   OUT_AIL_NM;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_set_sample_3D_distances (HSAMPLE samp, //)
                                                      F32     max_dist,
                                                      F32     min_dist,
                                                      S32     auto_3D_wet_atten)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_3D_distances(" PTRFMT ",%s,%s,%ld)" NL,
          samp,FTS(max_dist),FTS(min_dist),auto_3D_wet_atten);
      }

   IN_AIL;

   AIL_API_set_sample_3D_distances(samp,max_dist,min_dist,auto_3D_wet_atten);

   OUT_AIL;

   END;
}


//############################################################################
DXDEF void AILEXPORT     AIL_sample_3D_distances     (HSAMPLE samp, //)
                                                      F32 FAR * max_dist,
                                                      F32 FAR * min_dist,
                                                      S32 FAR * auto_3D_wet_atten)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_3D_distances(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          samp,max_dist,min_dist,auto_3D_wet_atten);
      }

   IN_AIL_NM;

   AIL_API_sample_3D_distances(samp,max_dist,min_dist,auto_3D_wet_atten);

   OUT_AIL_NM;

   END;
}

//############################################################################
DXDEF  S32 AILEXPORT    AIL_room_type   (HDIGDRIVER dig)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_room_type(" PTRFMT ")" NL,dig);
      }

   IN_AIL_NM;

   result = AIL_API_room_type(dig);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF  void AILEXPORT   AIL_set_room_type   (HDIGDRIVER dig, //)
                                             S32       EAX_room_type)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_room_type(" PTRFMT ",%d)" NL,dig,EAX_room_type);
      }

   IN_AIL;

   AIL_API_set_room_type(dig, EAX_room_type);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  F32 AILEXPORT    AIL_3D_rolloff_factor  (HDIGDRIVER dig)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_3D_rolloff_factor(" PTRFMT ")" NL,dig);
      }

   IN_AIL_NM;

   result = AIL_API_3D_rolloff_factor(dig);

   OUT_AIL_NM;

   RESULT
      {
      outresfloat((F32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF  void AILEXPORT   AIL_set_3D_rolloff_factor (HDIGDRIVER dig, //)
                                                   F32       factor )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_3D_rolloff_factor(" PTRFMT ",%f)" NL,dig,FTS(factor));
      }

   IN_AIL;

   AIL_API_set_3D_rolloff_factor(dig, factor);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  F32 AILEXPORT    AIL_3D_doppler_factor  (HDIGDRIVER dig)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_3D_doppler_factor(" PTRFMT ")" NL,dig);
      }

   IN_AIL_NM;

   result = AIL_API_3D_doppler_factor(dig);

   OUT_AIL_NM;

   RESULT
      {
      outresfloat((F32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF  void AILEXPORT   AIL_set_3D_doppler_factor (HDIGDRIVER dig, //)
                                                   F32       factor )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_3D_doppler_factor(" PTRFMT ",%s)" NL,dig,FTS(factor));
      }

   IN_AIL;

   AIL_API_set_3D_doppler_factor(dig, factor);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF  F32 AILEXPORT    AIL_3D_distance_factor  (HDIGDRIVER dig)
{
   F32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_3D_distance_factor(" PTRFMT ")" NL,dig);
      }

   IN_AIL_NM;

   result = AIL_API_3D_distance_factor(dig);

   OUT_AIL_NM;

   RESULT
      {
      outresfloat((F32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF  void AILEXPORT   AIL_set_3D_distance_factor (HDIGDRIVER dig, //)
                                                    F32       factor )
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_3D_distance_factor(" PTRFMT ",%s)" NL,dig,FTS(factor));
      }

   IN_AIL;

   AIL_API_set_3D_distance_factor(dig, factor);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void  AILEXPORT    AIL_set_sample_3D_position         (HSAMPLE S, //)
                                                      F32     X,
                                                      F32     Y,
                                                      F32     Z)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_3D_position(" PTRFMT ",%s,%s,%s)" NL,
          S,FTS(X),FTS(Y),FTS(Z));
      }

   IN_AIL;

   AIL_API_set_sample_3D_position(S,X,Y,Z);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_set_sample_3D_velocity         (HSAMPLE S, //)
                                                      F32     dX_per_ms,
                                                      F32     dY_per_ms,
                                                      F32     dZ_per_ms,
                                                      F32     magnitude)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_3D_velocity(" PTRFMT ",%s,%s,%s,%s)" NL,
          S,FTS(dX_per_ms),FTS(dY_per_ms),FTS(dZ_per_ms),FTS(magnitude));
      }

   IN_AIL;

   AIL_API_set_sample_3D_velocity(S,dX_per_ms,dY_per_ms,dZ_per_ms,magnitude);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_set_sample_3D_velocity_vector  (HSAMPLE S, //)
                                                      F32     dX_per_ms,
                                                      F32     dY_per_ms,
                                                      F32     dZ_per_ms)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_3D_velocity_vector(" PTRFMT ",%s,%s,%s)" NL,
          S,FTS(dX_per_ms),FTS(dY_per_ms),FTS(dZ_per_ms));
      }

   IN_AIL;

   AIL_API_set_sample_3D_velocity_vector(S,dX_per_ms,dY_per_ms,dZ_per_ms);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_set_sample_3D_orientation      (HSAMPLE S, //)
                                                      F32     X_face,
                                                      F32     Y_face,
                                                      F32     Z_face,
                                                      F32     X_up,
                                                      F32     Y_up,
                                                      F32     Z_up)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_sample_3D_orientation(" PTRFMT ",%s,%s,%s,%s,%s,%s)" NL,
          S,FTS(X_face),FTS(Y_face),FTS(Z_face),FTS(X_up),FTS(Y_up),FTS(Z_up));
      }

   IN_AIL;

   AIL_API_set_sample_3D_orientation(S,X_face,Y_face,Z_face,X_up,Y_up,Z_up);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF S32 AILEXPORT     AIL_sample_3D_position       (HSAMPLE  S, //)
                                                      F32 FAR *X,
                                                      F32 FAR *Y,
                                                      F32 FAR *Z)
{
   S32 result;
   
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_3D_position(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          S,X,Y,Z);
      }

   IN_AIL_NM;

   result = AIL_API_sample_3D_position(S,X,Y,Z);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32) result);
      }

   END;

   return result;
}

//############################################################################
DXDEF void AILEXPORT     AIL_sample_3D_velocity             (HSAMPLE  S, //)
                                                      F32 FAR *dX_per_ms,
                                                      F32 FAR *dY_per_ms,
                                                      F32 FAR *dZ_per_ms)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_3D_velocity(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          S,dX_per_ms,dY_per_ms,dZ_per_ms);
      }

   IN_AIL_NM;

   AIL_API_sample_3D_velocity(S,dX_per_ms,dY_per_ms,dZ_per_ms);

   OUT_AIL_NM;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_sample_3D_orientation          (HSAMPLE  S, //)
                                                      F32 FAR *X_face,
                                                      F32 FAR *Y_face,
                                                      F32 FAR *Z_face,
                                                      F32 FAR *X_up,
                                                      F32 FAR *Y_up,
                                                      F32 FAR *Z_up)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_3D_orientation(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          S,X_face,Y_face,Z_face,X_up,Y_up,Z_up);
      }

   IN_AIL_NM;

   AIL_API_sample_3D_orientation(S,X_face,Y_face,Z_face,X_up,Y_up,Z_up);

   OUT_AIL_NM;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_update_sample_3D_position      (HSAMPLE S, //)
                                                             F32     dt_milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_update_sample_3D_position(" PTRFMT ",%s)" NL,
          S,FTS(dt_milliseconds));
      }

   IN_AIL;

   AIL_API_update_sample_3D_position(S, dt_milliseconds);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void  AILEXPORT    AIL_set_listener_3D_position         (HDIGDRIVER dig, //)
                                                      F32     X,
                                                      F32     Y,
                                                      F32     Z)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_listener_3D_position(" PTRFMT ",%s,%s,%s)" NL,
          dig,FTS(X),FTS(Y),FTS(Z));
      }

   IN_AIL;

   AIL_API_set_listener_3D_position(dig,X,Y,Z);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_set_listener_3D_velocity         (HDIGDRIVER dig, //)
                                                      F32     dX_per_ms,
                                                      F32     dY_per_ms,
                                                      F32     dZ_per_ms,
                                                      F32     magnitude)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_listener_3D_velocity(" PTRFMT ",%s,%s,%s,%s)" NL,
          dig,FTS(dX_per_ms),FTS(dY_per_ms),FTS(dZ_per_ms),FTS(magnitude));
      }

   IN_AIL;

   AIL_API_set_listener_3D_velocity(dig,dX_per_ms,dY_per_ms,dZ_per_ms,magnitude);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_set_listener_3D_velocity_vector  (HDIGDRIVER dig, //)
                                                      F32     dX_per_ms,
                                                      F32     dY_per_ms,
                                                      F32     dZ_per_ms)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_listener_3D_velocity_vector(" PTRFMT ",%s,%s,%s)" NL,
          dig,FTS(dX_per_ms),FTS(dY_per_ms),FTS(dZ_per_ms));
      }

   IN_AIL;

   AIL_API_set_listener_3D_velocity_vector(dig,dX_per_ms,dY_per_ms,dZ_per_ms);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_set_listener_3D_orientation      (HDIGDRIVER dig, //)
                                                      F32     X_face,
                                                      F32     Y_face,
                                                      F32     Z_face,
                                                      F32     X_up,
                                                      F32     Y_up,
                                                      F32     Z_up)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_listener_3D_orientation(" PTRFMT ",%s,%s,%s,%s,%s,%s)" NL,
          dig,FTS(X_face),FTS(Y_face),FTS(Z_face),FTS(X_up),FTS(Y_up),FTS(Z_up));
      }

   IN_AIL;

   AIL_API_set_listener_3D_orientation(dig,X_face,Y_face,Z_face,X_up,Y_up,Z_up);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_listener_3D_position             (HDIGDRIVER  dig, //)
                                                      F32 FAR *X,
                                                      F32 FAR *Y,
                                                      F32 FAR *Z)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_listener_3D_position(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          dig,X,Y,Z);
      }

   IN_AIL_NM;

   AIL_API_listener_3D_position(dig,X,Y,Z);

   OUT_AIL_NM;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_listener_3D_velocity             (HDIGDRIVER  dig, //)
                                                      F32 FAR *dX_per_ms,
                                                      F32 FAR *dY_per_ms,
                                                      F32 FAR *dZ_per_ms)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_listener_3D_velocity(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          dig,dX_per_ms,dY_per_ms,dZ_per_ms);
      }

   IN_AIL_NM;

   AIL_API_listener_3D_velocity(dig,dX_per_ms,dY_per_ms,dZ_per_ms);

   OUT_AIL_NM;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_listener_3D_orientation          (HDIGDRIVER  dig, //)
                                                      F32 FAR *X_face,
                                                      F32 FAR *Y_face,
                                                      F32 FAR *Z_face,
                                                      F32 FAR *X_up,
                                                      F32 FAR *Y_up,
                                                      F32 FAR *Z_up)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_listener_3D_orientation(" PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
          dig,X_face,Y_face,Z_face,X_up,Y_up,Z_up);
      }

   IN_AIL_NM;

   AIL_API_listener_3D_orientation(dig,X_face,Y_face,Z_face,X_up,Y_up,Z_up);

   OUT_AIL_NM;

   END;
}

//############################################################################
DXDEF void AILEXPORT     AIL_update_listener_3D_position      (HDIGDRIVER dig, //)
                                                      F32     dt_milliseconds)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_update_listener_3D_position(" PTRFMT ",%s)" NL,
          dig,FTS(dt_milliseconds));
      }

   IN_AIL;

   AIL_API_update_listener_3D_position(dig, dt_milliseconds);

   OUT_AIL;

   END;
}

#endif

#ifdef MSS_FLT_SUPPORTED

//############################################################################
DXDEF HPROVIDER AILEXPORT AIL_digital_output_filter (HDIGDRIVER dig)
{
   HPROVIDER result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_digital_output_filter(" PTRFMT ")" NL,
         dig);
      }

   IN_AIL;

#ifdef MSS_VFLT_SUPPORTED
   if (dig->voice_filter != NULL)
      {
      result = dig->voice_filter->provider;     // e.g., EAX
      }
   else
#endif 
   if (dig->matrix_filter != NULL)
      {
      result = dig->matrix_filter->provider;    // e.g., Dolby Surround
      }
   else
      {
      result = 0;
      }

   OUT_AIL;

   RESULT
      {
      outreshex((U32) result);
      }

   END;

   return result;
}

//############################################################################
DXDEF S32 AILEXPORT AIL_enumerate_filters (HPROENUM  FAR *next, //)
                                           HPROVIDER FAR *dest,
                                           C8  FAR * FAR *name)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_enumerate_filters(" PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         next,dest,name);
      }

   IN_AIL_NM;

   result = AIL_API_enumerate_filters(next,dest,name);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF HDRIVERSTATE AILEXPORT AIL_open_filter        (HPROVIDER  provider, //)
                                                     HDIGDRIVER dig)
{
   HDRIVERSTATE result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_open_filter(" HPFMT "," PTRFMT ")" NL,
         provider,dig);
      }

   IN_AIL;

   result = AIL_API_open_filter(provider,dig);

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF void  AILEXPORT    AIL_close_filter       (HDRIVERSTATE filter)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_close_filter(" PTRFMT ")" NL,
         filter);
      }

   IN_AIL;

   AIL_API_close_filter(filter);

   OUT_AIL;

   END;
}

//############################################################################
DXDEF S32 AILEXPORT AIL_enumerate_filter_properties(HPROVIDER                  lib,   //)
                                                    HINTENUM FAR *             next,
                                                    RIB_INTERFACE_ENTRY FAR *  dest)
{
   S32 result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_enumerate_filter_properties(" HPFMT "," PTRFMT "," PTRFMT ")" NL,
         lib,next,dest);
      }

   IN_AIL_NM;

   if (RIB_request_interface(lib,
                            "MSS pipeline filter",
                             0,
                             NULL) == RIB_NOERR)
      {
      result = RIB_enumerate_interface(lib,
                                      "MSS pipeline filter",
                                       RIB_PROPERTY,
                                       next,
                                       dest);
      }
   else if (RIB_request_interface(lib,                                 
                                 "MSS voice filter",    
                                  0,                                   
                                  NULL) == RIB_NOERR)
      {
      result = RIB_enumerate_interface(lib,
                                      "MSS voice filter",
                                       RIB_PROPERTY,
                                       next,
                                       dest);

      }

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF S32 AILEXPORT AIL_enumerate_filter_sample_properties(HPROVIDER                  lib,   //)
                                                           HINTENUM FAR *             next,
                                                           RIB_INTERFACE_ENTRY FAR *  dest)
{
   S32 result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_enumerate_filter_sample_properties(" HPFMT "," PTRFMT "," PTRFMT ")" NL,
         lib,next,dest);
      }

   IN_AIL_NM;

   if (RIB_request_interface(lib,
                            "Pipeline filter sample services",
                             0,
                             NULL) == RIB_NOERR)
      {
      result = RIB_enumerate_interface(lib,
                                      "Pipeline filter sample services",
                                       RIB_PROPERTY,
                                       next,
                                       dest);
      }
   else if (RIB_request_interface(lib,                                 
                                 "Voice filter sample services",    
                                  0,                                   
                                  NULL) == RIB_NOERR)
      {
      result = RIB_enumerate_interface(lib,
                                      "Voice filter sample services",
                                       RIB_PROPERTY,
                                       next,
                                       dest);

      }

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF S32 AILEXPORT AIL_enumerate_output_filter_driver_properties(HPROVIDER                  lib,   //)
                                                                  HINTENUM FAR *             next,
                                                                  RIB_INTERFACE_ENTRY FAR *  dest)
{
   S32 result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_enumerate_output_filter_driver_properties(" HPFMT "," PTRFMT "," PTRFMT ")" NL,
         lib,next,dest);
      }

   IN_AIL_NM;

   if (RIB_request_interface(lib,                                 
                            "Voice filter driver services",    
                             0,                                   
                             NULL) == RIB_NOERR)
      {
      result = RIB_enumerate_interface(lib,
                                      "Voice filter driver services",
                                       RIB_PROPERTY,
                                       next,
                                       dest);

      }
   else if (RIB_request_interface(lib,
                                 "Pipeline filter driver services",
                                  0,
                                  NULL) == RIB_NOERR)
      {
      result = RIB_enumerate_interface(lib,
                                      "Pipeline filter driver services",
                                       RIB_PROPERTY,
                                       next,
                                       dest);
      }

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF S32 AILEXPORT AIL_enumerate_output_filter_sample_properties(HPROVIDER                  lib,   //)
                                                                  HINTENUM FAR *             next,
                                                                  RIB_INTERFACE_ENTRY FAR *  dest)
{
   S32 result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_enumerate_output_filter_sample_properties(" HPFMT "," PTRFMT "," PTRFMT ")" NL,
         lib,next,dest);
      }

   IN_AIL_NM;

   if (RIB_request_interface(lib,                                 
                            "Voice filter sample services",    
                             0,                                   
                             NULL) == RIB_NOERR)
      {
      result = RIB_enumerate_interface(lib,
                                      "Voice filter sample services",
                                       RIB_PROPERTY,
                                       next,
                                       dest);

      }
   else if (RIB_request_interface(lib,                                 
                                 "Pipeline filter sample services",    
                                  0,                                   
                                  NULL) == RIB_NOERR)
      {
      result = RIB_enumerate_interface(lib,
                                      "Pipeline filter sample services",
                                       RIB_PROPERTY,
                                       next,
                                       dest);

      }

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF S32 AILEXPORT AIL_filter_property(HPROVIDER   lib,  //)
                                         C8 const FAR *    name,
                                         void FAR *       before_value,
                                         void const FAR * new_value,
                                         void FAR *       after_value
                                         )
{
   UINTa token;
   union
   {
     UINTa token;
     PROVIDER_PROPERTY property;
   } un = {0};
     
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_filter_property(" HPFMT ",%s," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         lib,name,before_value,new_value,after_value);
      }

   IN_AIL_NM;

   if (RIB_request_interface_entry(lib,
                                  "MSS pipeline filter",
                                   RIB_FUNCTION,
                                  "PROVIDER_property",
                      (UINTa FAR*)&un.token) == RIB_NOERR)
      {
      if (RIB_request_interface_entry(lib,
                                     "MSS pipeline filter",
                                      RIB_PROPERTY,
                                      name,
                                     &token) == RIB_NOERR)
         {
         return( un.property( token, before_value,new_value,after_value ) );
         }
      }
   else if (RIB_request_interface_entry(lib,
                                       "MSS voice filter",
                                        RIB_FUNCTION,
                                       "PROVIDER_property",
                           (UINTa FAR*)&un.token) == RIB_NOERR)
      {
      if (RIB_request_interface_entry(lib,
                                     "MSS voice filter",
                                      RIB_PROPERTY,
                                      name,
                                     &token) == RIB_NOERR)
         {
         return( un.property( token, before_value,new_value,after_value ) );
         }
      }

   OUT_AIL_NM;

   END;

   return 0;
}

//############################################################################
DXDEF S32 AILEXPORT AIL_enumerate_sample_stage_properties(HSAMPLE                    samp,  //)
                                                          SAMPLESTAGE                stage,
                                                          HINTENUM FAR *             next,
                                                          RIB_INTERFACE_ENTRY FAR *  dest)
{
   S32 result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_enumerate_sample_stage_properties(" PTRFMT ",%ld," PTRFMT "," PTRFMT ")" NL,
         samp,stage,next,dest);
      }

   IN_AIL_NM;

   if (samp != NULL)
      {
      if ((stage >= SP_FILTER_0) && (stage <= SP_FILTER_7))
         {
         if (samp->pipeline[stage].active)
            {
            result = RIB_enumerate_interface(samp->pipeline[stage].provider,
                                            "Pipeline filter sample services",
                                             RIB_PROPERTY,
                                             next,
                                             dest);
            }
         }
#ifdef MSS_VFLT_SUPPORTED
      else
         {
         //
         // Note that matrix postmix filters can't have any per-sample attributes...
         //
         
         if (samp->driver->voice_filter != NULL)
            {
            result = RIB_enumerate_interface(samp->driver->voice_filter->provider,
                                            "Voice filter sample services",
                                             RIB_PROPERTY,
                                             next,
                                             dest);
            }
         }
#endif
      }

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF S32 AILEXPORT AIL_sample_stage_property  (HSAMPLE        samp, //)
                                                SAMPLESTAGE    stage,  
                                                C8 const FAR * name,
                                                void FAR *       before_value,
                                                void const FAR * new_value,
                                                void FAR *       after_value
                                                )
{
   UINTa token;
   S32 result = 0;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_sample_stage_property(" PTRFMT ",%ld,%s," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         samp,stage,name,before_value,new_value,after_value);
      }

   IN_AIL;

   if (samp != NULL)
      {
      if ((stage >= SP_FILTER_0) && (stage <= SP_FILTER_7))
         {
         if (samp->pipeline[stage].active)
            {
            FLTPROVIDER FAR *provptr = samp->pipeline[stage].TYPE.FLT.provider;

            if (provptr != NULL)
               {
               FLTSMP_SAMPLE_PROPERTY sample_property = provptr->sample_property;

               if (sample_property != NULL)
                  {
                  if (RIB_request_interface_entry(samp->pipeline[stage].provider,
                                                 "Pipeline filter sample services",
                                                  RIB_PROPERTY,
                                                  name,
                                                 &token) == RIB_NOERR)
                     {
                     result = sample_property( samp->pipeline[stage].TYPE.FLT.sample_state,token, before_value,new_value,after_value );
                     }
                  }
               }
            }
         }
#ifdef MSS_VFLT_SUPPORTED
      else
         {
         //
         // Note that matrix postmix filters can't have any per-sample attributes...
         //

         if (samp->driver->voice_filter != NULL)
            {
            if (RIB_request_interface_entry(samp->driver->voice_filter->provider,
                                           "Voice filter sample services",
                                            RIB_PROPERTY,
                                            name,
                                           &token) == RIB_NOERR)
               {
                result = samp->driver->voice_filter->output_sample_property( samp,token, before_value,new_value,after_value );
               }
            }
         }
#endif
      }

   OUT_AIL;

   RESULT
      {
      outresint((U32)result);
      }

   END;
   
   return( result );
}

//############################################################################
DXDEF S32 AILEXPORT AIL_output_filter_driver_property (HDIGDRIVER     dig, //)
                                                       C8 const FAR * name,
                                                       void FAR *       before_value,
                                                       void const FAR * new_value,
                                                       void FAR *       after_value
                                                       )
{
   UINTa token;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_output_filter_driver_property(" PTRFMT ",%s," PTRFMT "," PTRFMT "," PTRFMT ")" NL,
         dig,name,before_value,new_value,after_value);
      }

   IN_AIL_NM;

   if (dig != NULL)
      {
#ifdef MSS_VFLT_SUPPORTED
      if (dig->voice_filter != NULL)
         {
         if (RIB_request_interface_entry(dig->voice_filter->provider,
                                        "Voice filter driver services",
                                         RIB_PROPERTY,
                                         name,
                                        &token) == RIB_NOERR)
            {
            return( dig->voice_filter->driver_property( dig->voice_filter->driver_state,token, before_value,new_value,after_value ) );
            }
         }
      else 
#endif
      if (dig->matrix_filter != NULL)
         {
         if (RIB_request_interface_entry(dig->matrix_filter->provider,
                                        "Pipeline filter driver services",
                                         RIB_PROPERTY,
                                         name,
                                        &token) == RIB_NOERR)
            {
            return( dig->matrix_filter->driver_property( dig->matrix_filter->driver_state,token, before_value,new_value,after_value ) );
            }
         }
      }

   OUT_AIL_NM;

   END;
   
   return 0;
}

#endif

//############################################################################
DXDEF
S32 AILEXPORT AIL_size_processed_digital_audio(
                                 U32             dest_rate,
                                 U32             dest_format,
                                 S32             num_srcs,
                                 AILMIXINFO const FAR* src)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_size_processed_digital_audio(%li,%li,%li," PTRFMT ")" NL,
          dest_rate,dest_format,num_srcs,src);
      }

   IN_AIL_NM;

   result = AIL_API_size_processed_digital_audio(dest_rate,dest_format,num_srcs,src);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//############################################################################
DXDEF
S32 AILEXPORT AIL_process_digital_audio(
                                 void FAR       *dest_buffer,
                                 S32             dest_buffer_size,
                                 U32             dest_rate,
                                 U32             dest_format,
                                 S32             num_srcs,
                                 AILMIXINFO FAR* src)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_process_digital_audio(" PTRFMT ",%li,%li,%li,%li," PTRFMT ")" NL,
          dest_buffer,dest_buffer_size,dest_rate,dest_format,num_srcs,src);
      }

   IN_AIL_NM;

   result = AIL_API_process_digital_audio(dest_buffer,dest_buffer_size,dest_rate,dest_format,num_srcs,src);

   OUT_AIL_NM;

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}


#ifdef IS_DOS

//#############################################################################
REALFAR AILEXPORT  AIL_get_real_vect             (U32       vectnum)
{
   REALFAR result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_get_real_vect(0x%X)" NL,vectnum);
      }

   result = AIL_API_get_real_vect(vectnum);

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}
//#############################################################################
void    AILEXPORT  AIL_set_real_vect             (U32       vectnum,
                                              REALFAR     real_ptr)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_real_vect(0x%X,0x%X)" NL,vectnum,real_ptr);
      }

   AIL_API_set_real_vect(vectnum,real_ptr);

   END;
}

//#############################################################################
void    AILEXPORT  AIL_set_USE16_ISR             (S32        IRQ,
                                              REALFAR     real_base,
                                              U32       ISR_offset)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_USE16_ISR(%d,0x%X,%u)" NL,IRQ,real_base,ISR_offset);
      }

   AIL_API_set_USE16_ISR(IRQ,real_base,ISR_offset);

   END;
}

//#############################################################################
void    AILEXPORT  AIL_restore_USE16_ISR         (S32        IRQ)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_restore_USE16_ISR(%d)" NL,IRQ);
      }

   AIL_API_restore_USE16_ISR(IRQ);

   END;
}

//#############################################################################
U32   AILEXPORT  AIL_disable_interrupts        (void)
{
   U32 result;

   result = AIL_API_disable_interrupts();

   return result;
}

//#############################################################################
void    AILEXPORT  AIL_restore_interrupts        (U32       FD_register)
{
   AIL_API_restore_interrupts(FD_register);
}

//#############################################################################
S32    AILEXPORT  AIL_call_driver               (AIL_DRIVER *drvr,
                                              S32        fn,
                                              VDI_CALL   *in,
                                              VDI_CALL   *out)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_call_driver(" PTRFMT ",0x%X," PTRFMT "," PTRFMT ")" NL,
         drvr,fn,in,out);
      }

   result = AIL_API_call_driver(drvr,fn,in,out);

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//#############################################################################
S32    AILEXPORT  AIL_read_INI                  (AIL_INI    *INI,
                                              char      *filename)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_API_read_INI(" PTRFMT ",%s)" NL,INI,filename);
      }

   result = AIL_API_read_INI(INI,filename);

   if (result)
      {
      RESULT
         {
         INDENT;
         AIL_fprintf(AIL_debugfile,"Driver = %s" NL,INI->driver_name);
         INDENT;
         AIL_fprintf(AIL_debugfile,"Device = %s" NL,INI->device_name);
         INDENT;
         AIL_fprintf(AIL_debugfile,"IO     = %X" NL,INI->IO.IO);
         INDENT;
         AIL_fprintf(AIL_debugfile,"IRQ    = %d" NL,INI->IO.IRQ);
         INDENT;
         AIL_fprintf(AIL_debugfile,"DMA_8  = %d" NL,INI->IO.DMA_8_bit);
         INDENT;
         AIL_fprintf(AIL_debugfile,"DMA_16 = %d" NL,INI->IO.DMA_16_bit);
         }
      }
   else
      {
      RESULT
         {
         outresint((U32)result);
         }
      }

   END;

   return result;
}

//#############################################################################
IO_PARMS *   AILEXPORT AIL_get_IO_environment    (AIL_DRIVER *drvr)
{
   IO_PARMS *result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_get_IO_environment(" PTRFMT ")" NL,drvr);
      }

   result = AIL_API_get_IO_environment(drvr);

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}

//#############################################################################
AIL_DRIVER * AILEXPORT AIL_install_driver        (U8 const     *driver_image,
                                              U32       n_bytes)
{
   AIL_DRIVER *result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_install_driver(" PTRFMT ",%u)" NL,driver_image,n_bytes);
      }

   result = AIL_API_install_driver(driver_image,n_bytes);

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}

//#############################################################################
void         AILEXPORT AIL_uninstall_driver      (AIL_DRIVER *drvr)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_uninstall_driver(" PTRFMT ")" NL,drvr);
      }

   AIL_API_uninstall_driver(drvr);

   END;
}

//#############################################################################
S32 AILEXPORT AIL_install_DIG_INI(HDIGDRIVER *dig)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_install_DIG_INI(" PTRFMT ")" NL,dig);
      }

   result = AIL_API_install_DIG_INI(dig);

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//#############################################################################
HDIGDRIVER  AILEXPORT AIL_install_DIG_driver_file
                                             (char const *filename,
                                              IO_PARMS *IO)
{
   HDIGDRIVER result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_install_DIG_driver_file(%s," PTRFMT ")" NL,filename,IO);
      }

   result = AIL_API_install_DIG_driver_file(filename,IO);

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}

//#############################################################################
void         AILEXPORT AIL_uninstall_DIG_driver  (HDIGDRIVER dig)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_uninstall_DIG_driver(" PTRFMT ")" NL,dig);
      }

   AIL_API_uninstall_DIG_driver(dig);

   END;
}

//#############################################################################
HDIGDRIVER  AILEXPORT AIL_install_DIG_driver_image
                                             (void const *driver_image,
                                              U32     size,
                                              IO_PARMS *IO)
{
   HDIGDRIVER result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_install_DIG_driver_image(" PTRFMT ",%u," PTRFMT ")" NL,
         driver_image,size,IO);
      }

   result = AIL_API_install_DIG_driver_image(driver_image,size,IO);

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}

//#############################################################################
S32 AILEXPORT AIL_install_MDI_INI(HMDIDRIVER *mdi)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_install_MDI_INI(" PTRFMT ")" NL,mdi);
      }

   result = AIL_API_install_MDI_INI(mdi);

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//#############################################################################
HMDIDRIVER  AILEXPORT AIL_install_MDI_driver_file
                                             (char const *filename,
                                              IO_PARMS   *IO)
{
   HMDIDRIVER result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_install_MDI_driver_file(%s," PTRFMT ")" NL,filename,IO);
      }

   result = AIL_API_install_MDI_driver_file(filename,IO);

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}

//#############################################################################
void         AILEXPORT AIL_uninstall_MDI_driver  (HMDIDRIVER  mdi)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_uninstall_MDI_driver(" PTRFMT ")" NL,mdi);
      }

   AIL_API_uninstall_MDI_driver(mdi);

   END;
}

//#############################################################################
HMDIDRIVER  AILEXPORT AIL_install_MDI_driver_image
                                             (void const *driver_image,
                                              U32       size,
                                              IO_PARMS   *IO)
{
   HMDIDRIVER result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_install_MDI_driver_image(" PTRFMT ",%u," PTRFMT ")" NL,
         driver_image,size,IO);
      }

   result = AIL_API_install_MDI_driver_image(driver_image,size,IO);

   RESULT
      {
      outreshex((U32)result);
      }

   END;

   return result;
}

//#############################################################################
S32     AILEXPORT AIL_MDI_driver_type       (HMDIDRIVER mdi)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_MDI_driver_type(" PTRFMT ")" NL,mdi);
      }

   result = AIL_API_MDI_driver_type(mdi);

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//#############################################################################
void     AILEXPORT AIL_set_GTL_filename_prefix   (char const       *prefix)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_set_GTL_filename_prefix(%s)" NL,prefix);
      }

   AIL_API_set_GTL_filename_prefix(prefix);

   END;
}

//#############################################################################
S32     AILEXPORT AIL_timbre_status             (HMDIDRIVER  mdi,
                                              S32        bank,
                                              S32        patch)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_timbre_status(" PTRFMT ",%d,%d)" NL,mdi,bank,patch);
      }

   result = AIL_API_timbre_status(mdi,bank,patch);

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//#############################################################################
S32     AILEXPORT AIL_install_timbre            (HMDIDRIVER  mdi,
                                              S32        bank,
                                              S32        patch)
{
   S32 result;

   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_install_timbre(" PTRFMT ",%d,%d)" NL,mdi,bank,patch);
      }

   result = AIL_API_install_timbre(mdi,bank,patch);

   RESULT
      {
      outresint((U32)result);
      }

   END;

   return result;
}

//#############################################################################
void     AILEXPORT AIL_protect_timbre            (HMDIDRIVER  mdi,
                                              S32        bank,
                                              S32        patch)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_protect_timbre(" PTRFMT ",%d,%d)" NL,mdi,bank,patch);
      }

   AIL_API_protect_timbre(mdi,bank,patch);

   END;
}

//#############################################################################
void  AILEXPORT AIL_unprotect_timbre          (HMDIDRIVER  mdi,
                                              S32        bank,
                                              S32        patch)
{
   START
      {
      AIL_fprintf(AIL_debugfile,"AIL_unprotect_timbre(" PTRFMT ",%d,%d)" NL,mdi,bank,patch);
      }

   AIL_API_unprotect_timbre(mdi,bank,patch);

   END;
}


//############################################################################
//##                                                                        ##
//## End of locked code                                                     ##
//##                                                                        ##
//############################################################################

void __pascal AILDEBUG_end(void)
{
   if (locked)
      {
      AIL_vmm_unlock_range(AILDEBUG_start, AILDEBUG_end);

      locked = 0;

      }
}

#endif
*/


























////////////////////////////////////////////////////////////////////////////
#pragma pack()
